//============================================================================
// FILE: DiscreteProcess.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe générique des processus discrets
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _DISCRETEPROCESS_HH
#define _DISCRETEPROCESS_HH

// SYSTEM INCLUDES
//
//#include <>                                    //

// PROJECT INCLUDES
//
#include "Process.h" 

          /**@name CDPFORTEXLa classe des processus ponctuels
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include DiscreteProcess.h
[/P]
\end{itemize}

Les sous-classes de processus ponctuels propres au système simulé sont déclarées dans le fichier typiquement nommmé [F]UserDiscreteProcess.h[/F] et réalisées dans le fichier [F]UserDiscreteProcess.cc[/F].\\

Pour chaque classe propre à son application, l'utilisateur doit essentiellement coder deux méthodes (en plus de l'éventuelle fonction de précondition, décrite dans la classe [C]Process[/C]) :
\begin{itemize}
\item Une fonction d'exécution (nommée par exemple et arbitrairement {\em MyProcess\_Exec}) qui code les changements que le processus provoque sur le système. Un processus ponctuel ne peut être contrôlé que par le type d'action (directive) EXEC\index{EXEC}. Le moteur de simulation, lorsqu'il rencontre dans un événement la directive EXEC associée à un processus ponctuel, invoque cette fonction d'exécution, par [P]ExecVoidMethod(EXEC\_PROCESS);[/P]. 
\item Le constructeur de la classe, dans lequel l'utilisateur doit appeler le constructeur de la classe de base avec le symbole (et éventuellement le nom de classe) qu'il associe à la classe. Par exemple :
\begin{verbatim}
[P]MyProcess::MyProcess() : DiscreteProcess(MY_SYMBOL) { ... };[/P]
\end{verbatim}

Dans ce constructeur ou en dehors, l'utilisateur doit impérativement doter le processus de la fonction d'exécution du processus évoquée ci-dessus. Par exemple :
\begin{verbatim}
[P]  SetExec(MyProcess_Exec);[/P]
\end{verbatim}

\end{itemize}

Il est important de noter que le processus ponctuel est détruit par le moteur après son exécution effective (voir l'algorithme de la méthode [C]Run()[/C] en début de description de la classe [C]Simulation[/C]). Cependant, cette destruction intervient à un moment tel que l'utilisateur peut faire référence au processus (par son pointeur) dans les fonctions de postconséquence et d'autogénération invoquées après l'exécution de la directive sur le processus.\label{seqConsPostAuto}\\

Lorsqu'un processus discret {\em p} dans un événement {\em e} est différé parce que sa précondition n'est pas satisfaite, il est d'abord copié (par [Y]RawCopy[/Y]), puis attaché, avec une directive EXEC, à un nouvel événement (copie de {\em e} par [Y]RawCopy[/Y]), puis enfin {\em p} est détruit par son destructeur superficiel (c'est-à-dire sans détruire les méthodes).
*/
//@{
        ///
class DiscreteProcess : public Process
{
public:
  /**@name LIFECYCLE */
//@{ 
          // Juste pour le besoin de la swiguisation sous Linux 2.4.25 (Carré)
  DiscreteProcess();
          // Constructeur de base sans méthode. Appel exclusif dans RawCopy avec l'argument 1. Les méthodes sont affectées dans RawCopy
   DiscreteProcess(bool); // _no swig_
          /// Un des deux constructeurs qui doivent être appelés par les classes définies par l'utilisateur. Affecte le type DISCRETE\_PROCESS et le symbole de classe donné en argument. Utiliser la méthode [Y]ClassName[/Y] pour affecter un nom de classe.
   DiscreteProcess(UserProcessId); // _no swig_                             
          /// Un des deux constructeurs qui doivent être appelés par les classes définies par l'utilisateur. Mêmes affectations que ci-dessus, plus le nom de classe donné en second argument.\label{affectNomDProcess}
   DiscreteProcess(UserProcessId, char*); // _no swig_            
          /// Constructeur par copie profonde, y compris des méthodes. Invoque le constructeur par copie de la classe-mère.
   DiscreteProcess(const DiscreteProcess&); // _no swig_   
                   
          /// Constructeur par copie superficielle, c'est-à-dire sans copie des méthodes. La nouvelle instance pointe sur la même méthode dont le corps code l'exécution du processus. \index{méthodes!RawCopy - DiscreteProcess} 
   DiscreteProcess* RawCopy();   
                         
          /// Destructeur superficiel. Ne détruit pas l'entité sur laquelle le processus opère des changements d'état, ni les instances de méthodes.
   ~DiscreteProcess(); // _no swig_                    
          /// Méthode de destruction. Ne détruit pas l'entité sur laquelle le processus opère des changements d'état. Détruit les instances de [C]Method[/C] attachées au processus (précondition et exécution).
   void DeleteInDepth(); // _no swig_
//@}    

/**@name OPERATORS */
//@{
         /// Opérateur d'affectation. Invoque les opérateurs d'affectation des méthodes. Renvoie une référence à l'entité d'origine.
   const DiscreteProcess&      operator=(const DiscreteProcess&); 
//@}

/**@name [SC]Sp\'ecification de l'effet[/SC]
\\

Le message : [P]<processus>->SetExec({\em f});[/P] affecte un corps {\small {\tt f}} à l'instance de [C]Method[/C] (précisément instance de [C]ExecProcess[/C]) dont le symbole de classe est EXEC\_PROCESS. Ce corps réalise l'effet du processus sur le système simulé. Une entité notable est celle renvoyée par l'invocation de [Y]ProcessedEntity[/Y] sur le processus. C'est typiquement, mais non obligatoirement, celle dont le processus change les variables d'état. Par exemple, {\small {\tt f}} peut être : 
\begin{verbatim}
[P]void MyProcessClass_Fct2(ProcessMethod* pM) { 
  DiscreteProcess* pP = pM->DescribedDiscreteProcess();
  Entity* pE = pP->ProcessedEntity();
  ...
}[/P]
\end{verbatim} 
*/
//@{ 
         /// Dote le processus d'un corps de méthode qui code l'exécution du processus (c'est-à-dire les changements de valeur des variables d'état qu'il provoque).\index{méthodes!SetExec}
   void            SetExec(void (* function)(ProcessMethod*)); // _no swig_
//@}   
 

/**@name [SC]Programmation de l'exécution[/SC] */
//@{
         /// Insère dans l'agenda un événement d'exécution du processus récepteur de ce message, à la date passée en argument. La priorité de l'événement est 0. Renvoie TRUE seulement si l'événément a pu être inséré. Désalloue l'événement instancié dans le cas contraire. Le code appelant doit se charger de l'éventuelle désallocation du processus.\index{méthodes!PostExecutionEvent}
   Boolean            PostExecutionEvent(int); // _no swig_  
         /// Même effet que la méthode précédente. La priorité de l'événement est passée en second argument.
   Boolean            PostExecutionEvent(int, int);
//@}                      

// ACCESS

         //
   void         IsReadSDRec(Boolean val);
         // 
   Boolean      IsReadSDRec() const;

         //
   void         IsInitSDRec(Boolean val);
         //
   Boolean      IsInitSDRec() const;

// INQUIRY     
    // comptage
    static int   getNbConst() {return _nbConst;}
    static int   getNbDest() {return _nbDest;}    

protected:

   int static   _nbConst;
   int static   _nbDest;

   Boolean         mIsReadSDRec;
   Boolean         mIsInitSDRec;

private:
};  // _end class_

//@}  // fin DP suite dans CoreDiscreteProcess.h

// INLINE METHODS

// EXTERNAL REFERENCES

#endif  // _DISCRETEPROCESS_HH
