//============================================================================
// FILE: Process.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe générique des processus
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _PROCESS_HH
#define _PROCESS_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "CoreTypes.h"

/**@name Les processus 

Un processus spécifie comment le système simulé change d'état, en réponse à un stimulus (ponctuel, durable ou permanent). Dans \D, le stimulus est représenté par un événement (classe [C]Event[/C]), et l'état du système par la structure des entités (classe [C]Entity[/C]) et leurs descripteurs (classe [C]Descriptor[/C]). L'unique façon de provoquer l'effet d'un processus est de le déclarer comme conséquence d'un événement et de provoquer l'occurrence de cet événement. \\

La classe de base [C]Process[/C] comporte deux sous-classes prédéfinies :
\begin{itemize} 
\item ... pour les processus dits ponctuels ([C]DiscreteProcess[/C]) : ceux dont le plein effet sur le système est opéré en une unique unité d'horloge. En d'autres termes, un processus ponctuel n'a pas de durée.
\item ... pour les processus dits continus ([C]ContinuousProcess[/C]) : ceux dont l'effet sur le système est opéré en un ou plusieurs pas, chaque pas pouvant lui-même correspondre à plusieurs unités d'horloge. 
\end{itemize} 

Une classe propre au domaine ne peut être une spécialisation que de [C]DiscreteProcess[/C] ou de [C]ContinuousProcess[/C]. En d'autres termes, elle ne peut hériter directement de la classe de base. De plus, aucune instance directe de cette classe de base ne doit être créée.
*/
//@{  // fermeture dans CoreContinuousProcess.h

/**@name CDPFORTEXLa classe de base des processus
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include Process.h
[/P]
\end{itemize}
Les instances de processus possèdent par construction des méthodes (instances de la classe [C]Method[/C]) dont les corps codent ... :
\begin{itemize} 
\item une fonction de précondition,
\item et des fonctions d'éxécution (processud ponctuels), et d'initialisation, d'avancée et d'arrêt (processus continus).
\end{itemize} 
Ces corps de méthodes sont vides à la construction du processus. On leur affecte ensuite une valeur (qui est un pointeur sur une fonction écrite par l'utilisateur), par des invocations de [Y]SetPrecondition[/Y], [Y]SetExec[/Y], [Y]SetInitialize[/Y], [Y]SetGoOneStepForward[/Y] ou [Y]SetStop[/Y].  
\noLF
Par une convention propre à \D, ces corps prennent comme seul argument la méthode elle-même : 
\begin{verbatim}
[P]void myProcess_execProcess_Body(ProcessMethod* pM);[/P]
\end{verbatim}

Comme on prend soin, avant son invocation, de doter la méthode d'un pointeur sur le processus qu'elle décrit, ce processus est accessible dans le corps par un message [Y]DescribedContinuousProcess()[/Y] ou [Y]DescribedDiscreteProcess()[/Y] adressé à la méthode (voir la section 'Attributs fonctionnels : les méthodes') :
\begin{verbatim}
[P]  DiscreteProcess* pDP = pM->DescribedDiscreteProcess();[/P]
\end{verbatim}

{\large {\tt L'entité attachée au processus}}

Un processus opère un changement d'état sur le système simulé par les fonctions d'éxécution, d'initialisation et d'avancée. C'est le code de ces fonctions qui détermine quelles entités du système sont touchées par le processus. Il faut cependant spécifier avant leur invocation ce qu'on va appeler "l'entité cible" du processus. Sa valeur est renvoyée par la méthode [Y]ProcessedEntity[/Y] :
\begin{verbatim}
[P]  Entity* pPE = pDP->ProcessedEntity();[/P]
\end{verbatim}

Cette entité est typiquement la seule que va toucher le processus, mais elle peut aussi n'être qu'une entité touchée parmi d'autres, ou bien encore n'être qu'un point de repère dans la structure du système à partir duquel le code des fonctions va rechercher la ou les entités touchées.  
\noLF
L'entité cible du processus peut être déterminée de deux manières :
\begin{itemize} 
\item en la désignant explicitement, comme argument de la méthode [Y]ProcessedBasicEntity[/Y] appliquée au processus :
\begin{verbatim}
[P]  Process* pDP = new MyDiscreteProcess();
  Entity* pE = new MyEntity();
  pDP->ProcessedEntity(pE);[/P]
\end{verbatim}

\item en la spécifiant de manière fonctionnelle : c'est alors une instance d'[C]EntitySpec[/C] (spécification d'ensemble d'entités) qui est passée en argument de la méthode [Y]ProcessedEntitySpec[/Y] appliquée au processus. La spécification doit avoir le mode RETURN\_INSTANCES, le quantificateur NFIRST et l'effectif {\em 1} :
\begin{verbatim}
[P]  Process* pDP = new MyDiscreteProcess();
  EntitySpec* pES = new MyEntitySpec();
  pES->SetMode(RETURN_INSTANCES);
  pES->IntQuantity(1);
  pDP->ProcessedEntitySpec(pES);[/P]
\end{verbatim}

L'expansion de la spécification (vois la section 'Spécifications d'ensembles d'entités') est réalisée dans la méthode d'accès [Y] ProcessedEntity[/Y] si et seulement si une entité cible n'a pas été désignée explicitement.
\end{itemize}
*/
//@{
        ///
class Process 
{
public:

  /**@name LIFECYCLE*/
//@{
          // Constructeur de base. N'est normalement appelé que par les constructeurs des sous-classes prédéfinies.
   Process();  // _no swig_     
          // Constructeur de base sans méthode. Appel exclusif dans RawCopy avec l'argument 1. Les méthodes sont affectées dans RawCopy
   Process(bool);  // _no swig_  
          /// Constructeur par copie. Le nouveau processus porte sur la même entité que le processus copié. Les méthodes sont copiées par les constructeurs par copie des sous-classes.
   Process(const Process&);  // _no swig_                        
          /// Destructeur. Ne détruit pas l'entité sur laquelle le processus opère des changements d'état. Ne détruit pas les méthodes du processus.
   ~Process();  // _no swig_  
//@}                                    

// OPERATORS

         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const Process&) const {return TRUE;};
  Boolean         operator > (const Process&) const {return TRUE;};

  /**@name [SC]Identification et accès[/SC]  */
//@{     
          /// Renvoie le type du processus (une des valeurs prédéfinies énumérées de [Y]ProcessSubclassId[/Y]).\index{méthodes!Subclass - Process}
   ProcessSubclassId Subclass() const;
          /// Renvoie le symbole de classe du processus (une des valeurs énumérées de [Y]ProcessId[/Y]). Cette énumération doit être faite par l'utilisateur dans le fichier [F]UserTypes.h[/F], sous le nom {\em ProcessId}.\index{méthodes!ClassSymbol - Process}
   UserProcessId  ClassSymbol() const; 
          /// Affecte un nom de classe du processus. Pour une exploitation dans le fichier de spécifications de sorties, le nom ne doit comporter que des caractères ASCII\index{nom de classe (processus)}.\index{méthodes!ClassName - Process}  
   void           ClassName(char*);  // _no swig_   
          /// Renvoie le nom de classe du processus.  
   char*          ClassName() const; 
          // Renvoie le numéro unique du processus.  
   int            ProcessInstanceNumber() const;
          // Renvoie le nom d'instance du processus.\index{méthodes!InstanceName - Process}
   char*          InstanceName() const;

         // Affecte le nom d'instance standard au récepteur du message
   void           SetInstanceName(char*, int);  // _no swig_  
         /// Affecte un nom d'instance au récepteur du message, qui surcharge le nom d'instance par défaut attribué par le moteur. Ce nom par défaut est la concaténation du nom de classe et d'un nombre entier positif unique sur l'ensemble de toutes les instances de processus.\index{nom d'instance (processus)}\index{méthodes!SetInstanceName - Process}
   void           SetInstanceName(char*);  // _no swig_   

         /// Renvoie la méthode dont le symbole de classe est passé en argument.\index{méthodes!GetMethod - Process}
   Method*        GetMethod(UserMethodId);  // _no swig_ 

         /// Renvoie la méthode dont le nom de classe est passé en argument.
   Method*        GetMethod(char*);  // _no swig_
         // Renvoie la table des méthodes.
   pMethodTab     MethodTab() const;  // _no swig_
         // 
   TableauInt     DualMethodTab() const;  // _no swig_
//@} 

  /**@name [SC]L'entité attachée au processus[/SC] 
\\
 
On rappelle ici (voir page 'La classe de base des processus') qu'un processus opère un changement d'état sur le système simulé, et qu'il faut spécifier ce qu'on va appeler "l'entité cible" du processus. Cette entité est typiquement la seule que va toucher le processus, mais elle peut aussi n'être qu'une entité touchée parmi d'autres, ou bien encore n'être qu'un point de repère dans la structure du système à partir duquel le code des fonctions va rechercher la ou les entités touchées.
*/
//@{    
          /// Etablit que l'entité sur laquelle le processus opère un changement d'état doit être de la classe dont le symbole est fourni en argument.\index{méthodes!ProcessedEntityClassId}
   void           ProcessedEntityClassId(UserClassId);  // _no swig_
          /// Renvoie de symbole de classe de l'entité sur laquelle le processus opère un changement d'état. 
   UserClassId    ProcessedEntityClassId() const;

          /// Etablit que la spécification d'entités fournie en argument définira l'entité sur laquelle le processus opèrera un changement d'état. Lève l'exception [Y]BadProcessedEntitySpec[/Y].
   void           ProcessedEntitySpec(EntitySpec*);  // _no swig_
          /// Renvoie la spécification d'entités qui définira l'entité sur laquelle le processus opèrera un changement d'état.
   EntitySpec*    ProcessedEntitySpec() const;  // _no swig_
         // Etablit que le processus possède une spécification pour l'entité opérée.
   void           HasProcessedEntitySpec(Boolean);  // _no swig_
         // Renvoie TRUE si le processus est doté d'une spécification pour l'entité sur laquelle doit porter l'effet du processus.
   Boolean        HasProcessedEntitySpec() const;  // _no swig_

          /// Etablit que l'entité fournie en argument est celle sur laquelle le processus opère un changement d'état. Lève l'exception [Y]BadProcessedEntityClass[/Y].
   void           ProcessedEntity(BasicEntity*);  // _no swig_
          /// Renvoie l'entité sur laquelle le processus opère un changement d'état. Lève l'exception [Y]DoNotHaveProcessedEntity[/Y].\index{méthodes!ProcessedEntity - Process}
   BasicEntity*   ProcessedBasicEntity();
          /// Renvoie le résultat de [Y]ProcessedBasicEntity[/Y] étendu à la définition de la classe [C]Entity[/C].
   Entity*        ProcessedEntity();

         // Etablit que le processus possède une entité opérée.
   void           HasProcessedEntity(Boolean);  // _no swig_
         // Renvoie TRUE si le processus est doté d'une entité sur laquelle doit porter l'effet du processus.
   Boolean        HasProcessedEntity() const;  // _no swig_

          /// Renvoie l'entité sur laquelle le processus opère un changement d'état, à partir de la spécification attachée au processus. Lève l'exception [Y]DoNotHaveProcessedEntity[/Y].\index{méthodes!ProcessedEntity - Process} 
   BasicEntity*   SpecifiedProcessedBasicEntity();
          /// Renvoie le résultat de [Y]SpecifiedProcessedBasicEntity[/Y] étendu à la définition de la classe [C]Entity[/C]. 
   Entity*        SpecifiedProcessedEntity();
//@} 

/**@name [SC]Sp\'ecification de la pr\'econdition[/SC]
Le message [P]<processus>}->SetPrecondition({\em f});[/P] affecte un corps {\small {\tt  f}} à l'instance de [C]Method[/C] dont le symbole de classe est PROCESS\_PRECONDITION. Ce corps doit retourner un entier positif, nul ou négatif selon que l'action sur le processus peut être réalisée, doit être retardée ou doit être annulée, res\-pectivement. Par exemple, {\small {\tt f}} peut être : \\ 
\begin{verbatim}
[P]int MyProcessClass_Fct1(ProcessMethod* pM) {
  Process* pP = pM->DescribedDiscreteProcess();
  ...
  return -1;
}[/P]
\end{verbatim}

Pour un processus continu la précondition ne joue que sur l'initialisation, c'est-à-dire que la précondition n'est plus testée au cours de vie du processus. Si la fonction renvoie un entier positif (ou si elle est absente), le processus est initialisé. Si elle renvoie 0, un nouvel événement d'initialisation est programmé pour un instant simulé futur (l'horloge courante augmentée de la valeur renvoyée par la méthode [Y]PostponementDelay[/Y]). Si la fonction {\em f} renvoie un entier négatif, un événement d'arrêt du processus est programmé pour l'instant courant. \\

Pour un processus ponctuel, si la fonction {\em f} de précondition renvoie un entier positif (ou si elle est absente), le processus est exécuté. Si elle renvoie 0, un nouvel événement est créé par [Y]RawCopy()[/Y] (c'est-à-dire sans duplication des méthodes). Il est alors programmé pour un instant simulé futur (l'horloge courante augmentée de la valeur renvoyée par la méthode [Y]PostponementDelay[/Y]), avec comme seule action l'exécution de ce même processus. Si la fonction  renvoie un entier négatif, le processus n'est pas exécuté et aucune nouvelle exécution n'est programmée (abandon).\index{méthodes!SetPrecondition} \\

Dans les deux cas, si une méthode d'autogénération est affectée à l'événement concerné, elle ne sera invoquée que lorsque la précondition du processus deviendra satisfaite, c'est-à-dire lorsque les conséquences de l'événement auront effectivement pu être mises en {\oe}uvre. 
*/
  //@{ 
          /// Affecte un corps de méthode qui teste la précondition\index{précondition} du processus. Ce corps est le pointeur NULL à la construction de l'instance de processus. \label{defSetPreCond}
   void            SetPrecondition (int (*f)(ProcessMethod*));  // _no swig_     

         // Etablit que le processus est doté d'une méthode de précondition avec un corps non neutre.
   void           HasPrecondition(Boolean);  // _no swig_
         // Renvoie TRUE si le processus est doté d'un prédicat de précondition.
   Boolean        HasPrecondition() const;  // _no swig_

  //@}   

/**@name [SC]Sp\'ecification du délai de reprogrammation[/SC]
On peut spécifier ce délai directement avec [Y]PostponementDelay[/Y], ou bien en utilisant la méthode prédéfinie dont le symbole de classe est PROCESS\_POSTPONEMENT\_DELAY. Si un corps a été donné à la méthode, le moteur de simulation invoquera la méthode. La valeur renvoyée sera passée par le moteur en argument de [Y]PostponementDelay[/Y].\\
Le message  [/]<processus>}->SetPostponementDelay({\em f});[/P] affecte un corps {\em f} à cette instance de [C]Method[/C] dont le symbole de classe est PROCESS\_POSTPONEMENT\_DELAY. Ce corps doit retourner un entier positif. Par exemple, {\em f} peut être : \\ 
\begin{verbatim}
[P]int MyProcessClass_Fct2(ProcessMethod* pM) {
  Process* pP = pM->DescribedDiscreteProcess();
  ...
  return GetIntParameterValue("monProcess" "delai");
}[/P]
\end{verbatim}
*/
  //@{ 
          /// Affecte un corps de méthode qui renvoie le nombre d'unités de temps simulé après lequel doit être programmé l'événement de "ré-examen" du processus quand il a été retardé (du fait de la non satisfaction de sa précondition). Ce corps est le pointeur NULL à la construction de l'instance de processus. \label{defSetPreCond}
   void            SetPostponementDelay (int (*f)(ProcessMethod*));  // _no swig_ 
         /// Détermine le délai (en nombre d'unités de temps simulé) après lequel doit être programmé l'événement de "ré-examen" d'un processus retardé (du fait de la non satisfaction de sa précondition). Il est établi par défaut à 1 dans le constructeur de la classe.\index{méthodes!PostponementDelay}
void                PostponementDelay(int);  // _no swig_ 
         /// Renvoie le délai après lequel doit être programmé l'événement de "ré-examen" d'un processus retardé (voir l'algorithme de la méthode [Y]Run[/Y]() en début de description de la classe [C]Simulation[/C]).
   int            PostponementDelay() const;
         // Etablit que le processus est doté d'une méthode de délai de ré-examen avec un corps non neutre.
   void           HasPostponementDelay(Boolean);  // _no swig_
         // Renvoie TRUE si le processus est doté d'une méthode de délai de ré-examen.
   Boolean        HasPostponementDelay() const;  // _no swig_
  //@}
 
         // La méthode [Y]ExecIntCoreMethod[/Y] exécute le corps d'une méthode prédéfinie de type {\em int} (précondition).
   int             ExecIntCoreMethod(UserMethodId);  
         // La méthode [Y]ExecVoidCoreMethod[/Y] exécute le corps d'unem éthode prédéfinie de type {\em void}.
   void            ExecVoidCoreMethod(UserMethodId);
  
         // 
   void             AddInstance(UserProcessId id, char*);  // _no swig_  
         // 
   void             RemoveInstance(Process*);  // _no swig_

   /**@name [SC]Affectation de m\'ethodes sp\'ecifiques[/SC]
Outre les méthodes prédéfinies liées à la précondition du processus et, en ce qui concerne les sous-classes, à son exécution, il est possible de décrire un processus par des méthodes spécifiques, en fonction des besoins de l'application. Il faut alors construire l'instance de [C]Method[/C] et l'ajouter au tableau des méthodes par une invocation de [Y]AddMethod[/Y].
 */
//@{
         /// Ajoute la méthode à la liste des méthodes du processus, seulement si c'est une méthode de processus.\index{méthodes!AddMethod - Process}
   void                AddMethod(Method*);  // _no swig_
//@}

/**@name [SC]Affectation d'un corps aux m\'ethodes, et de valeurs aux arguments[/SC] 
L'appel [Y]AssignBody[/Y]\index{méthodes!AssignBody - Process} affecte un corps à l'instance de [C]Method[/C] dont le symbole de classe est donné en premier argument.\\

Les méthodes [Y]ExecVoidMethod[/Y] et [Y]ExecIntMethod[/Y] exécutent le corps de la méthode dont le symbole est passé en argument. La présente version n'offre que ces deux possibilités de type de méthode ({\em void} et {\em int}), et cette unique possibilité de type d'argument ({\em int}).\index{méthodes!ExecVoidMethod - Process}\index{méthodes!ExecPredicate - Process}\label{defExecVoidMethodProcess}
*/ 
//@{   
         /// 
   void    AssignBody(UserMethodId, void (*f)(ProcessMethod*));  // _no swig_  
         /// 
   void    AssignBody(UserMethodId, int (*f)(ProcessMethod*));  // _no swig_
         /// 
   void    SetIntArgValue(UserMethodId, UserArgumentId, int);        
         /// 
   void    SetEntityArgValue(UserMethodId, UserArgumentId, BasicEntity*);    
         /// 
   void    ExecVoidMethod(UserMethodId);   
         /// 
   int     ExecIntMethod(UserMethodId);
          
//@}

         // Appelé avec TRUE si le processus doit être supprimé avec l'événement correspondant dans RunCore
   void           ToBeDeleted(Boolean);  // _no swig_
         // Détermine si le processus doit être supprimé avec l'événement correspondant dans RunCore
   Boolean        ToBeDeleted() const;  // _no swig_


// début code gestion OutputSpec  
          // Détermine si l'occurrence du processus doit être enregistrée dans une instance d'une sous-classe de [C]ProcessOutputSpec[/C].
   void           IsRecorded(Boolean);  // _no swig_  
          // Renvoie TRUE si l'occurrence du processus doit être enregistrée dans une instance d'une sous-classe de [C]ProcessOutputSpec[/C].\index{méthodes!Process - IsRecorded}
   Boolean        IsRecorded() const;  // _no swig_  
          // Renvoie l'instance de [C]OutputSpec[/C] dans laquelle l'occurrence du processus est enregistrée.\index{méthodes!Process - GetOutputSpec}
   ProcessOutputSpec*  GetOutputSpec() const;  // _no swig_
// fin code gestion OutputSpec

   //static int          InstanceNumber(); 
   //static pProcessTab  InstanceTab();
   static Process*     GetInstance(int); // évite de faite une copie de tableau

   static pProcessTab  mpInstanceTab;         // + JPR 980202
   static int          mInstanceNumber;         // + JPR 980202

protected:

   ProcessSubclassId  mThisProcessSubclass;
   UserProcessId   mThisProcessClassId;
   char*           mThisProcessClassName;
   int             mThisProcessNumber;
   char*           mThisProcessInstanceName;

   BasicEntity*    mProcessedEntity;
   UserClassId     mProcessedEntityClassId;
   Boolean         mHasProcessedEntity;
   EntitySpec*     mProcessedEntitySpec;
   Boolean         mHasProcessedEntitySpec;

   Boolean         mHasPrecondition;
   Boolean         mHasPostponementDelay;
   int             mPostponementDelay;
   Boolean         mToBeDeleted;

   void            Subclass(ProcessSubclassId);
   void            ClassSymbol(UserProcessId);

// début code gestion OutputSpec  
   Boolean            mIsRecorded;
   ProcessOutputSpec* mpOutputSpec;
// fin code gestion OutputSpec  

   pMethodTab             mpMethodTab;    // + JPR 0204
   TableauInt             mpDualMethodTab;

private:
};  // _end class_
 
//@} // fin classe de base  ... suite dans DiscreteProcess.h

// INLINE METHODS


// EXTERNAL REFERENCES

#endif  // _PROCESS_HH
