//============================================================================
// FILE: C_CoreFunction.h
//
// TYPE: C-LIBRARY
// DESCRIPTION: Miscellaneous C functions for Control-Diese
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_COREFUNCTIONS_H
#define _C_COREFUNCTIONS_H

// SYSTEM INCLUDES
//
#include <stdarg.h>
#include <stdlib.h>

// PROJECT INCLUDES
//
#include "C_CoreTypes.h"
#include "C_Operation.h"
#include "C_ResourceSet.h" // pour AllocatedOperationSet to scratch
#include "C_ResourceAllocation.h" // + JPR 100226

/**@name Fonctions globales et générales 
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include C_CoreFunction.h
[/P]
\end{itemize} 
On présente ici les fonctions, au sens du langage {\em C}, c'est-à-dire les procédures (avec effets de bord) et les fonctions au sens strict (retournant une valeur) qui ne sont pas attachées à des classes d'objets sous le vocable et avec le rôle de "méthodes".\\

Elles sont globales parce qu'on peut les invoquer dans l'ensemble du code, celui de \CDb et celui spécifique au domaine d'application.
*/ 
//@{ 


/**@name Interprétation des arguments de la commande de simulation

La commande d'exécution du simulateur ([P]./main ...[/P]) possède des arguments, pour spécifier des données en entrée et préciser des options d'exécution. \\

\Db prédéfinit quelques arguments standards et propose, par la fonction [Y]ParseMainArguments[/Y], un moyen d'interpréter leurs valeurs dans la ligne de commande. Par rapport à un développement sous \BD, la commande d'exécution d'un simulateur développé sous \CDb, comprend les même arguments obligatoires et peut comprendre les mêmes arguments optionnels (voir à ce sujet la documentation de \BD, page 'Fonctions globales / Interprétation des arguments de la commande'). \\

Au-delà, la ligne de commande sous \CDb peut contenir des arguments nouveaux, que [Y]ParseMainArguments[/Y] ne sait interpréter. \CDb fournit donc la fonction [Y]C_ParseMainArguments[/Y] qui invoque [Y]ParseMainArguments[/Y] avant de rechercher dans la ligne de commande la présence éventuelle des arguments propres à \CD. \\

Au-delà encore, et comme dans un développement avec \BD, l'application propose généralement des options d'exécution particulière au domaine étudié. La mise en jeu de ces options est opérée par une fonction, qu'on peut appeler  [Y]App_ParseMainArgument[/Y], dont le principe de codage est exposé dans la documentation de \BD. Cette fonction doit invoquer [Y]C_ParseMainArguments[/Y] en son début :\\

[P]
\begin{verbatim}
int main (int argc, char* argv[]) {
  App_ParseMainArguments(argc, argv);
  ...
};

void App_ParseMainArguments (int argc, char* argv[]) {
//---------------------------------------------
  Boolean stdArgs_ok = C_ParseMainArguments(argc, argv);

  if(stdArgs_ok)  {
	for (int i=6;i<argc;i++) {
	  if (!strcmp(argv[i], "-w"))
		gMyTraceWarnings  = TRUE;
	  if (!strcmp(argv[i], "-o"))
		gMyTraceOperations  = TRUE;
	}
  }
}
\end{verbatim}
[/P]

{\small Rôle de C_ParseMainArguments}\\

Dans sa version courante, [Y]C_ParseMainArguments[/Y] teste la présence dans la ligne de commande des arguments suivants, et donne le cas échéant la valeur mentionnée aux variables globales correspondantes :
{\small
\begin{verbatim}
    -RscWarn          gTraceResourceWarnings  TRACE_RSC_WARN_ON
    -ActWarn          gTraceActivityWarnings  TRACE_ACT_WARN_ON
    -USO              gUpperSubsetOption      setSize | maxSpecAllocSize
    -ActSet n         gActSetSizeUpperBound   n
    -LogUPS lb[-ub]   g_CD_logUPS_begClock    lb
                      g_CD_logUPS_endClock    ub
    -ActLog           gTraceActivityLogs      TRUE
    -LogMIL lb[-ub]   g_CD_logMIL_begClock    lb
                      g_CD_logMIL_endClock    ub
    -LogMIL+ lb[-ub]  g_CD_logMILp_begClock   lb
                      g_CD_logMILp_endClock   ub
    -LogAIL lb[-ub]   g_CD_logAIL_begClock    lb
                      g_CD_logAIL_endClock    ub
    -LogESA lb[-ub]   g_CD_logESA_begClock    lb
                      g_CD_logESA_endClock    ub
    -LogESA+ lb[-ub]  g_CD_logESAp_begClock   lb
                      g_CD_logESAp_endClock   ub
\end{verbatim}
}
Si la ligne de commande est [P]main -h[/P], le résultat est l'affichage de la manière d'invoquer la commande, avec un rappel du sens des arguments obligatoires et optionnels.\\

  Les autres arguments reconnus, parmi ceux à partir du rang 6, sont les suivants\label{commentParseMainArguments} (on rappelle que l'argument de rang 0 est le module exécutable lui-même, et que les arguments de rang 1 à 5 sont les fichiers d'entrée standard) :
\begin{itemize} 

\item La présence de l'option {\small {\tt -RscWarn}} (resource-related warnings) permet d'éditer sur la sortie standard des messages sur un constat d'indisponibilité des ressources. \index{TRACE_RSC_WARN_ON} 

\item La présence de l'option {\small {\tt -ActWarn}} (activity-related warnings) permet d'éditer sur la sortie standard des avertissements sur des situations non standard ou peu fréquentes sur les activités (telle que l'interruption d'une activité par l'atteinte de la fin de la période de simulation). On doit chercher une justification à ces situations. \index{TRACE_ACT_WARN_ON} 

\item La présence de l'option {\small {\tt -USO}} (upper subsets option) établit une priorité entre règles de supériorité d'un jeux d'activités sur un autre. Voir la page 'Types prédéfinis ... / Variables globales ... / ... liées aux arguments de la commande'.\index{setSize}\index{maxSpecAllocSize}

\item La présence de l'option {\small {\tt -ActSet n}} borne supérieurement la taille d'un jeu d'activités primitives candidat à l'allocation. Voir la page 'Types prédéfinis ... / Variables globales ... / Borne supérieure de la taille ...' pour le sens ds valeurs possibles.\index{gActSetSizeUpperBound}

\item La présence de l'option {\small {\tt -LogUPS lb[-ub]}} permet d'éditer sur la sortie standard une trace de l'action de la procédure "moteur" d'interprétation du plan d'activités, et des changements d'état d'activités qui en résultent. Les nombres {\em lb} et {\em ub} sont les bornes de la période simulée dans laquelle la trace est activée.\index{g_CD_logUPS_begClock}

\item La présence de l'option {\small {\tt -ActLog}} complète l'option {\small {\tt -LogUPS lb[-ub]}} en traçant l'action des méthodes du type [Y]CheckSonsIf<état>[/Y] et [Y]CheckIfSon<état>[/Y].\index{gTraceActivityLogs}

\item La présence de l'option {\small {\tt -LogMIL lb[-ub]}} permet d'éditer sur la sortie standard une trace de l'action de la procédure "moteur" de génération du meilleur jeu d'activités à exécuter en l'instant courant. Les nombres {\em lb} et {\em ub} sont les bornes de la période simulée dans laquelle la trace est activée.\index{g_CD_logMIL_begClock}

\item La présence de l'option {\small {\tt -LogMIL+ lb[-ub]}} permet d'éditer sur la sortie standard une trace plus détaillée de l'action de la procédure "moteur" de génération du meilleur jeu d'activités à exécuter en l'instant courant.\index{g_CD_logMILp_begClock}

\item La présence de l'option {\small {\tt -LogAIL lb[-ub]}} permet d'éditer sur la sortie standard une trace de l'action de la procédure "moteur" d'exécution du meilleur jeu d'activités à partir de l'instant courant. Les nombres {\em lb} et {\em ub} sont les bornes de la période simulée dans laquelle la trace est activée.\index{g_CD_logAIL_begClock}

\item La présence de l'option {\small {\tt -LogESA lb[-ub]}} permet d'éditer sur la sortie standard une trace de l'action de la procédure "moteur" d'allocation des ressources à un jeu d'activité candidat, en l'instant courant. Les nombres {\em lb} et {\em ub} sont les bornes de la période simulée dans laquelle la trace est activée.\index{g_CD_logESA_begClock}

\item La présence de l'option {\small {\tt -LogESA+ lb[-ub]}} permet d'éditer sur la sortie standard une trace plus détaillée de l'action de la procédure "moteur" d'allocation des ressources, en l'instant courant.\index{g_CD_logESAp_begClock}

\end{itemize}


 */
//@{
          /// Interprète les arguments de la commande exécutable. Renvoie FALSE si la commande n'a pas pu être complètement interprétée. Commence par invoquer ParseMainArguments de la couche \D.\index{fonctions!C_ParseMainArguments}
Boolean C_ParseMainArguments (int argc, char* argv[]); // + JPR 100518
Boolean C_ParseMainArguments(int, char*[ ], Boolean);  // _no swig_ // + JPR 111123
//@}


/**@name Création d'instances de classes (entités, processus, etc.) 

Pour chaque type d'objet, la fonction doit être invoquée dans la fonction [Y]App_New<...>[/Y] (voir page équivalente dans la documentation de \BD), complémentairement au code qui invoque les constructeurs des classes propres à l'application. Par exemple :\\

[P]
\begin{verbatim}
Entity* App_NewEntityFromString(char* nom) {
  Entity* result = C_NewEntityFromString(nom);
  //
  // si la classe est predefinie dans bdiese ou cdiese :
  if(result) return result;
  //
  // sinon :
  if (!strcmp(nom, "activiteA")) return new Activite_A();
  ...
  return NULL;
}
\end{verbatim}
[/P]
*/
//@{
         /// Fonction d'instanciation des classes d'entités prédéfinies dans \CDb ([C]Activity[/C], \ldots). Ce sont les constructeurs sans paramètre qui sont invoqués. 
Entity* C_NewEntityFromString(char*);

         /// Fonction d'instanciation des classes de processus prédéfinies dans \CDb ([C]ResourceImmobilizationProcess[/C], \ldots). Ce sont les constructeurs sans paramètre qui sont invoqués.
Process* C_NewProcessFromString(char*);

         /// Fonction d'instanciation des classes d'événements prédéfinies dans \CDb ([C]UpdateSituationEvent[/C], \ldots). Ce sont les constructeurs sans paramètre qui sont invoqués.
Event* C_NewEventFromString(char*);

         /// Fonction d'instanciation des classes de spécifications d'ensembles d'entités prédéfinies dans \CD. Il n'y en a pas dans la version courante de \CD.
EntitySpec* C_NewEntitySpecFromString(char*);

         /// Fonction d'instanciation des classes de spécifications de domaines de valeurs prédéfinies dans \CD. Il n'y en a pas dans la version courante de \CD.
DescValueSpec* C_NewDescValueSpecFromString(char*);

         /// Fonction d'instanciation des classes de fichiers séquentiels de nom de classe donné, prédéfinies dans \CD. Il n'y en a pas dans la version courante de \CD.
SerialDataFile* C_NewSerialDataFileFromString(char*);

         /// Fonction d'instanciation des classes de fichiers séquentiels de symbole de classe donné, prédéfinies dans \CD. Il n'y en a pas dans la version courante de \CD.
SerialDataFile* C_NewSerialDataFileFromId(UserFileId);

         /// Fonction qui renvoie le symbole de la sous-classe de [C]SerialDataFile[/C] dont le nom de classe est passé en argument. Renvoie toujours -9 car il n'y en a pas dans le version courante de \CD.
UserFileId C_SerialDataFileIdFromString(char* s);
//@}  // fin création d'instances de classes
 
 
/**@name Gestion de la hiérarchie des classes d'entités 

Pour comprendre le motif et le principe de l'installation de la hiérarchie des classes d'entités, lire la page équivalente dans la documentation de \BD.\\

Ainsi, pour installer une hiérarchie de classes d'entités, l'utilisateur opére normalement de la façon suivante :
\begin{enumerate}

\item {\small Les classes du domaine.} Un fichier [F]UserEntity.h[/F] contient la déclaration des classes du domaine, avec pour le moins un constructeur sans argument, et avec spécification de l'héritage. Dans cet exemple, la classe [C]A[/C] hérite de la classe [C]B[/C] :

[P]
\begin{verbatim}
class A : public Entity { public: A();};
class B : public A { public: B();};
\end{verbatim}
[/P]

\item {\small Les métaclasses}. Quelque part dans le code, mais normalement dans une fonction dédiée, couramment appelée [Y]App_InstallMetaClasses[/Y], on crée les instances de [/C]MetaClass[/C] associées à [C]A[/C] et [C]B[/C] ... :

[P]
\begin{verbatim}
void App_InstallMetaClasses() {
  MetaClass* pMC;
  pMC = CreateMetaClass(ID_A, "a"); pMC->SetParentClassId(BD_ENTITY);
  pMC = CreateMetaClass(ID_B, "b"); pMC->SetParentClassId(ID_A);
}
\end{verbatim}
[/P]
... où ID_A et ID_B (resp. "a" et "b") sont les symboles (resp. les noms) de classes choisis pour [C]A[/C] et [C]B[/C].\\

Cette fonction est invoquée au début du code de toute application (mais après la création d'une instance de [C]Simulation[/C] et après l'invocation de [Y]D_InstallMetaClasses[/Y] et de  [Y]D_InstallMetaClasses[/Y], qui ont le même rôle pour les classes prédéfinies de \BD et de \CD). \index{fonctions!App_InstallMetaClasses}\label{commentInstallMetaClasses} 

[P]
\begin{verbatim}
int main(int argc, char argv[]) {
  ...
  pCurrentSim = new Simulation();
  ...
  D_InstallMetaClasses();
  C_InstallMetaClasses();
  App_InstallMetaClasses();
  SetHierarchy();
  ...
}
\end{verbatim}
[/P]

*/
//@{
         /// Fonction d'initialisation des informations sur les sous-classes de [C]BasicEntity[/C] propres à \CD. Cette fonction doit être invoquée au début du code de toute application (mais après la création d'une instance de [C]Simulation[/C]). \index{fonctions!C_InstallMetaClasses} 
void      C_InstallMetaClasses(); // _no swig_  

// Crée une instance de chaque classe devant figurer dans le fichier *.cls, c'est-à-dire celles dont l'utilisateur de Solfege pourra développer des spécialisations. Exploité seulement dans mainMakeCls.cc.
void      C_InstantiateClassesForCls(); // + JPR 090819

//@}  // fin gestion de la hiérarchie

/**@name Suppression d'instances de classes (entités, processus, etc.) */
//@{

         /// Si pObj est une instance d'une classe définie dans \CD, exécute le destructeur de cette classe, sinon appelle la fonction [Y]DeleteEntity[/Y].
void      C_DeleteEntity(BasicEntity* pObj);

///Supprime par [Y]C_DeleteEntity[/Y] toutes les instances (celles non encore désallouées au moment de l'appel de cette fonction) des sous-classes "utilisateur" de [C]BasicEntity[/C] (voir page \pageref{DeleteDOinstances}). Un appel de {\em DeleteEntityInstances} ne doit pas suivre immédiatement un appel de [Y]DeleteEntityClasses[/Y]. \index{fonctions!C_DeleteEntityInstances} 
void      C_DeleteEntityInstances(); // _no swig_  

///Supprime par [Y]C_DeleteEntity[/Y] toutes les instances (celles non encore désallouées au moment de l'appel de cette fonction) de la sous-classe "utilisateur" de [C]BasicEntity[/C] de symbole de classe {\em id}.  
void      C_DeleteEntityInstances(UserClassId id); // _no swig_

///Supprime toutes les instances (celles non encore désallouées) de toutes les classes, et affiche un rapport par [Y]MemoryReport[/Y]. Supprime aussi l'instance de [C]Simulation[/C] pointée par [Y]pCurrentSim[/Y]. Les instances de [C]BasicEntity[/C] sont désallouées par [Y]C_DeleteEntityInstances[/Y]. Les instances de [C]File[/C] sont désallouées par [Y]C_DeleteFileInstances[/Y].\index{fonctions!C_DeleteAll} 
void      C_DeleteAll(); // _no swig_

          // Desalloue en profondeur toutes les activités primitives. \index{fonctions!DeallocatePrimitiveActivityTab}
// voir commentaire dans Dhivine.4.2/source/main.cc
void      DeallocatePrimitiveActivityTab(); //+ JPR 110322 (Dhivine)

          // Desalloue en profondeur toutes les opérations.\index{fonctions!DeallocateOperationTab}
void      DeallocateOperationTab(); //+ JPR 110325 (Dhivine)
          //
//void DeletePrimitiveActivityInstances(); // _no swig_
          //
//void DeletePrimitiveActivityInstances(UserClassId);

/**@name Bilan constructions / destructions
Affiche un rapport sur les constructions des instances des classes prédéfinies de \CD. On place typiquement un appel à cette fonction en fin de la fonction 'main'. Les classes en question ne doivent pas avoir été supprimées par un appel approprié à [Y]DeleteEntityClasses[/Y]. Une séquence stantard est :
[P]
\begin{verbatim}
{
  ...
  C_DeleteEntityInstances();
  C_MemoryReport();
  DeleteEntityClasses();
  ...
 }
\end{verbatim}
[/P]
\index{fonctions!C_MemoryReport} 
*/
//@{

///Si l'argument est FALSE, invoque [Y]C_MemoryReport()[/Y] seulement si l'option {\tt -MR} a été prise dans la ligne de commande. Invoque [Y]C_MemoryReport()[/Y] sans condition si l'argument est TRUE. 
void C_MemoryReport (Boolean); // _no swig_ 

/**@name C_MemoryReport

Affiche un rapport sur les constructions des instances des classes prédéfinies de \BD et \CD. On place typiquement un appel à cette fonction en fin de fonction 'main'. Les classes en question ne doivent pas avoir été supprimées par un appel approprié à [Y]DeleteEntityClasses[/Y]. Une séquence stantard est :
[P]
\begin{verbatim}
{
  ...
  C_DeleteEntityInstances();
  C_MemoryReport();
  DeleteEntityClasses();
  ...
 }
\end{verbatim}
[/P]\index{fonctions!C_MemoryReport} 
*/
//@{
        ///
void C_MemoryReport (); // _no swig_ 
//@}      
//@}    

//@}  // fin suppression d'instances de classes

//===========================================================================

/**@name Fonctions de programmation d'événements prédéfinis 

Les événements prédéfinis le sont pour la plupart pour les besoins des procédures "moteur" de \CD. A cet effet, leur insertion dans l'agenda est donc gérée par le moteur lui-même. Les fonctions présentées dans cette page peuvent toutefois satisfaire un besoin particulier exceptionnel dans une application.\\

Le premier argument de ces fonctions est la valeur d'horloge à laquelle l'événement est programmé.
*/
//@{             
         /// Programmation d'un événement [C]UpdateSituationEvent[/C]. L'événement placé dans l'agenda n'est pas autogénérable. Un tel appel n'est fait que par le moteur de \CD, lorsqu'une composante non terminale d'une activité 'Meeting' vient de devenir CLOSED. Il faut en effet ouvrir la composante suivante. L'éventuelle présence dans l'agenda un événement du même type provenant d'une autogénération n'est pas remise en question. Surchargement possible, en second argument, de la priorité standard (PRIORITY_US).
void PostUpdateSituationEvent(int, int); // _no swig_  // + JPR 090331

Boolean C_ExistUpdateSituationEventAtDate(int laDate);  // + JPR 090331

         /// Programmation d'un événement [C]MakeInstructionListEvent[/C]. L'événement placé dans l'agenda n'est pas autogénérable. Un tel appel n'est fait que par le moteur de \CD, lorsqu'une opération vient d'être terminée (TERMINATED), c'est-à-dire que les conditions d'affectation des ressources viennent d'être modifiées. L'éventuelle présence dans l'agenda un événement du même type provenant d'une autogénération n'est pas remise en question. Surchargement possible, en second argument, de la priorité standard (PRIORITY_MAKE). Le troisième argument est l'instance d'[C]ActivitiesResourcesBlock[/C] au titre duquel on programme l'événement. Seulement si le quatrième argument est TRUE, les autres événémements du même type sont enlevés de l'agenda.
void PostMakeInstructionListEvent(int, int, Entity*, Boolean); // ARB // _no swig_

         /// Programmation d'un événement [C]MakeInstructionListEvent[/C], sans surchargement de la priorité standard : PRIORITY_MAKE.
void PostMakeInstructionListEvent(int, Entity*, Boolean);  //  ARB // _no swig_

         // Non surchargement de la priorité standard : PRIORITY_MAKE. Cette variante déprogramme en  même temps les proceedOperationprocess.
void PostMakeInstructionListEvent2(int, Entity*, Boolean);  //  ARB // _no swig_

Boolean C_ExistMakeInstructionListEventAtDate(int laDate, Entity* arb);
        
         /// Programmation d'un événement [C]ActInstructionListEvent[/C]. Surchargement possible, en second argument, de la priorité standard. Le troisième argument est le jeu d'activités primitives à exécuter).
void PostActInstructionListEvent(int, int, InstructionSet*);  // Pack // _no swig_
        
         /// Programmation d'un événement [C]ActInstructionListEvent[/C] sans surchargement de la priorité standard : PRIORITY_ACT.
void PostActInstructionListEvent(int, InstructionSet*);  // Pack // _no swig_
        
         /// Programmation d'un événement [C]ProceedOperationEvent[/C]. Non surchargement de la priorité standard : PRIORITY_PROCEED_OT. 
void PostProceedOperationEvent(int, Operation*);  // _no swig_  
    
//@} // fin fonctions de programmation

//===========================================================================

/* *@name Corps de méthodes prédéfinies */
// @{

/* *@name Corps de méthodes prédéfinies de processus */
// @{

         // / 
void updateSituationProcess_execProcess_Body(ProcessMethod*); // _no swig_
         // / 
void makeInstructionListProcess_execProcess_Body(ProcessMethod*); // _no swig_
         // + JPR 100531
void makeInstructionListProcess2_execProcess_Body(ProcessMethod*); // _no swig_
         // / 
void actInstructionListProcess_execProcess_Body(ProcessMethod*); // _no swig_

         // / 
void proceedUnitOperationProcess_initializeProcess_Body(ProcessMethod*); // _no swig_
         // / 
void proceedIntQuantityOperationProcess_initializeProcess_Body(ProcessMethod*); // _no swig_
         // / 
void proceedFloatQuantityOperationProcess_initializeProcess_Body(ProcessMethod*); // _no swig_
         // / 
void proceedTimeOperationProcess_initializeProcess_Body(ProcessMethod*); // _no swig_

         // / 
void proceedUnitOperationProcess_goOneStepForwardProcess_Body(ProcessMethod*); // _no swig_
         // / 
void proceedIntQuantityOperationProcess_goOneStepForwardProcess_Body(ProcessMethod*); // _no swig_
         // / 
void proceedFloatQuantityOperationProcess_goOneStepForwardProcess_Body(ProcessMethod*); // _no swig_
         // / 
void proceedTimeOperationProcess_goOneStepForwardProcess_Body(ProcessMethod*); // _no swig_

         // / 
void augmentOperationPercentAchievedProcess_execProcess_Body(ProcessMethod*); // _no swig_

         // / 
void proceedOperationProcess_stopProcess_Body(ProcessMethod*); // _no swig_

         // / 
void resourceImmobilizationProcess_execProcess_Body(ProcessMethod*); // _no swig_
         // / 
void resourceMobilizationProcess_execProcess_Body(ProcessMethod*); // _no swig_

// @}  // fin corps de méthodes prédéfinies de processus

// @}  // fin corps de méthodes prédéfinies

//===========================================================================

/**@name Fonctions sur les ensembles d'entités

La procédure d'allocation de resources à un jeu d'activités est amenée à réduire un jeu candidat si une pleine allocation (à toutes les activités) est en échec. Les fonctions [Y]SplitToSubsets[/Y] sont alors exploitées par le moteur à cet effet.\\

Dans la même procédure, le développement d'une spécification avec la modalité SETS invoque la fonction [Y]DispatchToSets[/Y].\\

Si ces fonctions sont présentées dans cette page, c'est parce qu'elle peuvent satisfaire un besoin particulier exceptionnel dans une application, lié à l'allocation de rsssources ou non.
*/
//@{
         /// Renvoie ((a b c) (a b d) (a c d) (b c d)) à partir de la liste d'entités (a b c d).
pEntityDTab* SplitToSubsets(pEntityTab*); // _no swig_ 
         /// Si n=2, renvoie ((a b) (a c) (a d) (b c) (b d) (c d)) à partir de la liste d'entités (a b c d). 
pEntityDTab* SplitToSubsets(int n, pEntityTab*); // _no swig_ 
         /// Se reporter à l"Ontologie des systèmes pilotés" §5.5.2.4.3.2 pour une documentation de cette fonction.
pEntityDTab* DispatchToSets(int n, pEntityTab*); // _no swig_ 

         // / 
pIntDTab* DispatchNintoK(int n, int k, int level);

         // / 
Boolean ScreenForAvailable(Entity* spec, 
			   pEntityTab* in, pEntityTab* out, 
			   int fctId);
         // / 
Boolean ScreenForAvailableOperatedObjects(Entity* spec, 
			   pEntityTab* in, pEntityTab* out, 
			   int fctId);
         // / 
Boolean ScreenForAvailablePerformers(Entity* spec, 
			   pEntityTab* in, pEntityTab* out, 
			   int fctId);
//@}  // fin Fonctions liées aux spécifications

//===========================================================================

/**@name Fonctions d'examen de fichier de données séquentielles 

\CD intègre une classe particulière de fichiers de données séquentielles, [C]BufferizedDataFile[/C], qui permet d'accéder dynamiquement, parmi les données contenues dans le fichier externe, \\
- à des enragistrements correspondant à une date passée : il s'agit d'une opération de remémorisation (autrement dit de souvenir) ; \\
- ou bien à des enregistrements correspondant à une date future : il s'agit d'une opération de prédiction.\\

Les services prédéfinis de remémorisation et de prédiction sont présentées ici. Dans chaque trio de fonctions, une est dédiée aux champs à valeur entière, une autre aux champs à valeur flottantes, la dernière aux champs  à valeur flottantes 'double précision'.\\

{\large  {\tt Mémorisation}}

Les fonctions suivantes renvoient, pour le fichier dont le symbole de classe est donné en premier argument, la somme des valeurs mémorisées pour le champ donné en second argument, entre les bornes (comprises) spécifiées par les deux derniers arguments. La borne 0 correspond à l'enregistrement lu au pas courant de la simulation, la borne 1 correspond au pas précédent, etc. \\

[Y]int    SumUpMemoryIntFieldValue(UserFileId id, int fieldId, int lb, int ub);[/Y]\\
[Y]float  SumUpMemoryFloatFieldValue(UserFileId id, int fieldId, int lb, int ub);[/Y]\\
[Y]double SumUpMemoryDoubleFieldValue(UserFileId id, int fieldId, int lb, int ub);[/Y]\\

Les fonctions suivantes ont le même rôle, avec le chemin d'accès au fichier externe en premier argument.\\

[Y]int    SumUpMemoryIntFieldValue(char* path, int fieldId, int lb, int ub);[/Y]\\
[Y]float  SumUpMemoryFloatFieldValue(char* path, int fieldId, int lb, int ub);[/Y]\\
[Y]double SumUpMemoryDoubleFieldValue(char* path, int fieldId, int lb, int ub);[/Y]\\

{\large  {\tt Prédiction}}

Les méthodes suivantes renvoient, pour le fichier dont le symbole de classe est donné en premier argument la somme des valeurs prédites pour le champ donné en second argument, entre les bornes (comprises) spécifiées par les deux derniers arguments. La borne 0 (qui correspondrait au pas courant) est interdite. La borne 1 correspond à l'enregistrement qui sera lu au pas suivant de la simulation, etc.\_


[Y]int     SumUpPredictionIntFieldValue(UserFileId id, int fieldId, int lb, int ub);[/Y]\\
[Y]float SumUpPredictionFloatFieldValue(UserFileId id, int fieldId, int lb, int ub);[/Y]\\
[Y]double SumUpPredictionDoubleFieldValue(UserFileId id, int fieldId, int lb, int ub);[/Y]\\

Les fonctions suivantes ont le même rôle, avec le chemin d'accès au fichier externe en premier argument.\\

[Y]int SumUpPredictionIntFieldValue(char* path, int fieldId, int lb, int ub);[/Y]\\
[Y]float SumUpPredictionFloatFieldValue(char* path, int fieldId, int lb, int ub);[/Y]\\
[Y]double SumUpPredictionDoubleFieldValue(char* path, int fieldId, int lb, int ub);[/Y]\\

{\large  {\tt Exemple}}

Soit le fichier externe représenté par la classe [C]MyFile[/C] de symbole MY_FILE.\\
Ses trois champs ont pour symboles FLD_DATE FLD_F et FLD_I, et de type {\em char*}, {\em float} et {\em int}.\\
Les enregistrement correspondent à des jours successifs. \\
Une partie continue des enregistrements est listée ici : \\
\begin{verbatim}
22/03/2001 12.4   7
22/03/2001 6.7    20
23/03/2001 14.9   13
24/03/2001 12.5   2
25/03/2001 20.0   19
26/03/2001 2.5    11
27/03/2001 8.3    6
28/03/2001 13.3   17
29/03/2001 7.4    1
\end{verbatim}

Alors, les appels suivants renvoient les valeurs affichées, compte tenu que:\\
- l'unite d'horloge de la simulation est '1 DAY' :\\ 
- la valeur courante de l'horloge de la simulation correspond au 25 mars 2001

[P]
\begin{verbatim}{
  ...
  int mi = SumUpMemoryIntFieldValue(MY_FILE, FLD_I, 1, 3);
  float mf = SumUpMemoryFloatFieldValue(MY_FILE, FLD_F, 3, 3);
  int pi = SumUpPredictionIntFieldValue(MY_FILE, FLD_I, 1, 3);
  float pf = SumUpPredictionFloatFieldValue(MY_FILE, FLD_F, 3, 3);
  printf("\nmi = %d mf = %3.1f\tpi = %d pf = %3.1f", mi, mf, pi, pf);
}
\end{verbatim}
[/P]
>>
\begin{verbatim}
mi = 35 mf =  6.7     pi = 34 pf = 13.3
\end{verbatim}
*/
//@{
int CommonSumUpMemoryIntFieldValue(BufferizedDataFile* pBF,
								   int fieldId, int lb, int ub);
int SumUpMemoryIntFieldValue(UserFileId id, int fieldId, 
				 int lb, int ub);
int SumUpMemoryIntFieldValue(char* path, int fieldId, 
				 int lb, int ub);
float SumUpMemoryFloatFieldValue(UserFileId id, int fieldId, 
				     int lb, int ub);
float SumUpMemoryFloatFieldValue(char* path, int fieldId, 
				     int lb, int ub);
double SumUpMemoryDoubleFieldValue(UserFileId id, int fieldId, 
				       int lb, int ub);
double SumUpMemoryDoubleFieldValue(char* path, int fieldId, 
				       int lb, int ub);

int SumUpPredictionIntFieldValue(UserFileId id, int fieldId, 
				 int lb, int ub);
int SumUpPredictionIntFieldValue(char* path, int fieldId, 
				 int lb, int ub);
float SumUpPredictionFloatFieldValue(UserFileId id, int fieldId, 
				     int lb, int ub);
float SumUpPredictionFloatFieldValue(char* path, int fieldId, 
				     int lb, int ub);
double SumUpPredictionDoubleFieldValue(UserFileId id, int fieldId, 
				       int lb, int ub);
double SumUpPredictionDoubleFieldValue(char* path, int fieldId, 
				       int lb, int ub);

//@}  // fin Fonctions d'examen de fichier de données séquentielles


/**@name  D'un élément d'énumération à une chaîne de caractères

{\small Pour diverses énumérations} \\

char* [Y]C_StateIdToString[/Y](C_StateId); \\
     SLEEPING=1 --> la chaîne "SLEEPING" \\ 
     WAITING=2 --> la chaîne "WAITING", etc. \\

char* [Y]C_ContinuationModeToString[/Y](C_ContinuationMode id); \\
     NEW_ENTITIES=1 --> la chaîne "NEW_ENTITIES" \\
     SAME_PREVIOUS_ENTITIES=2 --> la chaîne "SAME_PREVIOUS_ENTITIES", etc. \\

char* [Y]C_ProportionalityModeToString[/Y](C_ProportionalityMode id); \\
     NON_PROPORTIONAL=1 --> la chaîne "NON_PROPORTIONAL" \\
     PROPORTIONAL=2 --> la chaîne "PROPORTIONAL"

char* [Y]C_SelectionModeToString[/Y](C_SelectionMode id); \\
     ANYONE=1 --> la chaîne "ANYONE" \\
     MAX=2 --> la chaîne "MAX", etc. \\

char* [Y]C_QuantityIdToString[/Y](C_QuantityId id); \\
     C_ELEMENTS=0 --> la chaîne "C_ELEMENTS \\

char* [Y]C_GoalIdToString[/Y](C_GoalId id); \\
     C_TARGET=0 --> la chaîne "C_TARGET \\
     C_ANTI_TARGET=1 --> la chaîne "C_ANTI_TARGET \\

char* [Y]C_ActivityActionToString[/Y](C_ActivityActionMode id); \\
     C_TAKE_MIN=1 --> la chaîne "C_TAKE_MIN"  \\
     C_TAKE_MEAN=2 --> la chaîne "C_TAKE_MEAN" \\ 
     C_TAKE_MAX=3 --> la chaîne "C_TAKE_MAX" \\
\\

{\small Pour les classes d'entités},...\\
... de descripteurs, de méthodes et d'arguments, l'utilisateur peut exploiter, respectivement, les variables globales [Y]ClassIdToCharTab[/Y], [Y]DescIdToCharTab[/Y], [Y]MethodIdToCharTab[/Y] et [Y]ArgumentIdToCharTab[/Y]. Ce sont des tableaux de chaînes de caractères, contenant, à l'indice <classId> la chaîne correspondante. Par exemple :\\

[P]
\begin{verbatim}
printf("\nLe symbole de classe ACTIVITY, de valeur %d s'affiche '%s'", 
       ACTIVITY, ClassIdToCharTab(ACTIVITY));
\end{verbatim}
[/P]
>>
\begin{verbatim}
Le symbole de classe ACTIVITY, de valeur 11 s'affiche 'ACTIVITY'
\end{verbatim}

Ce sont les fonctions prédéfinies [Y]C_MakeClassIdToCharTab[/Y], [Y]C_MakeDescIdToCharTab[/Y], [Y]C_MakeMethodIdToCharTab[/Y] et [Y]C_MakeArgumentIdToCharTab[/Y] qui complètent ces tableaux pour la couche \CD, alors qu'ils ont déjà reçu des éléments par la couche \BD. Le programmeur d'une application a intérêt à compléter ces tableaux avec les chaînes correspondant aux symboles de classes de l'application. C'est la raison pour laquelle les entiers associés aux symboles doivent être supérieurs à ceux déjà utilisés par la couche \CD (valeur de [Y]gClassIdToCharTabSize[/Y] pour les classes d'entités). A cet effet, l'utilisateur développera des fonctions pouvant être appellées comme ci-dessous :  \index{fonctions!Make...IdToCharTab}

[P]
\begin{verbatim}
void  App_MakeClassIdToCharTab() {
  for(int k=gClassIdToCharTabSize;k<201;k++)  // le premier entier est 201
	ClassIdToCharTab.push_back("@");

  ClassIdToCharTab.push_back("ID_A");
  ClassIdToCharTab.push_back("ID_B");
  ClassIdToCharTab.push_back("ID_A_ELEM");
  ClassIdToCharTab.push_back("ID_SET_OF_A");
  ....
} 
void  App_MakeDescIdToCharTab() {...};
void  App_MakeMethodIdToCharTab() {...};  
void  App_MakeArgumentIdToCharTab() {...};
\end{verbatim}
[/P]

Ainsi :
[P]
\begin{verbatim}
printf("\nLe symbole de classe ID_A, de valeur %d s'affiche '%s'",
       ID_A, ClassIdToCharTab(ID_A));
\end{verbatim}
[/P]
>>
\begin{verbatim}
Le symbole de classe ID_A, de valeur 201 s'affiche 'ID_A'
\end{verbatim}
 */
//@{ 

         // /
void C_MakeClassIdToCharTab(); // _no swig_ 
         // /
void C_MakeDescIdToCharTab(); // _no swig_
         // /
void C_MakeMethodIdToCharTab(); // _no swig_
         // /
void C_MakeArgumentIdToCharTab(); // _no swig_  

         // fonctions de conversion associées aux tableaux de conversion enum -> chaine de caractères
         // invoquées dans C_EnumToInt, 
         // laquelle surcharge EnumToInt dans C_ParserSimu.yacc.y
         // initialisée à D_EnumToInt dans ParserSimu.yacc.y
         // /
void C_MakeStateIdToCharTab();
         // /
void C_MakeContinuationModeToCharTab();
         // /
void C_MakeDistributionModeToCharTab();
         // /
void C_MakeProportionalityModeToCharTab();
         // /
void C_MakeSelectionModeToCharTab();
         // /
void C_MakeStockAccesModeToCharTab();
         // /
void C_MakeQuantityIdToCharTab();
         // /
void C_MakeQuantityOptionToCharTab();
         // /
void C_MakeGoalIdToCharTab();
         // /
void C_MakeActivityActionToCharTab(); 

         // / Renvoie la chaîne de caractères correspondant au symbole d'état d'activité donné en argument. Par exemple, l'appel {\tt C_StateIdToString(1)} renvoie la chaîne {\tt "SLEEPING"}.\index{fonctions!C_StateIdToString}
char* C_StateIdToString(C_StateId);

          // / Renvoie un mot qui est l'identifiant d'un token de l'énumération des situations (d'activité ou d'opération) dont l'entier correspondant est passé en argument.\index{fonctions!C_ActivitySituationToString}
char* C_ActivitySituationToString(int);

         // / 
char* C_ContinuationModeToString(C_ContinuationMode id);

         // / 
//char* C_DistributionModeToString(C_DistributionMode id);

         // / 
char* C_ProportionalityModeToString(C_ProportionalityMode id);

         // / 
char* C_SelectionModeToString(C_SelectionMode id);

         // / 
char* C_StockAccesModeToString(C_StockAccesMode id);

         // / 
char* C_QuantityIdToString(C_QuantityId id);

         // / 
char* C_QuantityOptionToString(C_QuantityOption id);

         // / 
char* C_GoalIdToString(C_GoalId id);

         // / 
char* C_ActivityActionToString(C_ActivityActionMode id);

//@}  // fin Fonctions
  

/**@name De la chaîne affichant un élément d'énumération à un entier

Le fonction [Y]C_EnumToInt[/Y] est l'équivalente dans \CD à la fonction [Y]D_EnumToInt[/Y] fournie par \BD. \\

Se reporter à cette documentation pour la justification et l'usage de ce service. \\

A titre d'exemple, voici une utilisation de cette fonction : \\
[P]
\begin{verbatim}{
  ...
  printf("\nL'entier associé au symbole de chaîne '%s'
         \ndans l'énumération des états d'activités et d'opérations est %d", 
         "SLEEPING", C_EnumToInt("SLEEPING")); 
}
\end{verbatim}
[/P]
>>
\begin{verbatim}
L'entier associé au symbole de chaîne 'SLEEPING'
dans l'énumération des états d'activités et d'opérations est 1
\end{verbatim}

 */
//@{

          // / Renvoie l'entier correspondant à un token d'énumération dont l'identifiant est le mot passé en argument.\index{fonctions!C_EnumToInt}
int C_EnumToInt(char* str);

//@}  // fin Fonctions

char* C_StateIdToChar(C_StateId);  // obsolète, maintenu pour compatibilité
char* C_ActivitySituationEnumToChar(int);  // obsolète, maintenu pour compatibilité

//===========================================================================

/**@name Fonctions diverses */
//@{

/**@name Liste des activités qui ne sont éléments d'aucune autre 

[Y]pEntityTab* CheckUniqueRoot();[/Y]\\

Renvoie la liste des activités qui ne sont éléments d'aucune autre. \\

Cette fonction est invoquée par le moteur, lorsque le descripteur [C]ReadyToRun[/C] du plan global reçoit la valeur 1 ('ready'). Normalement à ce moment-là, cette liste ne doit contenir qu'un seul élément. Dans le cas contraire, l'exception [Y]NotAUniqueRootPlan[/Y] est levée.\\

Cette fonction est aussi disponible au développeur, à toutes fins utile.
*/
//@{
          ///
pEntityTab* CheckUniqueRoot();
//@}

/**@name Calcul de l'intégrale de Choquet 

[Y]float Choquet(BasicEntity*, pEntityTab*, CriterionSet*, float);[/Y]\\

Calcule l'intégrale de Choquet pour l'entité (typiquement le jeu d'activités) qui est en premier argument, relativement à la liste de critères qui est en second argument. Les poids de tous les jeux possibles de critères (associés ou non à ce jeu) sont en troisième argument. Le quatrième argument est la valeur qui sera renvoyée si l'ensemble de critères est vide.\\

La justification de l'invocation de cette fonction est donnée aux pages relatives aux buts (classe [C]Goal[/C]) et aux critères (classe [C]Criterion[/C]).
*/
//@{
          ///
float Choquet(BasicEntity*, pEntityTab*, CriterionSet*, float);
//@}

/**@name Comparaison de deux nombres entiers

Dans toutes les signatures ci-dessous, les arguments {\small {\tt x}} et{\small {\tt y}} sont deux entiers positifs ou manquant (valeur négative). \\

Les fonctions du premier groupe ci-dessous renvoient {\em vrai} si une donnée est manquante ou si le prédicat est {\em vrai}. Autrement dit, elles renvoient {\em faux} si les données sont présentes et le prédicat {\em faux} :\\

[Y]bool tMyNE(int x, int y);[/Y]\\
[Y]bool tMyEQ(int x, int y);[/Y]\\
[Y]bool tMyLT(int x, int y);[/Y]\\
[Y]bool tMyLE(int x, int y);[/Y]\\
[Y]bool tMyGT(int x, int y);[/Y]\\
[Y]bool tMyGE(int x, int y);[/Y]\\

Les fonctions du second groupe ci-dessous renvoient {\em faux} si une donnée est manquante ou si le prédicat est {\em faux}. Autrement dit, elles renvoient {\em vrai} si les données sont présentes et le prédicat {\em vrai} : \\

[Y]bool fMyNE(int x, int y);[/Y]\\
[Y]bool fMyEQ(int x, int y);[/Y]\\
[Y]bool fMyLT(int x, int y);[/Y]\\
[Y]bool fMyLE(int x, int y);[/Y]\\
[Y]bool fMyGT(int x, int y);[/Y]\\
[Y]bool fMyGE(int x, int y);[/Y]\\

A titre d'exemple :

[P]
\begin{verbatim}{
  ...
  bool ta = tMyNE(3, 4);
  bool tb = tMyNE(3, -9);
  bool fa = fMyNE(3, 3);
  bool fb = fMyNE(3, -9);
  printf("\nta = %d tb = %d\tfa = %d fb = %d", ta, tb, fa, fb);
}
\end{verbatim}
[/P]
>>
\begin{verbatim}
ta = 1 tb = 1     fa = 0 fb = 0
\end{verbatim}

 */
//@{
bool tMyNE(int x, int y);
bool tMyEQ(int x, int y);
bool tMyLT(int x, int y);
bool tMyLE(int x, int y);
bool tMyGT(int x, int y);
bool tMyGE(int x, int y);

bool fMyNE(int x, int y);
bool fMyEQ(int x, int y);
bool fMyLT(int x, int y);
bool fMyLE(int x, int y);
bool fMyGT(int x, int y);
bool fMyGE(int x, int y);
//@}

          //
bool AreDateEqualWRTTolerance(int d_i, int d_j, int tol_i, int tol_j);


          // Renvoie le tableau des activités primitives OPEN ou IN_COURSE. Le tableau doit être désalloué par le client.\index{fonctions!OpenPrimitiveActivityTab} 
//pEntityTab* OpenPrimitiveActivityTab(); //+ JPR 100908

          // Renvoie le tableau des activités primitives IN_COURSE. Le tableau doit être désalloué par le client.\index{fonctions!InCoursePrimitiveActivityTab}
//pEntityTab* InCoursePrimitiveActivityTab(); //+ JPR 100908

          // Ferme (TurnToClosed) les activités primitives IN_COURSE.\index{fonctions!TurnToClosedInCoursePrimitiveActivityTab}
//void TurnToClosedInCoursePrimitiveActivityTab(); //+ JPR 100908


//@}  // fin Fonctions diverses

//X///////////////X/////////////////////////////X////////////////////////////
// fonctions privées à l'usage du développeur de DIESE
//X///////////////X/////////////////////////////X////////////////////////////
// Les fonctions ci-dessous peuvent servir a diagnostiquer plus facilement les 
// fuites de memoires (MLK) quand on utilise un outil tel que valgrind.
// Il suffit de remplacer, par exemple, un appel a new InstructionSetAllocation(x)
// par un appel a MLK_CopyInstructionSetAllocation(x), pour reperer si c'est bien
// cet appel qui est trace par l'analyseur de memoire (valgrind).
//
//X///////////////X/////////////////////////////X////////////////////////////

InstructionSetAllocation* MLK_CopyInstructionSetAllocation(InstructionSetAllocation&);
InstructionSetAllocation* MLK_CopyInstructionSetAllocation2(InstructionSetAllocation&);
InstructionSetAllocation* MLK_CreateInstructionSetAllocation();
InstructionSetAllocation* MLK_CreateInstructionSetAllocation1();
InstructionAllocation* MLK_CreateInstructionAllocation();
InstructionAllocation* MLK_CreateInstructionAllocation2();
AllocatedOperatedObjectSet* MLK_CreateAllocatedOperatedObjectSet();
AllocatedOperationResourceSet* MLK_CreateAllocatedOperationResourceSet();
AllocatedPerformerSet* MLK_CreateAllocatedPerformerSet();
AllocatedOperationSet* MLK_CreateAllocatedOperationSet();
AllocatedDestinationObjectSet* MLK_CreateAllocatedDestinationObjectSet();
CoengagementItem* MLK_CreateCoengagementItem();
ResourceUsageItem* MLK_CreateResourceUsageItem();
UsageItem* MLK_CopyUsageItem(UsageItem&);
OperationUsageItem* MLK_CreateOperationUsageItem();

pEntityTab* MLK_CopyEntityTab(const pEntityTab&);
pEntityTab* MLK_CreateEntityTab();
pEntityDTab* MLK_CreateEntityDTab();

InstructionSetAllocation* MLK_ReturnExtensionWithComb0(InstructionSetAllocation* pSujet, Entity* pObjSet);
InstructionSetAllocation* MLK_ReturnExtensionWithComb1(InstructionSetAllocation* pSujet, Entity* pObjSet);
InstructionSetAllocation* MLK_ReturnExtensionWithComb2(InstructionSetAllocation* pSujet, Entity* pObjSet);
InstructionSetAllocation* MLK_ReturnExtensionWithComb3(InstructionSetAllocation* pSujet, Entity* pObjSet);

Descriptor*  MLK_CreateDescResourceList();
Descriptor*  MLK_CreateDescResourceId();

void MLK_DeleteInDepth(InstructionSetAllocation*);
void MLK_DeleteInDepth(InstructionAllocation*);
void MLK_DeleteInDepth(Operation*);
void MLK_DeleteEntity(Entity*);
void MLK_DeleteInDepthEntity(Entity*);


SplittableInstructionSet* MLK_CreateSplittableInstructionSet();
SplittableInstructionSet* MLK_CopySplittableInstructionSet(SplittableInstructionSet&);
SingleInstructionSet* MLK_CopySingleInstructionSet(SingleInstructionSet&);
SingleInstructionSet* MLK_CopySingleInstructionSet2(SingleInstructionSet&);

ResourceSet* MLK_CreateResourceSet();
InstructionAllocation* MLK_ExtractVAlloc1(InstructionSetAllocation*);
InstructionAllocation* MLK_ExtractVAlloc2(InstructionSetAllocation*);

//@}  // fin Fonctions globales


#endif  // _C_COREFUNCTIONS_H


