//============================================================================
// FILE: C_InconsistentResourceCommitment.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des conditions d'inconsistances d'alloc. de ressources
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_INCONSISTENTRESOURCECOMMITMENT_HH
#define _C_INCONSISTENTRESOURCECOMMITMENT_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "Entity.h"

#include "C_CoreTypes.h"
//#include "C_SingleResource.h"
#include "C_ResourceAllocation.h"

// EXTERNAL REFERENCES

// /X///////////////X/////////////////////////////X///////////////////////////
//
// CONDITIONS D'INCONSISTANCE D'UNE ALLOCATION DE RESSOURCES
//
// /X///////////////X/////////////////////////////X///////////////////////////

/**@name Conditions d'inconsistance d'une allocation de ressources
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_InconsistentResourceCommitment.h} \end{itemize}
Une allocation de ressources aux différentes activités d'un jeu candidat à l'exécution peut être inconsistante. Dans ce cas, la procédure d'allocation va tenter d'allouer un ou plusieurs sous-ensembles du jeu candidat, et ainsi de suite jusqu'à réussir à allouer un jeu (éventuellement réduit à une seule des activités du jeu initial) ou à échouer si aucune activité primitive ne peut recevoir d'allocation.\\

Il existe diverses causes d'inconsistance d'une allocation :\\
1) une ressource est allouée à plusieurs activités, alors que sa nature ne le permet pas. Par exemple, un ouvrier ne peut peut pas servir deux activités qui opèrent sur deux lieux éloignés.\\
2) une ressource est co-engagée avec une autre ressource dans la mise en oeuvre d'un jeu d'activités, alors que sa nature ne le permet pas. Par exemple, il est incompatible de faire utiliser tel instrument par un opérateur qui n'en a pas la compétence.\\
3) une ressource est allouée à une activité, alors que celle-ci l'exclut explicitement. Par exemple, une activité requiert exclusivement un modèle bien précis d'outil, et pas un autre qui aurait cependant la même fonction.\\
4) enfin, une allocation pourra être rejetée si et seulement si plusieurs parmi les causes ci-dessus sont conjointement observées.\\

Les classes d'entités qui permettent de modéliser ces situations sont, respectivement :\\
- {\bf ResourceSharingViolationCondition}, pour les causes 1 et 2 \\;
- {\bf ActivityInconsistencyCondition}, pour la cause 3 ;\\
- {\bf ActivitiesResourcesInconsistentCommitment}, pour la cause 4 ;\\

Ces classes reposent sur deux autres structures de données : \\
- {\bf OccurrenceDomain}, pour modéliser un domaine de situations d'engagement inacceptable ou impossible d'une ressource. Par exemple, pour exprimer qu'une ressource R ne doit pas être engagée sur plus d'un lieu à la fois, on créera la structure {LIEU, >, 1} qu'on attachera à R.\\
- {\bf OccurrenceCrossDomain}, pour modéliser une conjonction des situations définies ci-dessus.\\

C'est en fait toujours une instance d'{\bf OccurrenceCrossDomain} qu'on attachera à une ressource, même si la conjonction ne comporte qu'un seul {\bf OccurrenceDomain}.\\

Les contraintes sur les ressources sont attachées au système opérant (instance d'{\bf OperatingSystem}, qui est un composant du système de production. cet attachement se fait par la valeur du descripteur {\bf ResourceConstraintList}. Les éléments de cette liste sont les contraintes (des trois types) qui doivent toutes être satisfaites au moment où elles sont testées dans la procédure d'allocation. \\

Remarque : Il est inutile de faire figurer dans la liste une {\bf ActivitiesResourcesInconsistentCommitment} A possédant comme élément une contrainte C d'un des deux autres types qui figure aussi elle-même dans la liste. En effet, si C est violée, l'allocation est rejetée quel que soit le résultat du test des autres éléments de A. Si C n'est pas violée, A ne l'est pas non plus. Dans les deux cas, la présence de A ne change pas le résultat de l'allocation.\\
Par exemple, la liste ResourceConstraintList {aric1(aic1, rsvc1}, aic1) peut se réduire eu {aic1}.\\
*/ 
//@{

/**@name La classe de base
 */
//@{
          ///
class InconsistencyCondition : public Entity
{
public:
          // Constructeur de base.
  InconsistencyCondition();
          // Constructeur par copie.
  InconsistencyCondition(const InconsistencyCondition&);
          // Destructeur.
  ~InconsistencyCondition();

/** @name  OPERATION */
// @{
          //
//virtual Boolean Check();
// @}
};

//@}  // fin classe de base



/**@name Inconsistance sur le partage d'une ressource
 */
//@{
          ///
class ResourceSharingViolationCondition : public InconsistencyCondition
{
public:
          // Constructeur de base.
  ResourceSharingViolationCondition();
          // Constructeur par copie.
  ResourceSharingViolationCondition(const ResourceSharingViolationCondition&);
          // Destructeur.
  ~ResourceSharingViolationCondition();

/**@name  OPERATION */
//@{
          ///
  // Boolean Check(const InstructionSetAllocation* pUalloc);
  Boolean Check(InstructionSetAllocation* pUalloc);  // modif JPR 091019
//@}
};

//@}  // fin Inconsistance sur le partage d'une ressource



/**@name Inconsistance sur l'engagement conjoint de ressources dans une activité
 */
//@{
          ///
class ActivityInconsistencyCondition : public InconsistencyCondition
{
public:
          // Constructeur de base.
  ActivityInconsistencyCondition();
          // Constructeur par copie.
  ActivityInconsistencyCondition(const ActivityInconsistencyCondition&);
          // Destructeur.
  ~ActivityInconsistencyCondition();

/**@name  OPERATION */
//@{
          ///
Boolean Check(const pEntityTab* pRscSet);
//@}
};

//@}  // fin Inconsistance sur l'engagement conjoint ...



/**@name Inconsistance d'allocation sur un jeu d'activités
 */
//@{
          ///
class ActivitiesResourcesInconsistentCommitment : public Entity
{
public:
          // Constructeur de base.
  ActivitiesResourcesInconsistentCommitment();
          // Constructeur par copie.
  ActivitiesResourcesInconsistentCommitment(const ActivitiesResourcesInconsistentCommitment&);
          // Destructeur.
  ~ActivitiesResourcesInconsistentCommitment();

/**@name  OPERATION */
//@{
          // /
  Boolean Check(PrimitiveActivity* pAct,
				InstructionSetAllocation* pUAct, 
				InstructionSetAllocation*);
//@}
};

//@}  // fin Inconsistance d'allocation sur un jeu d'activités



/**@name Domaine de situations d'engagement inacceptable 
 */
//@{
          ///
class OccurrenceDomain : public Entity
{
public:
          // Constructeur de base.
  OccurrenceDomain();
          // Constructeur par copie.
  OccurrenceDomain(const OccurrenceDomain&);
          // Destructeur.
  ~OccurrenceDomain();

/**@name  OPERATION */
//@{
          // 
  Boolean CheckOnCoengagementList(const pEntityTab coengagementList);
          // 
  Boolean CheckOnResourceList(const pEntityTab* pRscList);
//@}
};

//@}  // fin Domaine de situations d'engagement



/**@name Conjonction de domaines de situations d'engagement inacceptable 
 */
//@{
          ///
class OccurrenceCrossDomain : public Entity
{
public:
          // Constructeur de base.
  OccurrenceCrossDomain();
          // Constructeur par copie.
  OccurrenceCrossDomain(const OccurrenceCrossDomain&);
          // Destructeur.
  ~OccurrenceCrossDomain();

/**@name  OPERATION */
//@{
          // 
  Boolean Check();
//@}
};

//@}  // fin Conjonction de domaines ...


//@}  // fin Conditions d'inconsistance d'une allocation de ressources


//@}  // fin Les ressources (C_Resource.h)


#endif  // _C_INCONSISTENTRESOURCECOMMITMENT_HH



