//============================================================================
// FILE: C_ResourceAllocation.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des classes représentant des jeux de ressources allouées
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_RESOURCEALLOCATION_HH
#define _C_RESOURCEALLOCATION_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "Entity.h"

#include "C_Operation.h"

// EXTERNAL REFERENCES

/////////////////////////////////////////////////////////////////////////////
//
// INSTRUCTION_ALLOCATION
//
/////////////////////////////////////////////////////////////////////////////

/**@name Structures pour les allocations de ressources\\

\begin{itemize} \item Directive d'inclusion : 
[P]\#include C_ResourceAllocation.h[/P]
\end{itemize}

L'allocation des ressources aux opérations et aux activités est réalisée par le moteur de simulation en fonction des spécifications exprimées dans les attributs [C]RequiredResources[/C] (classe [C]Operation[/C]) et d'autre part [C]OperatedObjectSpecification[/C], [C]PerformerSpecification[/C] (classe [C]PrimitiveActivity[/C]).\\

La procédure d'allocation s'appuie sur des structures de données qu'elle instancie elle-même, puis qu'elle alimente elle-même. Ces structures sont l'objet des tests de satisfaction des contraintes sur l'allocation des ressources (partageabilité, consistance d'allocations conjointes).\\

On comprend donc que le code de l'application, et les déclarations externalisées, n'ont pas vocation à entrer dans le cycle de vie de ces structures. Elles sont cependant présentées ici, parce qu'elles peuvent être un moyen d'inspection des ressources effectivement allouées, complémentaire des spécifications, lesquelles ne constituent que des allocations prévisionnelles, candidates en quelque sorte.\\

Selon qu'on s'intéresse à l'allocation à une activité primitive ou bien à un jeu d'activités primitives (candidates à la réalisation conjointe), on se documentera dans le premier ou dans le dernier des liens du haut de cette page.\\

Pour une présentation complète de l'algorithme d'allocation et des structures de données associées, se reporter au document l'Ontologie des Systèmes Pilotés".
*/ 
//@{

/**@name Jeu de ressources allouées à une activité primitive\\

C'est la structure nommée V dans l'ontologie. Dans DIESE, c'est une instance de la classe [C]InstructionAllocation[/C].\\

Lorsqu'on dispose d'un pointeur sur une instance de [C]PrimitiveActivity[/C], on accède à la structure V de la manière suivante :
[P]
\begin{verbatim}
  Entity* pInst = ......;  // l'instance de [C]PrimitiveActivity[/C]
  Entity* pV_alloc = pInst->GetEntityConstValue(ALLOCATION_ATTRIBUTE);
\end{verbatim}
[/P]

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]InstructionAllocation[/C]} & {\tiny INSTRUCTION_ALLOCATION} & {\tiny instructionAllocation} & {\tiny } & {\tiny ALLOCATED_OPERATED_OBJECT_SET ALLOCATED_OPERATION_RESOURCE_SET ALLOCATED_PERFORMER_SET ALLOCATED_OPERATION_SET ALLOCATED_DESTINATION_OBJECT_SET}
\end{tabular}

Les services offerts par cette classe sont les suivants. de manière générale, ils retournent un tables de pointeurs sur des entités ({\em pEntityTab*}) :

[Y]ExtractRscSet()[/Y] \\ \index{méthodes!ExtractRscSet} 
Renvoie la liste de toutes les ressources allouées à l'activité. Aux ressources en position d'objet opéré dans l'allocation (classe [C]OperatedObject[/C]), on substitue l'entité qui est la valeur du descripteur [C]ResourceEntity[/C] de l'objet opéré. La liste est complétée par le pointeur sur l'opération en jeu (classe [C]Operation[/C]). \\

[Y]ExtractPerfSet()[/Y] \\ \index{méthodes!ExtractPerfSet(} 
Renvoie la liste de toutes les ressources, instances directes ou ndirectes de [C]Performer[/C], allouées à l'activité. \\

[Y]ExtractOOSet()[/Y] \\ \index{méthodes!ExtractOOSet} 
Renvoie la liste de toutes les ressources en position d'objet opéré dans l'allocation (classe [C]OperatedObject[/C]). On leur substitue l'entité qui est la valeur du descripteur [C]ResourceEntity[/C] de l'objet opéré.\\

[Y]ExtractOtRscSet()[/Y] \\ \index{méthodes!ExtractOtRscSet} 
Renvoie la liste de toutes les ressources en position de ressource propre de l'opération, dans l'allocation. \\

[Y]ExtractRscSetButOperation()[/Y] \\ \index{méthodes!ExtractRscSetButOperation} 
Renvoie la même liste que [Y]ExtractRscSet[/Y], à l'exception du pointeur sur l'opération en jeu.\\

Un exemple d'exploitation de ces services est le suivant :
[P]
\begin{verbatim}
  ...
  Entity* pInst = ......;  // l'instance de [C]PrimitiveActivity[/C]
  Entity* pV_alloc = pInst->GetEntityConstValue(ALLOCATION_ATTRIBUTE);

  pEntityTab* perfList = pV_alloc->ExtractPerfSet();
  int nbPerf = perfList->size();
  for(int k=0;k<nbPerf;k++) {
    Performer* pPerf = (Performer*)perfList->get_nth(k);
    printf("\npuissance de '%s' = %f", 
	pPerf->InstanceName(), pPerf->GetIntVarValue(POWER));
  }
  ...
\end{verbatim}
[/P]

*/ 
//@{
          ///
class InstructionAllocation : public BasicEntity
{
public:
          // Constructeur de base.
  InstructionAllocation();
          // Constructeur par copie.
  InstructionAllocation(const InstructionAllocation&);
          // Destructeur.
  ~InstructionAllocation();
          // 
  void        DeleteInDepth();

          ///
  pEntityTab* ExtractRscSet();
          ///
  pEntityTab* ExtractPerfSet();
          ///
  pEntityTab* ExtractOOSet();  // + 110217
          ///
  pEntityTab* ExtractOtRscSet();  // + 110217
          ///
  pEntityTab* ExtractRscSetButOperation();
          /// Affiche sur la sortie standard le contenu de l'allocation de ressources.
  void        DisplayRscSet();
          // 
  Boolean IsBetterAllocatedThan(InstructionAllocation*, Entity*, Boolean);

 };

//@}  // fin InstructionAllocation

/////////////////////////////////////////////////////////////////////////////
//
// INSTRUCTION_SET_ALLOCATION
//
/////////////////////////////////////////////////////////////////////////////

/**@name Au plus bas niveau de l'allocation à un jeu d'activités : le CoengagementItem\\

Dans un jeu d'activités, les différentes activités primitives requièrent leurs propres ressources, et une de ces ressources peut être requises par plusieurs activités primitives. Ces ressources sont des instances de différentes sous-classes de [C]Resource[/C].\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]CoengagementItem[/C]} & {\tiny COENGAGEMENT_ITEM} & {\tiny coengagementItem} & {\tiny } & {\tiny }
\end{tabular}

Dans une allocation à un jeu d'activités, la classe [C]CoengagementItem[/C] représente un sous-ensemble des ressources allouées qui sont instances d'une même sous-classe de [C]Resource[/C]. Elle possède à cet effet deux descripteurs :

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny V} & {\tiny int} & {\tiny [C]ResourceId[/C] RESOURCE_ID resourceId} & {\tiny RESOURCE} & {\tiny le symbole de la sous-classe de ressources dont les ressources dans [C]ResourceList[/C] sont instances}\hline
{\tiny C} & {\tiny pEntityTab*} & {\tiny [C]ResourceList[/C] RESOURCE_LIST resourceList} & {\tiny tableau vide} & {\tiny des instances de la sous-classe de symbole [C]ResourceId[/C]}
\end{tabular}

Le code de l'application y accède avec les services standard d'accès aux valeurs de descripteurs. Ce code n'a pas de raison de modifier leurs valeurs (parce que gérées par le moteur de simulation).
*/ 
// @{
          //
class CoengagementItem : public DescriptedEntity
{
public:
          // Constructeur de base.
  CoengagementItem();
          // Constructeur par copie.
  CoengagementItem(const CoengagementItem&);
  /*
          // Destructeur.
  ~CoengagementItem();
  */
 };

// @}  // fin CoengagementItem

//===========================================================================

/**@name Un UsageItem associe des CoengagementItem's à une ressource\\

Dans une structure d'allocation à un jeu d'activités, il y a une instance de la classe [C]UsageItem[/C] par ressource ou opération allouée par au moins une activité du jeu.\\

Cette ressource, ou opération, caractérise l'instance de [C]UsageItem[/C]. Elle est complétée par la liste des autres ressources "co-engagées" avec elle dans le jeu. Ces instances de [C]CoengagementItem[/C] sont les éléments de l'instance de [C]UsageItem[/C].:__

Une telle structure est augmentée par le moteur de simulation lors des allocations successives des activités du jeu. Elle est en même temps exploitée pour détecter des violations de contraintes d'allocation : co-usage interdit de deux ressources, partageabilité interdite d'une ressource au profit de deux opéerations, etc.\\

Selon que c'est une ressource ou oune pération qui caractérise l'instance de [C]UsageItem[/C], le moteur de simulation crée une instance d'une des deux sous-classes [C]ResourceUsageItem[/C] ou [C]OperationUsageItem[/C].

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]UsageItem[/C]} & {\tiny USAGE_ITEM} & {\tiny usageItem} & {\tiny COENGAGEMENT_ITEM} & {\tiny }
\end{tabular}

{\large {\tt ResourceUsageItem}} :

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]ResourceUsageItem[/C]} & {\tiny RESOURCE_USAGE_ITEM} & {\tiny resourceUsageItem} & {\tiny} & {\tiny }
\end{tabular}

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny Entity*} & {\tiny [C]ResourceUsed[/C] RESOURCE_USED resourceUsed} & {\tiny NULL} & {\tiny la ressource qui caractérise le [C]UsageItem[/C]}
\end{tabular}

{\large {\tt OperationUsageItem}} :

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]OperationUsageItem[/C]} & {\tiny OPERATION_USAGE_ITEM} & {\tiny operationUsageItem} & {\tiny} & {\tiny }
\end{tabular}

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny Entity*} & {\tiny [C]OperationUsed[/C] OPERATION_USED operationUsed} & {\tiny NULL} & {\tiny l'opération qui caractérise le [C]UsageItem[/C]}
\end{tabular}


*/ 
//@{
          ///
class UsageItem : public DescriptedEntity
{
public:
          // Constructeur de base.
  UsageItem();
          // Constructeur par copie.
  UsageItem(const UsageItem&);
  /*
          // Destructeur.
  ~UsageItem();
  */
          // 
  void        DeleteInDepth();

          /// Renvoie le [C]GetCoengagementItem[/C] portant sur la ressource ou l'opération dont le symbole est passé en paramètre.\index{méthodes!GetCoengagementItem}
  CoengagementItem* GetCoengagementItem(UserClassId);

          /// Renvoie la liste des symboles de classe qui font l'objet d'un [C]CoengagementItem[/C] dans ce [C]UsageItem[/C].\index{méthodes!GetCoengagementItemIdList}
 TableauInt*        GetCoengagementItemIdList();

 };

//@}  // fin UsageItem

//---------------------------------------------------------------------------

/* *@name ResourceUsageItem ...
*/ 
// @{
          //
class ResourceUsageItem : public UsageItem
{
public:
          // Constructeur de base.
  ResourceUsageItem();
          // Constructeur par copie.
  ResourceUsageItem(const ResourceUsageItem&);
          // Destructeur.
  ~ResourceUsageItem();
 };

// @}  // fin PureResourceUsageItem

//---------------------------------------------------------------------------

/* *@name OperationUsageItem ...
*/ 
// @{
          //
class OperationUsageItem : public UsageItem
{
public:
          // Constructeur de base.
  OperationUsageItem();
          // Constructeur par copie.
  OperationUsageItem(const OperationUsageItem&);
          // Destructeur.
  ~OperationUsageItem();
 };

// @}  // fin OperationUsageItem

//---------------------------------------------------------------------------

/**@name Jeu de ressources allouées à un jeu d'activités\\

C'est la structure nommée U dans l'ontologie. Dans DIESE, c'est une instance de la classe [C]InstructionSetAllocation[/C].\\

Elle identifie, non seulement les ressources et opérations engagées dans les différentes activités du jeu, mais aussi, pour chacune d'elles :
\begin{itemize}
\item les autres classes de ressources et d'opérations "co-engagées"
\item pour chacune de ces autres classes, ses instances "co-engagées", et donc en même temps  et implicitement leur nombre
\end{itemize}

A cet effet, cette classe est caractérisée par une liste d'éléments qui sont des instances de [C]UsageItem[/C].

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]InstructionSetAllocation[/C]} & {\tiny INSTRUCTION_SET_ALLOCATION} & {\tiny instructionSetAllocation} & {\tiny USAGE_ITEM} & {\tiny }
\end{tabular}
*/ 
//@{
          ///
class InstructionSetAllocation : public BasicEntity
{
public:
          // Constructeur de base.
  InstructionSetAllocation();
  InstructionSetAllocation(int);  // 100312 to scratch
  InstructionSetAllocation(int, int);  // 100312 to scratch
          // Constructeur par copie.
  InstructionSetAllocation(const InstructionSetAllocation&);
          // Destructeur.
  ~InstructionSetAllocation();
          // 
  void        DeleteInDepth();

  /// Extrait la liste de toutes les structures [C]CoengagementItem[/C] qui se trouvent dans les différents [C]UsageItem[/C] de la présente allocation. Il n'y a qu'un seul [C]CoengagementItem[/C] portant sur une resource ou une opération donnée {\em obj}. On y rassemble toutes les ressources et l'opération qu'on trouve coengagées avec {\em obj} dans les différents [C]UsageItem[/C].\index{méthodes!ExtractCoengItemGlobalList}
  pEntityTab* ExtractCoengItemGlobalList();

          /// Extrait la liste d'entités (opération ou ressource) d'une structure [C]InstructionSetAllocation[/C] (allocation de resources à un jeu d'activités primitives).\index{méthodes!ExtractRscSet}
  pEntityTab* ExtractRscSet();

          /// Extrait une structure [C]InstructionAllocation[/C] (allocation de resources à une activité primitive {\em act}) d'une structure [C]InstructionSetAllocation[/C] (allocation de ressources à un jeu d'activités primitives), mais supposée construite sur un jeu avec la seule activité {\em act}.\index{méthodes!ExtractVAlloc}
  InstructionAllocation* ExtractVAlloc();

          // Extrait l'{\bf Operation} {\em ot} désignée dans une structure {\bf InstructionSetAllocation} (allocation de resources à un jeu d'activités primitives), mais supposée construite sur un jeu avec la seule activité {\em act} sur laquelle pourrait porter {\em ot}.\index{méthodes!ExtractOperation}
  Operation* ExtractOperation();

          // Augmente une structure U (cf. ontologie) avec l'ensemble de resources qui sont les éléments de l'entité passée en paramètre.\index{méthodes!ReturnExtensionWithComb}
    //InstructionSetAllocation* ReturnExtensionWithComb(AllocatedSet*);
    InstructionSetAllocation* ReturnExtensionWithComb(Entity*); 

          // Augmente cette structure U (cf. ontologie) par une autre. La première est physiquement modifiée. \index{méthodes!ExtendItWithAlloc}
  void ExtendItWithAlloc(InstructionSetAllocation*);

          // Crée une copie de cette structure U (cf. ontologie) et retourne la copie augmentée par une autre. La première n'est pas physiquement modifiée. \index{méthodes!ReturnExtensionWithAlloc}
  //InstructionSetAllocation* ReturnExtensionWithAlloc(const InstructionSetAllocation*);
  InstructionSetAllocation* ReturnExtensionWithAlloc(InstructionSetAllocation*);

          // Renvoie le {\bf UsageItem} portant sur la ressource ou l'opération passée en argument.\index{méthodes!GetUsageItem}
  UsageItem* GetUsageItem(Entity* pObj);

          // \index{méthodes!CheckActivityInconsistencyConditions}
  Boolean CheckActivityInconsistencyConditions(PrimitiveActivity* pAct);

          // \index{méthodes!CheckActivityInconsistencyConditions}
  Boolean CheckResourceSharingViolationConditions(PrimitiveActivity* pAct);
          ///  Affiche sur la sortie standard le contenu de l'allocation de ressources.
  void    SpecialDisplay();
 };

//@}  // fin InstructionSetAllocation

//===========================================================================


//===========================================================================



//@}  // fin Allocations de ressources

#endif  // _C_RESOURCEALLOCATION_HH
