//============================================================================
// FILE: C_ResourceSet.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des ensembles de ressources
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_RESOURCESET_HH
#define _C_RESOURCESET_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "Entity.h"

// EXTERNAL REFERENCES

// /X///////////////X/////////////////////////////X////////////////////////////
//
// ENSEMBLE DE RESSOURCES
//
// /X///////////////X/////////////////////////////X////////////////////////////

/**@name Des ensembles de ressources qui ne sont pas des ressources agrégées\\

On peut regrouper des instances de [C]Resource[/C] dans des entités "conteneurs" qui ne possèdent pas les propriétés et le fonctionnement des ressources (en d'autres termes qui n'héritent pas, directement ou indirectement de la classe [C]Resource[/C]).\\

Entre autres justifications imaginables dans les domaines d'application, les deux suivantes expliquent l'identification de classes prédéfinies dans \CDb :
\begin{itemize}
\item Un pool de ressources est une entité immatérielle qui permet de spécifier un ensemble de références à des resources matérielles présentes dans le système de production. \CD utilise ce concept pour identifier l'ensemble des ressources mobilisables pour l'exécution d'un plan d'activités. Lorsque plusieurs plans sont conduits conjointement, on leur associe des pools complètement disjoints, dans autant d'instances de [C]ActivitiesResourcesBlock[/C].\\

\item Au cours de la procédure d'allocation des ressources à une activité (à l'intérieur d'un jeu), les ressources progressivement allouées alimentent un ensemble attaché à l'activité. Cet ensemble n'a pas à se comporter comme une ressource (en particulier on ne cherche pas à l'allouer).
\end{itemize}

La classe [C]ResourceSet[/C] est donc la classe-mère des "pools de ressources" et les "ensembles de ressources allouées", vers lesquels pointent les deuxième et troisième liens en haut de cette page.
*/
//@{

/**@name Classe de base des ensembles de ressources\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]ResourceSet[/C]} & {\tiny RESOURCE_SET} & {\tiny resourceSet} & {\tiny } & {\tiny }
\end{tabular}

Cette classe ne possède ni descripteur, ni méthode, ni service particulier.\\

Elle distingue collectivement des ressources les ensembles qui ne sont pas des ressources. On peut par exemple rencontrer le passage de code suivant :
[P]
\begin{verbatim}{
  //...
  pEntityTab* pRscTab = MyGetRscTab();
  int laTaille = pRscTab->size();
  for(int k=0;k<laTaille;k++) {
    BasicEntity* pRsc_k = pRscTab->get_nth(k);
	if(pRsc_k->IsInstanceOf(RESOURCE) {
      // ... traitement si 'ressource'
    }
	else if(pRsc_k->IsInstanceOf(RESOURCE_SET) {
      // ... traitement si 'ensemble de ressources'
    }
	else {
      // ... traitement si entité d'un autre type
	}
  }
  //...
}
\end{verbatim}
[/P]
*/ 
//@{
          ///
class ResourceSet : public Entity
{
public:
          // Constructeur de base.
  ResourceSet();
          // Constructeur par copie.
  ResourceSet(const ResourceSet&);
          // Destructeur.
  ~ResourceSet();

};

//@}  // fin classe ResourceSet

// /X///////////////X/////////////////////////////X////////////////////////////
//
// POOLS DE RESSOURCES DU DOMAINE D'APPLICATION
//
// /X///////////////X/////////////////////////////X////////////////////////////

/**@name Les pools de ressources dans le domaine d'application\\

Un pool de ressources est un ensemble d'éléments de deux natures diférentes : 
\begin{itemize}
\item Des instances de [C]Resource[/C] (atomiques et/ou agrégées). Le pool est alors une instance de [C]SimpleResourcePool[/C].\\
Par exemple Jean, Paul, Luc et l'équipe ([C]LaborTeam[/C]), constituée de Jean et Pierre, peuvent être les 4 éléments d'une instance de [C]SimpleResourcePool[/C].

\item Des pools de ressources. Dans ce cas, il existe deux points de vue, deux interprétations de cet ensemble : 
    \begin{itemize}\\
    \item les pools-éléments participent conjointement au même pool englobant, lequel est alors un [C]MultipleResourcePool[/C]. \\
Par exemple, le pool du personnel [C]P1[/C] et le pool du matériel [C]M1[/C] participent conjointement (dans le pool [C]P1M1[/C]) au fonctionnement d'un atelier.\\

    \item les pools-éléments sont des ensembles disjoints affectés à la réalisation de plans d'activités différents. Le pool englobant est alors une [C]ResourcePoolDisjunction[/C].\\
Par exemple les [C]MultipleResourcePool[/C] [C]P1M1[/C] et [C]P2M2[/C], constituent une disjonction et pourront être affectés comme ressources de deux plans différents.
    \end{itemize}
\end{itemize}

Les classes [C]SimpleResourcePool[/C] et [C]MultipleResourcePool[/C] héritent de la classe [C]ResourcePoolConjunction[/C], pour marquer que les éléments interviennent conjointement dans leur rôle de ressource.
*/ 
//@{

/**@name Disjonction de pools de ressources\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]ResourcePoolDisjunction[/C]} & {\tiny RESOURCE_POOL_DISJUNCTION} & {\tiny resourcePoolDisjunction} & {\tiny RESOURCE_POOL_CONJUNCTION} & {\tiny }
\end{tabular}

Une instance de cette classe est l'unique composant du système opérant ([C]OperatingSystem[/C]), lui-même un des composants du système de production.\\

L'installation de ce composant peut être fait dans le code internalisé de l'application, ou bien dans un fichier de données externalisées. Dans ce dernier cas, on pourra écrire :
[P]
\begin{verbatim}
+ I singleWorker, pJean;
+ I singleWorker, pPaul;
+ I singleWorker, pLuc;
+ I singleWorker, pPierre;

+ I singleTool, pTool1;
+ I singleTool, pTool2;

+ I laborTeam pEq1,
    <- E <I><, pJean>;
    <- E <I><, pPierre>;
;
+ I simpleResourcePool pWorkerPool,
    <- E <I><, pJean>;
    <- E <I><, pPaul>;
    <- E <I><, pLuc>;
    <- E <I><, pWorkerPool>;
;
+ I simpleResourcePool pToolPool,
    <- E <I><, pTool1>;
    <- E <I><, pTool2>;
;
+ I multipleResourcePool pPool1
    <- E <I><, pWorkerPool>;
    <- E <I><, pToolPool>;
;
+ I resourcePoolDisjunction lesPools,
    <- E <I><, pPool1>;
; 
+ I operatingSystem pOS
    <- C<I><, lesPools>;
;
+ I productionSystem pSP 
    // ...
    <- C <I><, pOS>;
;
\end{verbatim}
[/P]

*/ 
//@{
          ///
class ResourcePoolDisjunction : public ResourceSet
{
public:
          // Constructeur de base.
  ResourcePoolDisjunction();
          // Constructeur par copie.
  ResourcePoolDisjunction(const ResourcePoolDisjunction&);
          // Destructeur.
  ~ResourcePoolDisjunction();
 };

//@}  // fin ResourcePoolDisjunction

//---------------------------------------------------------------------------

/**@name Conjonction de pools de ressources\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]ResourcePoolConjunction[/C]} & {\tiny RESOURCE_POOL_CONJUNCTION} & {\tiny resourcePoolConjunction} & {\tiny } & {\tiny }
\end{tabular}
 
Cette classe n'est pas destinée à recevoir d'instances directes.\\

On rappelle que ses instances (indirectes) encapsulent des ressources ou des pools de ressources qui interviennent conjointement au service d'un plan d'activités, contrairement aux instances de [C]ResourcePoolDisjunction[/C].
*/ 
//@{
          ///
class ResourcePoolConjunction : public ResourceSet
{
public:
          // Constructeur de base.
  ResourcePoolConjunction();
          // Constructeur par copie.
  ResourcePoolConjunction(const ResourcePoolConjunction&);
          // Destructeur.
  ~ResourcePoolConjunction();
 };

//@}  // fin ResourcePoolConjunction

//---------------------------------------------------------------------------

/**@name Pool simple de ressources atomiques ou agrégées\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]SimpleResourcePool[/C]} & {\tiny SIMPLE_RESOURCE_POOL} & {\tiny simpleResourcePool} & {\tiny RESOURCE} & {\tiny }
\end{tabular}

Une telle entité a vocation à être un élément d'une instance de [C]MultipleResourcePool[/C].
*/ 
//@{
          ///
class SimpleResourcePool : public ResourcePoolConjunction
{
public:
          // Constructeur de base.
  SimpleResourcePool();
          // Constructeur par copie.
  SimpleResourcePool(const SimpleResourcePool&);
          // Destructeur.
  ~SimpleResourcePool();
 };

//@}  // fin SimpleResourcePool

//---------------------------------------------------------------------------

/**@name Pool multiple de ressources atomiques ou agrégées\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]MultipleResourcePool[/C]} & {\tiny MULTIPLE_RESOURCE_POOL} & {\tiny multipleResourcePool} & {\tiny RESOURCE_POOL_CONJUNCTION} & {\tiny }
\end{tabular}

C'est une instance de cette classe qu'on couple à un plan d'activités, comme les deux composants d'une instance de [C]ActivitiesResourcesBlock[/C]. On exprime ainsi que c'est dans ce pool, et lui seulement, que les activités du plan peuvent trouver les ressources requises.\\

Cette structure peut être programmée dans le code internalisé de l'application, ou bien déclarée dans un fichier de données externalisées. Dans ce dernier cas, on pourra écrire :
[P]
\begin{verbatim}
+ I monManager pSD                  // le système décisionnel
    // ...
;
+ I activityConjunction pPlan1      // un plan d'activités
    // ...
;
+ I simpleResourcePool pWorkerPool,
    // ...
;
+ I simpleResourcePool pToolPool,
    // ...
;
+ I multipleResourcePool pPool1     // le pool à coupler avec le plan
    <- E <I><, pWorkerPool>;
    <- E <I><, pToolPool>;
;
+ I activitiesResourcesBlock arb1
	<- C <I><, pPlan1>;
	<- C <I><, pPool1>;             // le couplage avec le plan
;  
+ I activitiesResourcesBlockSet arbSet
	<- E <I><,arb1>;                // un des blocs activités-ressources  ...
;  
+ I strategy maStrategie
	<- C <I><,arbSet>;              // ... constituant la stratégie ...
;
<I><, pSD>
	<- C <I><, maStrategie>;        // ... du système décisionnel
;
\end{verbatim}
[/P]
*/ 
//@{
          ///
class MultipleResourcePool : public ResourcePoolConjunction
{
public:
          // Constructeur de base.
  MultipleResourcePool();
          // Constructeur par copie.
  MultipleResourcePool(const MultipleResourcePool&);
          // Destructeur.
  ~MultipleResourcePool();
 };

//@}  // fin MultipleResourcePool

//@}  // fin Pools dans le domaine d'application

// /X///////////////X/////////////////////////////X////////////////////////////
//
// ENSEMBLE DE RESSOURCES ALLOUEES
//
// /X///////////////X/////////////////////////////X////////////////////////////

/**@name Des classes gérées par le moteur de simulation : les ensembles de ressources allouées

On rappelle ici que c'est exclusivement le moteur de simulation qui crée et gère les structures de données au titre de la procédure d'allocation des ressources aux activités. Les classes ici documentées en font partie, et en conséquence le développeur d'une application n'a pas à en créer d'instances. \\

Cependant, il peut être utile ou nécessaire d'accéder aux valeurs encapsulées par ces structures, car elles sont des photographies, totales ou partielles, de l'état d'allocation des activités.\\

Selon qu'on souhaite accéder aux allocations des "performers", des ressources propres aux opérations ou aux objets opérés par les activités, on récupèrera des instances des sous-classes de [C]AllocatedResourceSet[/C] : respectivement [C]AllocatedResourceSet[/C], [C]AllocatedOperationResourceSet[/C] et  [C]AllocatedOperatedObjectSet[/C]. 
*/ 
//@{

/**@name La classe de base des ensembles de ressources allouées\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]AllocatedResourceSet[/C]} & {\tiny ALLOCATED_RESOURCE_SET} & {\tiny allocatedResourceSet} & {\tiny } & {\tiny }
\end{tabular}

Cette classe n'est pas destinée à recevoir des instances directes.
*/ 
//@{
          ///
class AllocatedResourceSet : public ResourceSet
{
public:
          // Constructeur de base.
  AllocatedResourceSet();
          // Constructeur par copie.
  AllocatedResourceSet(const AllocatedResourceSet&);
          // Destructeur.
  ~AllocatedResourceSet();
 };

//@}  // fin AllocatedResourceSet

//---------------------------------------------------------------------------

/**@name Ensemble de ressources allouées de type 'OperatedObject'\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]AllocatedOperatedObjectSet[/C]} & {\tiny ALLOCATED_OPERATED_OBJECT_SET} & {\tiny allocatedOperatedObjectSet} & {\tiny OPERATED_OBJECT} & {\tiny }
\end{tabular}

Une instance de cette classe est un des composants de la classe [C]InstructionAllocation[/C], dont une instance est attachée à toute activité primitive (voir la page dédiée). On y accède donc par un code tel que le suivant :
[P]
\begin{verbatim}{
  //...
  PrimitiveActivity* pAct = GetNamedEntity("monActivite1");
  InstructionAllocation* pV_alloc = pAct->GetEntityConstValue(ALLOCATION_ATTRIBUTE);
  Entity* pOOSet = pV_alloc->GetComponent(ALLOCATED_OPERATED_OBJECT_SET);
  //...
}
\end{verbatim}
[/P]
*/ 
//@{
          ///
class AllocatedOperatedObjectSet : public AllocatedResourceSet
{
public:
          // Constructeur de base.
  AllocatedOperatedObjectSet();
          // Constructeur par copie.
  AllocatedOperatedObjectSet(const AllocatedOperatedObjectSet&);
          // Destructeur.
  ~AllocatedOperatedObjectSet();
 };

//@}  // fin AllocatedOperatedObjectSet

//---------------------------------------------------------------------------

/**@name Ensemble de ressources allouées de type 'OperatedObject' (avec rôle de destination)\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]AllocatedDestinationObjectSet[/C]} & {\tiny ALLOCATED_DESTINATION_OBJECT_SET} & {\tiny allocatedDestinationObjectSet} & {\tiny OPERATED_OBJECT} & {\tiny }
\end{tabular}

Une instance de cette classe est un des composants de la classe [C]InstructionAllocation[/C], dont une instance est attachée à toute activité primitive (voir la page dédiée). On y accède donc par un code tel que le suivant :
[P]
\begin{verbatim}{
  //...
  TransferActivity* pAct = GetNamedEntity("monActiviteDeTransfert1");
  InstructionAllocation* pV_alloc = pAct->GetEntityConstValue(ALLOCATION_ATTRIBUTE);
  Entity* pOOSet = pV_alloc->GetComponent(ALLOCATED_DESTINATION_OBJECT_SET);
  //...
}
\end{verbatim}
[/P]
*/ 
//@{
          ///
class AllocatedDestinationObjectSet : public AllocatedResourceSet // + JPR 040323
{
public:
          // Constructeur de base.
  AllocatedDestinationObjectSet();
          // Constructeur par copie.
  AllocatedDestinationObjectSet(const AllocatedDestinationObjectSet&);
          // Destructeur.
  ~AllocatedDestinationObjectSet();
 };

//@}  // fin AllocatedDestinationObjectSet

//---------------------------------------------------------------------------

/**@name Ensemble de ressources propres d'opérations allouées\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]AllocatedOperationResourceSet[/C]} & {\tiny ALLOCATED_OPERATION_RESOURCE_SET} & {\tiny allocatedOperationResourceSet} & {\tiny RESOURCE} & {\tiny }
\end{tabular}

Une instance de cette classe est un des composants de la classe [C]InstructionAllocation[/C], dont une instance est attachée à toute activité primitive (voir la page dédiée). On y accède donc par un code tel que le suivant :
[P]
\begin{verbatim}{
  //...
  PrimitiveActivity* pAct = GetNamedEntity("monActivite1");
  InstructionAllocation* pV_alloc = pAct->GetEntityConstValue(ALLOCATION_ATTRIBUTE);
  Entity* pOOSet = pV_alloc->GetComponent(ALLOCATED_OPERATION_RESOURCE_SET);
  //...
}
\end{verbatim}
[/P]
*/ 
//@{
          ///
class AllocatedOperationResourceSet : public AllocatedResourceSet
{
public:
          // Constructeur de base.
  AllocatedOperationResourceSet();
          // Constructeur par copie.
  AllocatedOperationResourceSet(const AllocatedOperationResourceSet&);
          // Destructeur.
  ~AllocatedOperationResourceSet();
 };

//@}  // fin AllocatedOperationResourceSet

//---------------------------------------------------------------------------

/**@name Ensemble de ressources allouées de type 'Performer'\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]AllocatedPerformerSet[/C]} & {\tiny ALLOCATED_PERFORMER_SET} & {\tiny allocatedPerformerSet} & {\tiny RESOURCE} & {\tiny }
\end{tabular}

Une instance de cette classe est un des composants de la classe [C]InstructionAllocation[/C], dont une instance est attachée à toute activité primitive (voir la page dédiée). On y accède donc par un code tel que le suivant :
[P]
\begin{verbatim}{
  //...
  PrimitiveActivity* pAct = GetNamedEntity("monActivite1");
  InstructionAllocation* pV_alloc = pAct->GetEntityConstValue(ALLOCATION_ATTRIBUTE);
  Entity* pOOSet = pV_alloc->GetComponent(ALLOCATED_PERFORMER_SET);
  //...
}
\end{verbatim}
[/P]
*/ 
//@{
          ///
class AllocatedPerformerSet : public AllocatedResourceSet
{
public:
          // Constructeur de base.
  AllocatedPerformerSet();
          // Constructeur par copie.
  AllocatedPerformerSet(const AllocatedPerformerSet&);
          // Destructeur.
  ~AllocatedPerformerSet();
 };

//@}  // fin AllocatedPerformerSet

//---------------------------------------------------------------------------

/* *@name Ensemble de ressources allouées de type 'Operation'\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]AllocatedOperationSet[/C]} & {\tiny ALLOCATED_OPERATION_SET} & {\tiny allocatedOperationSet} & {\tiny OPERATION} & {\tiny }
\end{tabular}
*/ 
// @{
          // /
class AllocatedOperationSet : public Entity 
// 
// !!! et non 'public : AllocatedResourceSet', pour que les éléments  
// !!! puissent être des OPERATION (qui ne sont pas des RESOURCE)
//
{
public:
          // Constructeur de base.
  AllocatedOperationSet();
          // Constructeur par copie.
  AllocatedOperationSet(const AllocatedOperationSet&);
          // Destructeur.
  ~AllocatedOperationSet();
 };

// @}  // fin AllocatedOperationSet

//---------------------------------------------------------------------------


//@}  // fin classes techniques

//@}  // fin Ensemble de ressources


#endif  // _C_RESOURCESET_HH
