//============================================================================
// FILE: CoreContinuousProcess.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des divers descripteurs constants de Diese
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _CORECONTINUOUSPROCESS_HH
#define _CORECONTINUOUSPROCESS_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "ContinuousProcess.h"
#include "File.h"

/**@name CDPFORTEXLe processus continus de lecture d'un fichier de données séquentielles

\label{procLectureSeqDF}Cette sous-classe de processus continu permet d'installer un mécanisme de lecture, enregistrement par enregistrement, d'un fichier de données séquentielles. On rappelle simplement ici que la série d'enregistrements correspond de manière bijective à une série d'intervalles temporels tous de la même longueur (exemple typique : fichier de données climatiques journalières). \\

Cette installation nécessite la déclaration préalable d'une classe de [C]SequentialDataFile[/C] : le processus ne pourra lire que des fichiers physiques qui auront fait l'objet d'une instantiation de cette classe (fichier logique). Le symbole de la sous-classe en question de [C]SequentialDataFile[/C] est une propriété du processus de lecture (accessible par [Y]FileClassId()[/Y]).\\

Sont prédéfinies dans DIESE les méthodes d'initialisation, d'avancée et d'arr\^et du processus. C'est la méthode prédéfinie d'initialisation qui instancie le fichier logique [C]SequentialDataFile[/C] et l'associe au fichier physique voulu. Pour cela, la méthode exploite complémentairement les informations attachées à la sous-classe de [C]SequentialDataFile[/C] et celles attachées au processus de lecture. Les informations sur le fichier sont exploitées en priorité. Elles sont complétées par celles sur le processus. A ce stade seulement, si le nom complet n'a pas été spécifié, il est reconstitué en concaténant le nom du répertoire et le nom simple.\\

L'initialisation prédéfinie ouvre le fichier en lecture et saute un nombre d'enregistrements égal à la valeur renvoyée par la méthode [Y]GetNbTopCommentLines()[/Y].\\

Il est important de noter que le développeur peut remplacer la méthode d'initialisation prédéfinie par une autre, plus adaptée à ses besoins. Cela peut \^etre le cas lorsque on souhaite débuter la lecture après ou avant celui de rang immédiatement supérieur à [Y]GetNbTopCommentLines()[/Y]. Cela peut aussi \^etre le cas lorsque le nom du fichier 
\noLF
 physique est établi selon une règle particulière. Cette situation est rencontrée par exemple lorsque le processus est géré par un événement autogénéré : le fichier à lire peut \^etre différent à chaque lancement du processus et son nom établi selon un algorithmique particulier. Voici le contenu prédéfini de cette méthode, base d'un éventuel surchargement :
\begin{verbatim}
[P]void readSequentialDataFileProcess_initializeProcess_Body(ProcessMethod* pM) {
  ReadSequentialDataFileProcess* pCP 
        = (ReadSequentialDataFileProcess*)(pM->DescribedContinuousProcess());
  UserFileId fileClassId = pCP->FileClassId();
  SequentialDataFile* pF 
        = (SequentialDataFile*)pApp_NewSerialDataFileFromId(fileClassId);
  if(! pF)}
    UnknownClassSymbol("SequentialDataFile", fileClassId); // exit(0)
  pCP->SetFile(pF);
  pCP->SetNamesFromFile(pF);
  pCP->AssignFullNameToFile(pF);
  pF->Open();
  pF->Skip(pF->GetNbTopCommentLines());
}[/P]
\end{verbatim}

La méthode prédéfinie d'arr\^et ne fait que fermer le fichier. Elle peut également \^etre surchargée pour coder des actions particulières lorsque la fin du fichier est rencontrée. Voici son contenu, base d'un éventuel surchargement :
\begin{verbatim}
[P]void readSequentialDataFileProcess_stopProcess_Body(ProcessMethod* pM) {
  ReadSequentialDataFileProcess* pCP 
        = (ReadSequentialDataFileProcess*)(pM->DescribedContinuousProcess());
  SequentialDataFile* pF = pCP->GetFile();
  pF->Close();
  pCP->ToBeStopped(TRUE);
}[/P]
\end{verbatim}

La méthode prédéfinie d'avancée appelle la méthode [Y]ReadNextEnrgt[/Y] sur l'instance de [C]SequentialDataFile[/C], et la méthode d'arr\^et seulement si la fin du fichier est atteinte. Puis, le cas échéant, la méthode qui fait l'objet du paragraphe suivant. \\

Un processus de cette classe est aussi doté d'une méthode d'exploitation des données contenues dans l'enregistrement qui vient d'\^etre lu sur le fichier physique dans le pas courant du processus (et, on le sait, installés dans les "champs" du fichier logique). Cette méthode n'est pas dotée d'un corps prédéfini. Si le développeur veut l'exploiter, il doit obligatoirement définir et écrire la fonction qui deviendra le corps de la méthode (par un appel à [Y]SetPostRead[/Y]). A titre d'exemple, voici un contenu typique de cette fonction :
\begin{verbatim}
[P]void myReadFiles_postReadProcesss_Body(ProcessMethod* pM) {
  ReadSequentialDataFileProcess* pCP 
        = (ReadSequentialDataFileProcess*)pM->DescribedProcess();
  BasicEntity* pE = pCP->ProcessedBasicEntity();
  SequentialDataFile* pF = pCP->GetFile();
  float x = pF->GetFloatFieldValue(FIELD_X);
  // la valeur d'un champ est affectée à un descripteur d'une entité
  pE->SetFloatVarValue(X, x);
  ...
}[/P]
\end{verbatim}

On spécifie le fichier que doit lire le processus de l'une des deux manières suivantes :\\
\begin{itemize}
\item en utilisant les services [Y]SetFilePath[/Y], [Y]SetFileSimpleName[/Y] et [Y]SetFileFullName[/Y] dans le constructeur du processus ;
\item en précisant ces informations dans le fichier externe de la structure du système. 
\end{itemize}

Pour mettre en oeuvre un processus de cette classe, on l'encapsule, comme c'est la règle générale, dans un événement. Tois possibilités sont offertes :\\
\begin{itemize}
\item instancier la classe de base [C]Event[/C] dans le fichier de la structure du système, en ajoutant la date d'occurrence et les bonnes directives (INIT et PROCEED, normalement) sur un processus de lecture préalablement instancié dans le même fichier :
\begin{verbatim}
[P]DailyWeatherFileReading::DailyWeatherFileReading() 
: ReadBufferizedDataFileProcess() {
  ClassSymbol(DAILY_WEATHER_FILE_READING);
  ClassName("dailyWeatherFileReading");
  ProcessedEntityClassId(BD_ENTITY);
  Step(24);
  FileClassId(DAILY_WEATHER_FILE);
  SetInitialize(dailyWeatherFileReading_initializeProcess_Body);
  SetPostRead(dailyWeatherFileReading_postReadProcess_Body);
};

// fichier de la structure du système :
    ...
    + P dailyWeatherFileReading pWR
	    ENTITE = <I><,laMeteo>;
	    REPERTOIRE_FICHIER "/home/durand/CLIMATS/";
	    NOM_SIMPLE_FICHIER "meteo_1982_2003.txt";
    ;
    + V event myEvent
        DATE_OCCUR = 8;
        PRIORITE = 1;
        PROCESS_ACTION = {<P><,pWR> (INIT PROCEED)};
	;[/P]
\end{verbatim}

\item définir dans l'application une sous-classe d'[C]Event[/C] qui réalisera, dès l'instanciation de l'événement, celle du processus de lecture, et qui programmera son initialisation et sa première avancée. Il ne restera plus qu'à préciser la date d'occurrence : soit dans le constructeur, soit dans le fichier de la structure du système ;
\begin{verbatim}
[P]DailyWeatherFileReadingEvent::DailyWeatherFileReadingEvent() {
  ClassName("dailyWeatherFileReadingEvent");
  DiscreteProcess* pDP = new DailyWeatherFileReading();
  AddProcessAction(pDP, INIT);
  AddProcessAction(pDP, PROCEED);
}

// fichier de la structure du système :
    ...
    + V event dailyWeatherFileReadingEvent
        DATE_OCCUR = 8;
        PRIORITE = 1;
	;
    ...[/P]
\end{verbatim}

\item instancier la classe de processus de lecture dans le fichier de la structure du système, et lui passer une directive d'intialisation et de poursuite (INIT_PRCD_EVT)
\begin{verbatim}
[P]    + P dailyWeatherFileReading pWR
	    ENTITE = <I><,laMeteo>;
	    REPERTOIRE_FICHIER "/home/durand/CLIMATS/";
	    NOM_SIMPLE_FICHIER "meteo_1982_2003.txt";
	    INIT_PRCD_EVT DATE_OCCUR = 8; PRIORITE = 1;;
    ;[/P]
\end{verbatim}
\end{itemize}
Dans les trois cas, on peut spécifier le fichier à lire dans le fichier de la structure du système, ou surcharger ces informations si elles ont été placées au niveau du processus ou de la sous-classe de [C]SequentialDataFile[/C].
*/

//@{

/**@name La classe ReadSequentialDataFileProcess
\begin{itemize} \item Directive d'inclusion : {\tt \#include CoreContinuousProcess.h} \end{itemize}
*/
//@{
        ///
class ReadSequentialDataFileProcess : public ContinuousProcess
{
 public:

/**@name LIFECYCLE */
//@{
	/// Constructeur de base. 
  ReadSequentialDataFileProcess();
       // Constructeur de base sans méthode. Appel exclusif dans RawCopy avec l'argument 1. Les méthodes sont affectées dans RawCopy
  //ReadSequentialDataFileProcess(bool); // _no swig_
        /// Constructeur ave affectation du symbole de classe, qui sera celui du fichier lu.
  ReadSequentialDataFileProcess(UserProcessId, char*);                  
          /// Constructeur par copie profonde, y compris des méthodes.  Invoque le constructeur par copie de la classe-mère.
  ReadSequentialDataFileProcess(const ReadSequentialDataFileProcess&); // _no swig_     
                   
          // Constructeur par copie superficielle, c'est-à-dire sans copie des méthodes. La nouvelle instance pointe sur les mêmes méthodes dont les corps codent l'initialisation du processus et son avancée pour une période de un {\em pas}. L'entité sur laquelle le processus copié opère un changement d'état est la m\^eme que pour le processus original.\index{méthodes!RawCopy - ReadSequentialDataFileProcess}    
  //ReadSequentialDataFileProcess* RawCopy(); 

	/// Destructeur. 
  ~ReadSequentialDataFileProcess();
                    
          /// Méthode de destruction. Ne détruit pas l'entité sur laquelle le processus opère des changements d'état. Détruit les instances de [C]Method[/C] attachées au processus (précondition, initialisation, avancée, arrêt).
   void DeleteInDepth(); // _no swig_
//@}

/**@name [SC]Identification et localisation du fichier lu[/SC] */
//@{
	/// Renvoie le symbole de classe du fichier lu par le processus.\index{méthodes!FileClassId}
  UserFileId	FileClassId ();
	/// Affecte le symbole de classe du fichier lu par le processus.
  void   	FileClassId (UserFileId);

	/// Renseigne le répertoire physique du fichier. C'est le chemin d'accès au fichier relativement au répertoire dans lequel la simulation est exécutée. Le client de ce service doit se charger de la désallocation éventuelle de la chaîne passée en argument.\index{méthodes!SetFilePath}
  void		SetFilePath (char*);  

	/// Renseigne le nom physique du fichier. C'est le chemin d'accès au fichier relativement au répertoire dans lequel la simulation est exécutée. Le client de ce service doit se charger de la désallocation éventuelle de la chaîne passée en argument.\index{méthodes!SetFullName}
  void		SetFileFullName (char*);  
  
	/// Renseigne le nom simple du fichier. 
  void		SetFileSimpleName (char*); // _no swig_ 


	//  Récupère le nom du répertoire, le nom simple et/ou le nom complet du fichier attaché, et affecte au présent processus de lecture.
  void          SetNamesFromFile(SequentialDataFile* pF);
  
	// Affecte un nom au fichier associé, à partir du nom du répertoire, du nom simple et/ou du nom complet qui définissent le présent processus de lecture. 
  void          AssignFullNameToFile(SequentialDataFile* pF);   

	// Affiche le nom de répertoire, le nom simple et le nom complet spécifiant le fichier attaché au présent processus. 
  void          DisplayNames () const; // _no swig_

        /// Renvoie le nom physique du répertoire du fichier. 
  char*         GetFilePath () const; // _no swig_ 

        /// Renvoie le nom physique du fichier. 
  char*         GetFileFullName () const; // _no swig_   

	/// Renvoie le nom simple du fichier. 
  char*         GetFileSimpleName () const; // _no swig_  

	// Détermine le fichier (SDF) lu par le processus.
  void          SetFile(SequentialDataFile*); 

	// Renvoie le pointeur sur le fichier (SDF) lu par le processus.
  SequentialDataFile*         GetFile () const;

        /// Renvoie vrai si le répertoire du fichier est spécifié. 
  Boolean         PathSpecified () const; // _no swig_ 
        /// Renvoie vrai si le nom complet du fichier est spécifié. 
  Boolean         FullNameSpecified () const; // _no swig_ 
        /// Renvoie vrai si le nom complet du fichier est spécifié. 
  Boolean         SimpleNameSpecified () const; // _no swig_ 
        /// Renvoie vrai si le nom du fichier est spécifié, soit par le nom complet, soit par le répertoire et le nom simple. 
  Boolean         FileNameSpecified () const; // _no swig_ 


         // Dote le processus d'un corps de méthode qui exploite les données lues dans un enregistrement du fichier. Cette méthode\index{méthodes!AssignPostRead}
   void            SetPostRead(void (* function)(ProcessMethod*)); // _no swig_
          // Appelée par le moteur. Affecte la valeur TRUE lors de l'affectation  d'une méthode (par [Y]SetPostRead[/Y]). La valeur FALSE est attribuée par défaut.
  void          HasPostRead(Boolean);   // _no swig_ 
  Boolean       HasPostRead() const;
//@}

 protected:
  
  UserFileId    mFileClassId;
  char*		    mFilePath;	      // Nom du répertoire (path)
  char*		    mFileSimpleName;  // Nom simple du fichier
  char*		    mFileFullName;	  // Nom complet du fichier (path)
  SequentialDataFile*  mpFile;	  // Pointeur sur l'instance de SDF

  Boolean       mHasPostRead;
};  // _end class_

//@} // fin classe

/**@name Corps de méthodes prédéfinies pour la lecture d'un fichier de données séquentielles

Ces méthodes de processus sont invoquées automatiquement dès lors qu'on a déclaré, et inséré dans l'agenda, un processus de la classe [C]ReadSequentialDataFileProcess[/C].\\

Au niveau d'une sous-classe de [C]ReadSequentialDataFileProcess[/C], le modélisateur peut vouloir surcharger leurs corps. On indique donc ici la signature au niveau de [C]ReadSequentialDataFileProcess[/C], qui doit être reprise pour la sous-classe (avec un préfixe reprenant le nom de cette sous-classe). L'affectation de ces corps aux méthodes se fait par un appel à SetInitialize et/ou SetGoOneStepForward dans le constructeur de la sous-classe.\\

Si ces méthodes ne recoivent pas un nouveau corps par surchargement, c'est la forme prédéfinie dans le moteur qui est activée.\\

On rappelle ici que ce processus de lecture possède aussi une autre méthode, [Y]PostRead[/Y], dont aucune forme standard de corps n'est incluse dans le moteur. Il revient donc au modélisateur de coder ce corps (sur la base des explications dans la documentation du processus de lecture), et d'affecter ce corps par un appel à [Y]SetPostRead[/Y].
*/
//@{
         /// Initialisation de la lecture du fichier. Récupère les noms du fichier traité par le processus, et en établit le chemin complet. Puis ouvre le fichier et saute éventuellement le nombre approprié de lignes en début du fichier.
void readSequentialDataFileProcess_initializeProcess_Body(ProcessMethod*); // _no swig_
         /// Lecture du premier enregistrement non lu du fichier. Invoque le corps d'une éventuelle méthode [Y]PostRead[/Y].
void readSequentialDataFileProcess_goOneStepForwardProcess_Body(ProcessMethod*);

//
void readSequentialDataFileProcess_stopProcess_Body(ProcessMethod*); // _no swig_

//@} // fin corps de méthodes

//@} // fin processus continu de lecture


//@}  // ouverture dans Process.h

#endif  // _CORECONTINUOUSPROCESS_HH
