//============================================================================
// FILE: DescriptedEntity.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe générique des entités
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _DESCRIPTEDENTITY_HH
#define _DESCRIPTEDENTITY_HH

// SYSTEM INCLUDES
//
//#include <>                                   //

// PROJECT INCLUDES
//
#include "BasicEntity.h"
#include "Descriptor.h"

/**@name CDPFORTEXLa classe des entit\'es avec descripteurs
\begin{itemize} \item Directive d'inclusion :
[P]
\#include DescriptedEntity.h
[/P]
\end{itemize}
Les classes héritant de la classe de base [C]DescriptedEntity[/C] sont équipées d'attributs et de services liés aux variables d'état et aux propriétés constantes des entités. Si une classe d'entité dans le domaine n'est pas caractérisée par des variables d'état ou des propriétés constantes, il suffit d'en faire une spécialisation de [C]BasicEntity[/C].\\

Cette classe est donc la classe mère appropriée pour les classes d'entités du domaine qui ne sont pas caractérisées par des descripteurs fonctionnels (méthodes). Les instances créées possèdent, outre les services hérités de la classe [C]BasicEntity[/C], tous les services nécessaires à la gestion des descripteurs, et seulement eux dans un souci d'économie d'espace mémoire.
*/
//@{ 
          /// 
class DescriptedEntity : public BasicEntity
{
public:

  /**@name LIFECYCLE */
//@{
          /// Constructeur de base, invoquant d'abord le constructeur de base de [C]BasicEntity[/C]. L'instance créée possède le type BD_DESCRIPTED_ENTITY, valeur renvoyée par [Y]EntityType[/Y] et qui surcharge celle donnée par le constructeur de [C]BasicEntity[/C].
   DescriptedEntity(); // _no swig_

          /// Constructeur par copie profonde, invoquant d'abord le constructeur par copie de [C]BasicEntity[/C]. En ce qui concerne les descripteurs, chacun est copié avec son constructeur par copie, puis la-dite copie se voit affecter l'entité copiée comme entité décrite. Ajoute l'entité en cours de création par copie à la liste des instances (indirectes) de [C]DescriptedEntity[/C].
   DescriptedEntity(const DescriptedEntity&); // _no swig_  
   DescriptedEntity(const DescriptedEntity&, bool triggerMonitor); // _no swig_   
          /// Même effet que le constructeur ci-dessus, mais les éléments ou les composants ne sont pas nécessairement copiés. Si l'argument 'partsCopied' est FALSE, ce rôle est laissé aux constructeurs par copie des sous-classes de l'application. 
   DescriptedEntity(const DescriptedEntity&, int partsCopied); // _no swig_  
   DescriptedEntity(const DescriptedEntity&, bool triggerMonitor, int partsCopied); // _no swig_
  
          /// Destructeur superficiel, invoquant d'abord le destructeur superficiel de [C]BasicEntity[/C]. Les descripteurs sont détruits. Lorsque l'entité détruite possède des descripteurs dont les valeurs sont des entités, ces entités ne sont pas détruites.
   ~DescriptedEntity(); // _no swig_
//@}                             

/**@name OPERATORS */
//@{
          /// Opérateur de infériorité exploité par les fonctions <Y>SortDown</Y> et <Y> SortUp</Y>. Renvoie {\em vrai} si et seulement si l'entité à gauche de l'opérateur est inférieure à l'entité à droite. En considérant le descripteur {\em d} dont le symbole de classe est passé en argument des fonctions de tri, il y a infériorité si la valeur de {\em d} est plus petite pour l'entité à gauche que pour l'entité à droite.  
  Boolean         operator < (DescriptedEntity&);

          /// Opérateur de supériorité exploité par les fonctions <Y>SortDown</Y> et <Y> SortUp</Y>.. Renvoie {\em vrai} si et seulement si l'entité à gauche de l'opérateur est supérieure ou égale à l'entité à droite. En considérant le descripteur {\em d} dont le symbole de classe est passé en argument des fonctions de tri, il y a supériorité si la valeur de {\em d} est plus grande pour l'entité à gauche que pour l'entité à droite ou si ls deux valeurs sont égales. 
  Boolean         operator > (DescriptedEntity&);
//@}                             

/**@name [SC]Acc\`es \`a la place dans la hi\'erarchie de classes[/SC] */
//@{ 

         /// Invoque [Y]AddInstanceToClass[/Y] de [C]BasicEntity[/C] puis équipe les descripteurs en fonction d'éventuelles spécifications de sortie. Cette méthode doit obligatoirement être invoquée dans le constructeur de chaque sous-classe utilisateur des entités avec descripteurs. A utiliser seulement si on a déjà déclaré le symbole de classe et le nom de classe.
void               AddInstanceToClass(); // _no swig_

          // Service 'moteur'. 
void               AddInstanceToClass(Boolean noEquip); // _no swig_
//@}

/**@name [SC]Opérations sur les listes de descripteurs[/SC] */
//@{        
          // Service 'moteur'.  
   void            AddDTS2EntityInstOSIfNew(Descriptor* pD);

         /// Ajoute le descripteur à la liste des descripteurs constants du récepteur, s'il n'y est pas déjà. Lève l'exception [Y]BadDescriptorTypeWhenAdd[/Y].\index{méthodes!AddConstantDescriptor} 
   void            AddConstantDescriptor(Descriptor*); // _no swig_

         // Ajoute le descripteur à la liste des descripteurs constants du récepteur, s'il n'y est pas déjà, et indique s'il est privé ou non. Lève l'exception [Y]BadDescriptorTypeWhenAdd[/Y].
   void            AddConstantDescriptor(Descriptor*, Boolean);

         /// Ajoute le descripteur à la liste des descripteurs variables du récepteur, s'il n'y est pas déjà. Lève l'exception [Y]BadDescriptorTypeWhenAdd[/Y].\index{méthodes!AddVariableDescriptor}
   void            AddVariableDescriptor(Descriptor*); // _no swig_

         // Ajoute le descripteur à la liste des descripteurs variables du récepteur, s'il n'y est pas déjà, et indique s'il est privé ou non. Lève l'exception [Y]BadDescriptorTypeWhenAdd[/Y].
   void            AddVariableDescriptor(Descriptor*, Boolean);

         /// Renvoie la position du descripteur constant dont le symbole de classe est passé en argument, ou bien -9 s'il n'existe pas.\index{méthodes!GetConstantDescriptorPosition}
   int   GetConstantDescriptorPosition(UserDescriptorId) const;

         /// Renvoie le descripteur constant dont le symbole de classe est passé en argument. Lève l'exception [Y]InexistantConstantDescriptor[/Y].\index{méthodes!GetConstantDescriptor}
   ConstantDescriptor* GetConstantDescriptor(UserDescriptorId) const;

         /// Renvoie la position du descripteur variable dont le symbole de classe est passé en argument, ou bien -9 s'il n'existe pas.\index{méthodes!GetVariableDescriptorPosition}
   int   GetVariableDescriptorPosition(UserDescriptorId) const;

         /// Renvoie le descripteur variable dont le symbole de classe est passé en argument. Lève l'exception [Y]InexistantVariableDescriptor[/Y].\index{méthodes!GetVariableDescriptor}
   VariableDescriptor* GetVariableDescriptor(UserDescriptorId) const;

         /// Renvoie le descripteur variable ou constant dont le symbole de classe est passé en argument. Lève l'exception [Y]InexistantDescriptor[/Y].\index{méthodes!GetDescriptor}
   Descriptor*         GetDescriptor(UserDescriptorId) const; 

         /// Renvoie le descripteur variable ou constant dont le nom de classe est passé en argument. Lève l'exception [Y]InexistantDescriptor[/Y].
   Descriptor*         GetDescriptor(char*) const;  // _no swig_

         /// Renvoie la table des descripteurs constants.
   pConstantDescriptorTab ConstantDescriptorTab() const; // _no swig_

         /// Renvoie la table des descripteurs variables.
   pVariableDescriptorTab VariableDescriptorTab() const;  // _no swig_

//@}

/**@name [SC]Affectation et accès aux valeurs des descripteurs[/SC] */ 
//@{
         /// Renvoie {\em vrai} si la valeur courante du descripteur de classe {\em id} est sa valeur par défaut et {\em faux} sinon.\index{méthodes!DefaultValueIsAssigned}
   Boolean         DefaultValueIsAssigned(UserDescriptorId id) const;

  /**@name [SC]Affectation et acc\`es standard[/SC]
Les méthodes [Y]SetValue[/Y]\index{méthodes!SetValue - DescriptedEntity} affectent une valeur (entière, flottante, flottante double précision, entité ou chaîne de caractères) au descripteur dont le symbole de classe est donné en premier argument. La méthode [Y]SetStringValue[/Y] fait préalablement une copie de la chaîne passée en argument.\\
Les méthodes [Y]Get..Value[/Y]\index{méthodes!Get..Value - DescriptedEntity} renvoient la valeur du descripteur dont le symbole de classe est donné en premier argument.\\
Les deux groupes de méthodes lèvent les exceptions [Y]BadDescriptorValueType[/Y] et [Y]InexistantDescriptor[/Y], ainsi que les exceptions [Y]OutOf..Domain[/Y]. Si un démon est attaché au descripteur, il sera déclenché, sauf si un argument de type {\em Boolean} est fourni et que sa valeur est FALSE.\\
Le premier argument de toutes les fonctions doit être de type {\em int} ({\em UserDescriptorId} est un alias de {\em int}). En général, l'utilisateur établiera une correspondance entre ces valeurs entières et des symboles évocateurs dans son domaine d'application. Cette correspondance sera typiquement réalisée dans le fichier [F]UserTypes.h[/F], par des déclarations de types énumérés : [P]enum <type> {symb1, ..., symbn};[/P]. Voir page 'Les types d'identificateurs de classes'.\\

Le service [Y]DepthDeleteStringTabValue[/Y] récupère la mémoire allouée au tableau de chaînes qui est la valeur du descripteur dont le symbole est passé en argument, ainsi que la mémoire allouée aux chaînes éléments du tableau.\index{méthodes!DepthDeleteStringTabValue - DescriptedEntity}
*/
  //@{
         /// 
   void            SetIntValue(UserDescriptorId, int) ; // _no swig_
         /// 
   void            SetIntValue(UserDescriptorId, int, Boolean);
         /// 
   void            SetFloatValue(UserDescriptorId, float); // _no swig_
         /// 
   void            SetFloatValue(UserDescriptorId, float, Boolean);  
         /// 
   void            SetDoubleValue(UserDescriptorId, double); // _no swig_
         /// 
   void            SetDoubleValue(UserDescriptorId, double, Boolean);  
         /// 
   void            SetEntityValue(UserDescriptorId, BasicEntity*); // _no swig_
         /// 
   void            SetEntityValue(UserDescriptorId, BasicEntity*, Boolean);
         /// 
   void            SetEntitySpecValue(UserDescriptorId, EntitySpec*); 
         /// 
   void            SetStringValue(UserDescriptorId, char*); // _no swig_
         /// 
   void            SetStringValue(UserDescriptorId, char*, Boolean); 
         /// 
   void            SetIntDomainValue(UserDescriptorId, pIntIntervalTab*); // _no swig_
         /// 
   void            SetIntDomainValue(UserDescriptorId, pIntIntervalTab*, Boolean);   
         /// 
   void            SetFloatDomainValue(UserDescriptorId, pFloatIntervalTab*); // _no swig_ 
         /// 
   void            SetFloatDomainValue(UserDescriptorId, pFloatIntervalTab*, Boolean);   
         /// 
   void            SetDoubleDomainValue(UserDescriptorId, pDoubleIntervalTab*); // _no swig_
         /// 
   void            SetDoubleDomainValue(UserDescriptorId, pDoubleIntervalTab*, Boolean);   
         /// 
   void            SetStringTabValue(UserDescriptorId, TableauString*); // _no swig_  
         /// 
   void            SetStringTabValue(UserDescriptorId, TableauString*, Boolean);   
         /// 
   void            ClearEntityTabValues(UserDescriptorId); // _no swig_
         /// 
   void            SetEntityTabValue(UserDescriptorId, pEntityTab*); // _no swig_  
         /// 
   void            SetEntityTabValue(UserDescriptorId, pEntityTab*, Boolean); 
         /// 
   void            ClearDescValueSpecTabValues(UserDescriptorId); // _no swig_
         /// 
   void            SetDescValueSpecTabValue(UserDescriptorId, pDescValueSpecTab*); // _no swig_  
         /// 
   void            SetDescValueSpecTabValue(UserDescriptorId, pDescValueSpecTab*, Boolean);
         /// 
   void            SetIntTabValue(UserDescriptorId, TableauInt*); // _no swig_
         /// 
   void            SetIntValueElement(UserDescriptorId, int, int); // _no swig_
         /// 
   void            ClearFloatTabValues(UserDescriptorId); // _no swig_
         /// 
   void            SetFloatTabValue(UserDescriptorId, TableauFloat*); // _no swig_
         /// 
   void            SetFloatValueElement(UserDescriptorId, int, float); // _no swig_
         /// 
   void            SetDoubleTabValue(UserDescriptorId, TableauDouble*); // _no swig_
         /// 
   void            SetDoubleValueElement(UserDescriptorId, int, double); // _no swig_
    
         /// 
   int             GetIntValue(UserDescriptorId); // _no swig_   
         /// 
   int             GetIntValue(UserDescriptorId, Boolean);
         /// 
   float           GetFloatValue(UserDescriptorId); // _no swig_
         /// 
   float           GetFloatValue(UserDescriptorId, Boolean);
         /// 
   double          GetDoubleValue(UserDescriptorId); // _no swig_
         /// 
   double          GetDoubleValue(UserDescriptorId, Boolean);
         /// 
   BasicEntity*    GetBasicEntityValue(UserDescriptorId); // _no swig_
         /// 
   Entity*         GetEntityValue(UserDescriptorId); // _no swig_
         /// 
   BasicEntity*    GetBasicEntityValue(UserDescriptorId, Boolean);
         /// 
   Entity*         GetEntityValue(UserDescriptorId, Boolean);
         /// 
   char*           GetStringValue(UserDescriptorId); // _no swig_
         /// 
   char*           GetStringValue(UserDescriptorId, Boolean); 
         /// 
   EntitySpec*     GetEntitySpecValue(UserDescriptorId);   
         /// 
   pIntIntervalTab* GetIntDomainValue(UserDescriptorId); // _no swig_ 
         /// 
   pIntIntervalTab* GetIntDomainValue(UserDescriptorId, Boolean);       
         /// 
   IntInterval*     GetIntDomainValueElement(UserDescriptorId, int); // _no swig_    
         /// 
   IntInterval*     GetIntDomainValueElement(UserDescriptorId, int, Boolean);
         /// 
   pFloatIntervalTab* GetFloatDomainValue(UserDescriptorId); // _no swig_ 
         /// 
   pFloatIntervalTab* GetFloatDomainValue(UserDescriptorId, Boolean);       
         /// 
   FloatInterval*     GetFloatDomainValueElement(UserDescriptorId, int); // _no swig_    
         /// 
   FloatInterval*     GetFloatDomainValueElement(UserDescriptorId, int, Boolean);
         /// 
   pDoubleIntervalTab* GetDoubleDomainValue(UserDescriptorId); // _no swig_ 
         /// 
   pDoubleIntervalTab* GetDoubleDomainValue(UserDescriptorId, Boolean);       
         /// 
   DoubleInterval*     GetDoubleDomainValueElement(UserDescriptorId, int); // _no swig_    
         /// 
   DoubleInterval*     GetDoubleDomainValueElement(UserDescriptorId, int, Boolean);
         /// 
   TableauString* GetStringTabValue(UserDescriptorId); // _no swig_ 
         /// 
   TableauString* GetStringTabValue(UserDescriptorId, Boolean);
         /// 
   void DepthDeleteStringTabValue(UserDescriptorId); // _no swig_// + JPR 120216
         /// 
   pEntityTab* GetEntityTabValue(UserDescriptorId); // _no swig_ 
         /// 
   pEntityTab* GetEntityTabValue(UserDescriptorId, Boolean);
         /// 
   pDescValueSpecTab* GetDescValueSpecTabValue(UserDescriptorId); // _no swig_ 
         /// 
   pDescValueSpecTab* GetDescValueSpecTabValue(UserDescriptorId, Boolean);
         /// 
   TableauInt*      GetIntTabValue(UserDescriptorId); // _no swig_
         /// 
   TableauFloat*      GetFloatTabValue(UserDescriptorId); // _no swig_
         /// 
   TableauDouble*      GetDoubleTabValue(UserDescriptorId); // _no swig_
         /// 
   int      GetIntValueElement(UserDescriptorId, int); // _no swig_
         /// 
   float      GetFloatValueElement(UserDescriptorId, int); // _no swig_
         /// 
   double      GetDoubleValueElement(UserDescriptorId, int); // _no swig_
  //@}

  /**@name [SC]Affectation et accès sans vérification de la classe[/SC]
Les méthodes [Y]SetVarValue[/Y] et [Y]SetConstValue[/Y] ne diffèrent des méthodes [Y]SetValue[/Y] que par le fait qu'elles portent sur des descripteurs dont on sait qu'ils existent et qu'ils sont, respectivement variables ou constants : elles ne cherchent donc pas à lever l'exception [Y]InexistantDescriptor[/Y].\\
Il en est de même pour les méthodes [Y]Get..VarValue[/Y]\index{méthodes!Get..VarValue - DescriptedEntity} et [Y]Get..ConstValue[/Y]\index{méthodes!Get..ConstValue - DescriptedEntity}.
*/     
  //@{
         /// 
   void            SetIntVarValue(UserDescriptorId, int); // _no swig_
         /// 
   void            SetIntVarValue(UserDescriptorId, int, Boolean);
         /// 
   void            SetFloatVarValue(UserDescriptorId, float); // _no swig_
         /// 
   void            SetFloatVarValue(UserDescriptorId, float, Boolean);

   void            SetDoubleVarValue(UserDescriptorId, double); // _no swig_
         /// 
   void            SetDoubleVarValue(UserDescriptorId, double, Boolean);  
         /// 
   void            SetEntityVarValue(UserDescriptorId, BasicEntity*); // _no swig_
         /// 
   void            SetEntityVarValue(UserDescriptorId, BasicEntity*, Boolean);
         /// 
   void            SetStringVarValue(UserDescriptorId, char*); // _no swig_
         /// 
   void            SetStringVarValue(UserDescriptorId, char*, Boolean);
         /// 
   void            SetEntitySpecVarValue(UserDescriptorId, EntitySpec*);
         /// 
   void            SetIntDomainVarValue(UserDescriptorId, pIntIntervalTab*); // _no swig_
         /// 
   void            SetIntDomainVarValue(UserDescriptorId, pIntIntervalTab*, Boolean);
         /// 
   void            SetFloatDomainVarValue(UserDescriptorId, pFloatIntervalTab*); // _no swig_
         /// 
   void            SetFloatDomainVarValue(UserDescriptorId, pFloatIntervalTab*, Boolean);
         /// 
   void            SetDoubleDomainVarValue(UserDescriptorId, pDoubleIntervalTab*); // _no swig_
         /// 
   void            SetDoubleDomainVarValue(UserDescriptorId, pDoubleIntervalTab*, Boolean);
         /// 
   void            SetStringTabVarValue(UserDescriptorId, TableauString*); // _no swig_
         /// 
   void            SetStringTabVarValue(UserDescriptorId, TableauString*, Boolean); 
         /// 
   void            SetEntityTabVarValue(UserDescriptorId, pEntityTab*); // _no swig_
         /// 
   void            SetEntityTabVarValue(UserDescriptorId, pEntityTab*, Boolean); 
         /// 
   void            SetDescValueSpecTabVarValue(UserDescriptorId, pDescValueSpecTab*); // _no swig_
         /// 
   void            SetDescValueSpecTabVarValue(UserDescriptorId, pDescValueSpecTab*, Boolean);

         /// 
   void            SetIntConstValue(UserDescriptorId, int);
         /// 
   void            SetFloatConstValue(UserDescriptorId, float);
         /// 
   void            SetDoubleConstValue(UserDescriptorId, double); 
         /// 
   void            SetEntityConstValue(UserDescriptorId, BasicEntity*);
         /// 
   void            SetStringConstValue(UserDescriptorId, char*);
         /// 
   void            SetEntitySpecConstValue(UserDescriptorId, EntitySpec*);
         /// 
   void            SetIntDomainConstValue(UserDescriptorId, pIntIntervalTab*);
         /// 
   void            SetFloatDomainConstValue(UserDescriptorId, pFloatIntervalTab*);
         /// 
   void            SetDoubleDomainConstValue(UserDescriptorId, pDoubleIntervalTab*);
         /// 
   void            SetStringTabConstValue(UserDescriptorId, TableauString*);
         /// 
   void            SetEntityTabConstValue(UserDescriptorId, pEntityTab*);
         /// 
   void            SetDescValueSpecTabConstValue(UserDescriptorId, pDescValueSpecTab*);
         /// 
   void            SetIntTabConstValue(UserDescriptorId, TableauInt*);
         /// 
   void            SetIntConstValueElement(UserDescriptorId, int, int);
         /// 
   void            SetFloatTabConstValue(UserDescriptorId, TableauFloat*);
         /// 
   void            SetFloatConstValueElement(UserDescriptorId, int, float);
         /// 
   void            SetDoubleTabConstValue(UserDescriptorId, TableauDouble*);
         /// 
   void            SetDoubleConstValueElement(UserDescriptorId, int, double);






         /// 
   int             GetIntVarValue(UserDescriptorId); // _no swig_
         /// 
   int             GetIntVarValue(UserDescriptorId, Boolean);
         /// 
   float           GetFloatVarValue(UserDescriptorId); // _no swig_
         /// 
   float           GetFloatVarValue(UserDescriptorId, Boolean);
         /// 
   double          GetDoubleVarValue(UserDescriptorId); // _no swig_
         /// 
   double          GetDoubleVarValue(UserDescriptorId, Boolean);
         /// 
   BasicEntity*    GetBasicEntityVarValue(UserDescriptorId); // _no swig_
         /// 
   Entity*         GetEntityVarValue(UserDescriptorId); // _no swig_
         /// 
   BasicEntity*    GetBasicEntityVarValue(UserDescriptorId, Boolean);
         /// 
   Entity*         GetEntityVarValue(UserDescriptorId, Boolean);
         /// 
   char*           GetStringVarValue(UserDescriptorId); // _no swig_
         /// 
   char*           GetStringVarValue(UserDescriptorId, Boolean);
         /// 
   EntitySpec*     GetEntitySpecVarValue(UserDescriptorId);
         /// 
   pIntIntervalTab* GetIntDomainVarValue(UserDescriptorId); // _no swig_
         /// 
   pIntIntervalTab* GetIntDomainVarValue(UserDescriptorId, Boolean);
         /// 
   pFloatIntervalTab* GetFloatDomainVarValue(UserDescriptorId); // _no swig_
         /// 
   pFloatIntervalTab* GetFloatDomainVarValue(UserDescriptorId, Boolean);
         /// 
   pDoubleIntervalTab* GetDoubleDomainVarValue(UserDescriptorId); // _no swig_
         /// 
   pDoubleIntervalTab* GetDoubleDomainVarValue(UserDescriptorId, Boolean);
         /// 
   TableauString*     GetStringTabVarValue(UserDescriptorId); // _no swig_
         /// 
   TableauString*     GetStringTabVarValue(UserDescriptorId, Boolean);
         /// 
   pEntityTab*     GetEntityTabVarValue(UserDescriptorId); // _no swig_
         /// 
   pEntityTab*     GetEntityTabVarValue(UserDescriptorId, Boolean);
         /// 
   pDescValueSpecTab*     GetDescValueSpecTabVarValue(UserDescriptorId); // _no swig_
         /// 
   pDescValueSpecTab*     GetDescValueSpecTabVarValue(UserDescriptorId, Boolean);



    
         /// 
   int             GetIntConstValue(UserDescriptorId) const;
         /// 
   float           GetFloatConstValue(UserDescriptorId) const;
         /// 
   double          GetDoubleConstValue(UserDescriptorId) const;
         /// 
   BasicEntity*    GetBasicEntityConstValue(UserDescriptorId) const;
         /// 
   Entity*         GetEntityConstValue(UserDescriptorId) const;
         /// 
   char*           GetStringConstValue(UserDescriptorId) const;
         /// 
   EntitySpec*     GetEntitySpecConstValue(UserDescriptorId) const;
         /// 
   pIntIntervalTab* GetIntDomainConstValue(UserDescriptorId) const;
         /// 
   pFloatIntervalTab* GetFloatDomainConstValue(UserDescriptorId) const;
         /// 
   pDoubleIntervalTab* GetDoubleDomainConstValue(UserDescriptorId) const;
         /// 
   TableauString*     GetStringTabConstValue(UserDescriptorId) const;
         /// 
   pEntityTab*     GetEntityTabConstValue(UserDescriptorId) const;
         /// 
   pDescValueSpecTab* GetDescValueSpecTabConstValue(UserDescriptorId) const;
         /// 
   TableauInt*      GetIntTabConstValue(UserDescriptorId) const;
         /// 
   TableauFloat*      GetFloatTabConstValue(UserDescriptorId) const;
         /// 
   TableauDouble*      GetDoubleTabConstValue(UserDescriptorId) const;
         /// 
   int              GetIntConstValueElement(UserDescriptorId, int) const;
         /// 
   float              GetFloatConstValueElement(UserDescriptorId, int) const;
         /// 
   double              GetDoubleConstValueElement(UserDescriptorId, int) const;
  //@}

  /**@name [SC]Affectation avec ajustement du type[/SC]
Les méthodes [Y]SetCastValue[/Y]\index{méthodes!SetCastValue - DescriptedEntity} ne diffèrent des méthodes [Y]SetValue[/Y] qu'en ce qu'elles tolèrent, dans une certaine limite, que la valeur passée en argument ne soit pas du même type que le descripteur auquel elle est affectée. Un argument de type {\em int} pourra être affecté (après un {\em cast}) à un descripteur entier, flottant ou double. Un argument de type {\em float} ou {\em double} pourra être affecté (après un {\em cast}) à un descripteur flottant ou double. Cette méthode ne peut accepter aucune autre différence de type : par exemple un argument de type {\em float} ne peut pas être affecté à un descripteur entier.
*/ // ci dessous JPR 990427 pour le parsing des fichiers de données
  //@{
         /// 
   void            SetIntCastValue(UserDescriptorId, int);
   /// 
   void            SetFloatCastValue(UserDescriptorId, float);
         /// 
   void            SetDoubleCastValue(UserDescriptorId, double);
  //@}

  /**@name [SC]Incrémentation[/SC]
Les méthodes [Y]IncrValue[/Y]\index{méthodes!IncrValue - DescriptedEntity} et [Y]DecrValue[/Y]\index{méthodes!DecrValue - DescriptedEntity} augmentent de 1 et diminue de 1 (respectivement) la valeur d'un descripteur variable dont le type doit être entier. \\ 

Les méthodes [Y]IncreaseIntValue[/Y]\index{méthodes!IncreaseIntValue - DescriptedEntity} et [Y]DecreaseIntValue[/Y]\index{méthodes!DecreaseIntValue - DescriptedEntity} augmentent de et diminue (respectivement) d'une valeur passée en paramètre la valeur d'un descripteur variable dont le type doit être entier. \\ 

Ces groupes de méthodes ne lèvent que l'exception [Y]BadDescriptorValueType[/Y]. Si un démon est attaché au descripteur, il sera déclenché, sauf si un argument de type {\em Boolean} est fourni et que sa valeur est FALSE.\\
*/
  //@{
         /// 
   void            IncrValue(UserDescriptorId); // _no swig_
         /// 
   void            IncrValue(UserDescriptorId, Boolean);
         /// 
   void            DecrValue(UserDescriptorId); // _no swig_
         /// 
   void            DecrValue(UserDescriptorId, Boolean);
         /// 
   void            IncreaseIntValue(UserDescriptorId, int); // _no swig_
         /// 
   void            IncreaseIntValue(UserDescriptorId, int, Boolean);
         /// 
   void            DecreaseIntValue(UserDescriptorId, int); // _no swig_
         /// 
   void            DecreaseIntValue(UserDescriptorId, int, Boolean);
  //@}

  /**@name [SC]Affectation de valeur partielle[/SC]
Les méthodes [Y]AddValueElement[/Y]\index{méthodes!Add..ValueElement - DescriptedEntity} complètent une valeur de descripteur, lorsque celle-ci est un ensemble de plusieurs éléments. C'est le cas des descripteurs dont la valeur est un tableau d'entités, de chaînes de caractères, un tableau d'intervalles d'entiers ou un tableau de flottants. 
*/
  //@{

         ///Ajoute l'intervalle {\em [inf, sup]} en fin du tableau qui est la valeur du descripteur dont le symbole est passé en premier argument. {\em inf} doit être supérieur à la borne supérieure du dernier intervalle déjà ajouté.
   void            AddIntDomainValueElement(UserDescriptorId, int inf, int sup);
         ///
   void            AddFloatDomainValueElement(UserDescriptorId, float inf, float sup);
         ///
   void            AddDoubleDomainValueElement(UserDescriptorId, double inf, double sup);
         /// Le moteur désallouera la chaîne str par [Y]free[/Y]. L'utilisateur doit donc l'allouer par [Y]strdup[/Y].
   void            AddStringValueElement(UserDescriptorId, char* str);
         /// Ajoute l'entité {\em pObj} en fin du tableau qui est la valeur du descripteur dont le symbole est passé en premier argument.
   void            AddEntityValueElement(UserDescriptorId, BasicEntity* pObj); // _no swig_
         /// 
   void            AddEntityValueElement(UserDescriptorId, BasicEntity* pObj, Boolean);
         /// 
void               RemoveEntityValueElement(UserDescriptorId descId, BasicEntity* pObj);
         /// Ajoute une à une les entités du tableau fourni en second argument en fin du tableau qui est la valeur du descripteur dont le symbole est passé en premier argument.
   void            AddEntityValueElements(UserDescriptorId, pEntityTab* pObj); // _no swig_
         /// Ajoute la spécification de domaine de valeur {\em pDVS} en fin du tableau qui est la valeur du descripteur dont le symbole est passé en premier argument.
   void            AddDescValueSpecValueElement(UserDescriptorId, DescValueSpec* pDVS); // _no swig_
         /// 
   void            AddDescValueSpecValueElement(UserDescriptorId, DescValueSpec* pDVS, Boolean);
         /// 
void               RemoveDescValueSpecValueElement(UserDescriptorId descId, DescValueSpec* pDVS);
         /// Ajoute une à une les entités du tableau fourni en second argument en fin du tableau qui est la valeur du descripteur dont le symbole est passé en premier argument.
   void            AddDescValueSpecValueElements(UserDescriptorId, pDescValueSpecTab*); // _no swig_
         /// 
   void AddIntValueElement(UserDescriptorId descId, int);
         /// 
   void AddFloatValueElement(UserDescriptorId descId, float);
         /// 
   void AddDoubleValueElement(UserDescriptorId descId, double);
         /// Sélectionne les descripteurs de l'entité {\em pOrig} qui sont aussi descripteurs de l'entité réceptrice du message, et leur affecte, dans l'entité réceptrice, les valeurs des descripteurs dans {\em pOrig}. Pour les descripteurs de type {\em STRING_TAB} et {\em ENTITY_TAB}, la valeur affectée est une copie du tableau d'origine. Pour les descripteurs de type {\em STRING}, la valeur affectée est une copie de la chaîne d'origine.\index{méthodes!CopyDescriptorValuesFrom}
   void CopyDescriptorValuesFrom(DescriptedEntity* pOrig); // + JPR 090209
  //@} // fin Affectation de valeur partielle
    
//@}  // Affectation/acc\`es aux valeurs des descripteurs
 

/**@name [SC]Affichage[/SC] */
//@{
         /// Affiche les informations sur la structure du récepteur, ainsi que les valeurs des descripteurs.\index{méthodes!DisplayAttributes - DescriptedEntity}
   void                DisplayAttributes() const; 

//@}

   Boolean IsEngineManaged() const;
   void IsEngineManaged(Boolean);

   UserDescriptorId GetDescIdForSorting() const;
   void SetDescIdForSorting(UserDescriptorId);
   
// INQUIRY

protected:

   pConstantDescriptorTab mpConstantDescriptorTab;
   // -----------------
   // | 0 | 1 | 2 | 3 |  <- indice
   // -----------------
   // | 7 | 2 | 8 | 1 |  <- symbole du descripteur
   // ----------------- 

   TableauInt             mpDualConstantDescriptorTab; // + JPR 991026
   // ce tableau contient en position k l'indice du tableau 
   // mpConstantDescriptorTab correspondant au descripteur de symbole k
   // une valeur -9 indique que le descripteur de symbole k n'existe pas
   // -------------------------------------
   // | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |  <- indice
   // -------------------------------------
   // |-9 | 3 | 1 |-9 |-9 |-9 |-9 | 0 | 2 |  <- indice dans mp..DescriptorTab
   // -------------------------------------

   
   pVariableDescriptorTab mpVariableDescriptorTab; 
   TableauInt             mpDualVariableDescriptorTab; // + JPR 991026

  // début code gestion OutputSpec
  // fin code gestion OutputSpec

   // + JPR 110412
   Boolean mIsEngineManaged;
   // + JPR 110413
   UserDescriptorId mDescIdForSorting;

// PRIVATE OPERATIONS

// PRIVATE ACCESS
   Boolean  HasConstantDescriptors() const;
   Boolean  HasVariableDescriptors() const;

   friend class Simulation; //appel Has..Descriptors
};  // _end class_

//@} // fin classe

// INLINE METHODS

// EXTERNAL REFERENCES


//-X///////////////X/////////////////////////////X////////////////////////////

#endif  // _DESCRIPTEDENTITY_HH
