//============================================================================
// FILE: C_SingleResource.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des ressources
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_SINGLERESOURCE_HH
#define _C_SINGLERESOURCE_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//

#include "C_Resource.h"
#include "C_CoreTypes.h"
#include "C_TransferInformation.h"

// EXTERNAL REFERENCES

/**@name Les ressources atomiques 
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include C_SingleResource.h
[/P]
\end{itemize}

Une ressource atomique (classe [C]SingleResource[/C]) est une ressource qui n'est pas l'assemblage d'autres ressources. En d'autres termes, ses contraintes d'usage, son fonctionnement et sa dynamique lui sont intrinsèques et ne dépendent d'aucune autre.\\
\begin{itemize}
\item Selon le mode de détermination de sa valeur de disponibilité/indisponibilité, une resource est soit une ressource "à état discret" soit une ressource "à capacité".\\
Les second et troisième liens en haut de la page pointent sur ces deux catégories.\\

\item Du point de vue de son usage, une resource est soit "réutilisable" soit "consommable".
\begin{itemize}
\item Une ressource {\em consommable} est exploitable une fois et une seule. En d'autres termes, chaque unité de capacité n'est engageable qu'une et une seule fois (pour une ressource "à capacité") ou bien l'engagement provoque une indisponibilité définitive (pour une ressource "à état discret").

\item Une ressource {\em réutilisable} redevient exploitable après qu'elle a été exploitée pour une activité. En d'autres termes, chaque unité de capacité est réengageable ou bien l'engagement ne provoque pas une indisponibilité définitive.
\end{itemize}

\item Les deux points de vues sont complémentaires : il existe des ressources à état discret réutilisables, des ressources à capacité consommables, etc.
\end{itemize}

*/ 
//@{

//X//////////////////////////////////////////////////////////////////////////
//
// CLASSES DE BASE
//
//X//////////////////////////////////////////////////////////////////////////

/**@name Classe de base des ressources atomiques\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]SingleResource[/C]} & {\tiny SINGLE_RESOURCE} & {\tiny singleResource} & {\tiny } & {\tiny }
\end{tabular}

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny int} & {\tiny [C]IsDiscreteState[/C] IS_DISCRETE_STATE isDiscreteState} & {\tiny 1} & {\tiny {\em faux} seulement pour les [C]SingleCapacitatedResource[/C]}\hline
{\tiny C} & {\tiny int} & {\tiny [C]IsReusable[/C] IS_REUSABLE isReusable} & {\tiny 1} & {\tiny {\em faux} seulement pour les [C]SingleDiscreteStateConsumableResource[/C] et les [C]SingleCapacitatedConsumableResource[/C]}\hline
{\tiny C} & {\tiny pEntity*} & {\tiny [C]TimeAvailabilityAttribute[/C] TIME_AVAILABILITY_ATTRIBUTE timeAvailabilityAttribute} & {\tiny NULL} & {\tiny une instance de [C]TimeAvailabilityConstraint[/C]}\hline
{\tiny C} & {\tiny pEntityTab*} & {\tiny [C]StateAvailabilityAttribute[/C] STATE_AVAILABILITY_ATTRIBUTE stateAvailabilityAttribute} & {\tiny un tableau vide} & {\tiny une liste d'instances de [C]StateAvailabilityConstraint[/C]}\hline
{\tiny C} & {\tiny pEntity*} & {\tiny [C]AvailabilityTimetableAttribute[/C] AVAILABILITY_TIMETABLE_ATTRIBUTE availabilityTimetableAttribute} & {\tiny NULL} & {\tiny une instance de [C]RecurentAvailabilityTimetable[/C]}
\end{tabular}
*/
//@{ 
        ///
class SingleResource : public Resource 
{
public:
          // Constructeur de base.
  SingleResource();
          // Constructeur par copie.
  SingleResource(const SingleResource&);
          // Destructeur.
  ~SingleResource();
  
/**@name  [SC]Tests de disponibilité[/SC] \\\\
Ces services sont appliqués directement à une instance de [C]SingleResource[/C], ou bien indirectement par ceux du même nom au niveau de la classe mère [C]Resource[/C], lorsque le composant ou l'élément est une instance de [C]SingleResource[/C].
*/
//@{
         /// Renvoie {\em vrai} si la ressource satisfait la contraintes de disponibilité liée au temps qui lui est attachée.\index{méthodes!CheckTimeAvailabilityConstraint}
Boolean CheckTimeAvailabilityConstraint();  // _no swig_ 

         /// Renvoie {\em vrai} si la ressource satisfait toutes les contraintes de disponibilité liées à son état qui lui sont attachées.\index{méthodes!CheckStateAvailabilityConstraints}
Boolean CheckStateAvailabilityConstraints();  // _no swig_ 

         /// Renvoie {\em vrai} si la ressource satisfait toutes les contraintes de disponibilité liées à sa capacité qui lui sont attachées.\index{méthodes!CheckCapacityAvailabilityConstraints}
Boolean CheckCapacityAvailabilityConstraints();  // _no swig_ 

         // / Renvoie {\em faux} si la valeur du descripteur de symbole AVAILABILITY_STATUS est {\em faux}. Sinon, renvoie {\em vrai} si la ressource satisfait toutes les contraintes de disponibilité liées à son état et toutes la contraintes de disponibilité liée au temps qui lui sont attachées.\index{méthodes!IsTimeAndStateAvailable}
  //Boolean IsTimeAndStateAvailable();  // _no swig_  

//@}
 };

//@}  // fin classe de basedes ressources atomiques

//bool singleDiscreteStateResource_availabilityPredicate_Body(EntityMethod*);
//bool singleCapacitatedResource_availabilityPredicate_Body(EntityMethod*);

//X//////////////////////////////////////////////////////////////////////////
//
// resources atomiques 'DISCRETE STATE'
//
//X//////////////////////////////////////////////////////////////////////////

/**@name Ressources atomiques à état discret

Une ressource "à état discret" est caractérisée par un élément qualitatif de son état global.\\

Sa valeur de disponibilité est booléenne, et est fonction de cette facette d'état et des contraintes de disponibilité, liées à cet état, attachées à la resource.\\

 Par exemple, une instance de la classe [C]Pneu[/C] est une resource caractérisée par son état de pression. Elle est disponible si le descripteur en question a une valeur comprise entre 1 et 2 bars, et non disponible sinon. Dans ce premier exemple, l'état qualitatif a deux réalisations possibles. Une instance de la classe [C]voie[/C] est disponible si la valeur de son descripteur [C]FeuTricolore[/C] est vert, non disponible si la valeur est rouge ou orange : ici, l'état qualitatif a trois réalisations possibles.
*/ 
//@{

/**@name La classe de base\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]SingleDiscreteStateResource[/C]} & {\tiny SINGLE_DISCRETE_STATE_RESOURCE} & {\tiny singleDiscreteStateResource} & {\tiny } & {\tiny }
\end{tabular}
*/ 
//@{ 
         ///
class SingleDiscreteStateResource : public SingleResource
{
public:
          // Constructeur de base.
  SingleDiscreteStateResource();
          // Constructeur par copie.
  SingleDiscreteStateResource(const SingleDiscreteStateResource&);
          // Destructeur.
  ~SingleDiscreteStateResource();
  
/* *@name ACCESS */
// @{

// @}
 };

//@}  // fin classe de base ressources atomiques à état discret

//X//////////////////////////////////////////////////////////////////////////
//
// resources atomiques 'DISCRETE STATE' 'REUSABLE'
//
//X//////////////////////////////////////////////////////////////////////////

/**@name Ressources atomiques à état discret et réutilisables\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]SingleDiscreteStateReusableResource[/C]} & {\tiny SINGLE_DISCRETE_STATE_REUSABLE_RESOURCE} & {\tiny singleDiscreteStateReusableResource} & {\tiny } & {\tiny }
\end{tabular}
*/ 
//@{
        ///
class SingleDiscreteStateReusableResource : public SingleDiscreteStateResource
{
public:
          // Constructeur de base.
  SingleDiscreteStateReusableResource();
          // Constructeur par copie.
  SingleDiscreteStateReusableResource(const SingleDiscreteStateReusableResource&);
          // Destructeur.
  ~SingleDiscreteStateReusableResource();
  
 };

//@}  // fin ressources atomiques à état discret et réutilisables

// /X///////////////X/////////////////////////////X////////////////////////////
//
// SINGLE PERFORMER
//
// /X///////////////X/////////////////////////////X////////////////////////////

/**@name Une ressource atomique à état discret réutilisable : les performers\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]SinglePerformer[/C]} & {\tiny SINGLE_PERFORMER} & {\tiny singlePerformer} & {\tiny } & {\tiny }
\end{tabular}

La classe des performers atomiques ([C]SinglePerformer[/C]) est dotée d'un descripteur [C]Power[/C] qui mesure sa puissance. \\

La puissance est exploitée par le moteur d'allocation pour multiplier la réquisition de ressources par l'opération. Par exemple, une opération qui requiert nominalement un outil, et un seul, en requerra deux si la puissance du performer est de 2.0, et si on a spécifié que la proportionnalité devait s'appliquer (descripteur [C]ProportionnalityMode[/C] de la spécification de ressources propres de l'opération).

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny V} & {\tiny float} & {\tiny [C]Power[/C] POWER power} & {\tiny 1.0} & {\tiny {\em la puissance du performer}
\end{tabular}
*/ 
//@{
        ///
class SinglePerformer : public SingleDiscreteStateReusableResource
{
public:
   SinglePerformer();

        /// Renvoie la puissance du performer. Pour une telle ressource atomique, c'est la valeur du descripteur  [C]Power[/C]. Ce service est exploité par le moteur dans le calcul de la puissance d'un performer aggrégé homogène.
  float GetPower();  // + JPR 111027
};

//@}  // fin Performer

// /X///////////////X/////////////////////////////X////////////////////////////
//
// SINGLE WORKER
//
// /X///////////////X/////////////////////////////X////////////////////////////

/**@name Un performer particulier : le travailleur humain\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]SingleWorker[/C]} & {\tiny SINGLE_WORKER} & {\tiny singleWorker} & {\tiny } & {\tiny }
\end{tabular}

Cette classe ne possède ni descripteur, ni méthode, ni service particulier.\\

Elle peut servir dans une application à distinguer de manière pratique, parmi les instances de [C]SingleDiscreteStateReusableResource[/C], les travailleurs humains et les outils, entre autres sortes de ressources.\\

Elle peut être aussi utilisée pour contraindre la classe des éléments d'une équipe de travailleurs humains simples (ressources atomiques). Noter à ce sujet que la classe [C]HomogeneousLaborTeam[/C] peut avoir des ressources-éléments qui ne sont pas elles-mêmes atomiques.
*/ 
//@{
        ///
class SingleWorker : public SinglePerformer
{
public:
   SingleWorker();
};

//@}  // fin SingleWorker

// /X///////////////X/////////////////////////////X////////////////////////////
//
// SINGLE TOOL
//
// /X///////////////X/////////////////////////////X////////////////////////////

/**@name Une ressources atomique à état discret réutilisable : les outils\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]SingleTool[/C]} & {\tiny SINGLE_TOOL} & {\tiny singleTool} & {\tiny } & {\tiny }
\end{tabular}

Cette classe ne possède ni descripteur, ni méthode, ni service particulier.\\

Elle peut servir dans une application à distinguer de manière pratique, parmi les instances de [C]SingleDiscreteStateReusableResource[/C], les travailleurs humains et les outils, entre autres sortes de ressources.
*/ 
//@{
        ///
class SingleTool : public SingleDiscreteStateReusableResource
{
public:
   SingleTool();
};

//@}  // fin Tool

//X//////////////////////////////////////////////////////////////////////////
//
// resources atomiques 'DISCRETE STATE' 'CONSUMABLE'
//
//X//////////////////////////////////////////////////////////////////////////

/**@name Ressources atomiques à état discret et consommables\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]SingleDiscreteStateConsumableResource[/C]} & {\tiny SINGLE_DISCRETE_STATE_CONSUMABLE_RESOURCE} & {\tiny singleDiscreteStateConsumableResource} & {\tiny } & {\tiny }
\end{tabular}

Cette classe ne possède ni descripteur, ni méthode, ni service particulier.\\

La valeur du descripteur hérité [C]IsReusable[/C] est 0.
*/ 
//@{
        ///
class SingleDiscreteStateConsumableResource : public SingleDiscreteStateResource
{
public:
          // Constructeur de base.
  SingleDiscreteStateConsumableResource();
          // Constructeur par copie.
  SingleDiscreteStateConsumableResource(const SingleDiscreteStateConsumableResource&);
          // Destructeur.
  ~SingleDiscreteStateConsumableResource();
};

//@}  // fin ressources atomiques à état discret et consommables

//@}  // fin ressources atomiques à état discret

//X//////////////////////////////////////////////////////////////////////////
//
// resources atomiques 'CAPACITATED'
//
//X//////////////////////////////////////////////////////////////////////////

/**@name Ressources atomiques à capacité

Une ressource atomique à capacité (classe [C]SingleCapacitatedResource[/C]) est caractérisée par un vecteur de valeurs exprimant une capacité multidimensionnelle. Son degré de disponibilité est, à un instant donné, le vecteur des valeurs courantes sur ces dimensions.\\

L'état de disponibilité de la ressource dépend de son degré de disponibilité et des contraintes de disponibilité liées à cette capacité (instances de [C]CapacityAvailabilityConstraint[/C]) attachées à la ressource.\\

Pour exemple, soit une instance de la classe [C]MultiStock[/C] dans le domaine d'application, contenant 10 entités de classe [C]P1[/C] et 5 entités de classe [C]P2[/C]. Ceci peut-être modélisé par deux descripteurs : 
\begin{itemize}
\item [C]P1Content[/C] dont la valeur est 10
\item [C]P2Content[/C] dont la valeur est 5
\end{itemize}

Si on a fait hériter cette classe de [C]SingleCapacitatedResource[/C], elle possède le descripteur [C]ResourceCapacitySupply[/C], qui a pour valeur une instance de [C]MultipleResourceCapacity[/C]. Cette dernière possède deux éléments de classe [C]IntegerResourceCapacity[/C], qui ont été ajoutés par le développeur de l'application par le service [Y]InstallIntegerCapacity[/Y], de la manière suivante :

[P]
\begin{verbatim}{
  //...
  SingleCapacitatedResource* pMS = new MultiStock();
  int nb_P1 = pMS->GetIntVarValue(P1_CONTENT);
  pMS->InstallFloatCapacity(pMS->GetDescriptor(P1_CONTENT), nb_P1);
  int nb_P2 = pMS->GetIntVarValue(P2_CONTENT);
  pMS->InstallFloatCapacity(pMS->GetDescriptor(P2_CONTENT), nb_P2);
  //...
}
\end{verbatim}
[/P]

On ne le fait pas dans le code ci-dessus, mais on peut déclarer que la capacité offerte est inférieure au contenu.\\

Selon que [C]MultiStock[/C] est une instance de [C]SingleCapacitatedReusableResource[/C] ou de [C]SingleCapacitatedConsumableResource[/C], cette capacité pourra être utilisée ou consommée. Pour poursuivre l'illustration, on envisage ici une consommation. On distingue deux situations de développement :  
\begin{itemize}
\item la consommation est réalisée par une opération de transfert dans le cadre d'une activité de transfert ([C]TransferActivity[/C]) ;
\item la consommation est réalisée dans un processus ou dans la "procédure de transition d'état" d'une opération banale.
\end{itemize}

            {\large {\tt Dans le second cas}}, on exploite le service [Y]ConsumeCapacity[/Y], argumenté par la grandeur consommée et la quantité qui ne sera plus consommable :
[P]
\begin{verbatim}{
  //...
  pMS->ConsumeCapacity(P1_CONTENT, 3);
  //...
}
\end{verbatim}
[/P]

[Y]ConsumeCapacity[/Y] assure que la capacité restant disponible n'est pas inférieure au niveau minimal (valeur de [C]IntLowerLevel[/C] de la [C]IntegerResourceCapacity[/C]) si la ressource est un [C]SingleStock[/C], et pas inférieure à 0 sinon.\\

Dans le premier cas {\large {\tt (opération de transfert)}}, la consommation est une procédure déjà codée dans le moteur de simulation. Cette procédure est alimentée par des informations attachées par l'utilisateur à son instance de [C]TransferActivity[/C] :
\begin{itemize}
\item Une [C]TransferActivity[/C] est dotée d'un descripteur dont la valeur est une instance de [C]TransferInformation[/C], et de deux descripteurs qui pointent sur l'objet opéré "source" et sur l'objet opéré "destination" du transfert.
\item La [C]TransferInformation[/C] est dotée d'un descripteur [C]TransferQuantityId[/C] (ce qui sera transféré) et d'un descripteur [C]TransferQuantityIntValue[/C] (en quelle quantité).
\end{itemize}

Dans le cas d'un transfert, la procédure d'allocation des ressources, crée deux instances de [C]MultipleResourceCapacity[/C], une pour exprimer la demande (au profit de la destination) et une autre pour exprimer l'offre (par la source). Puis elle teste la ou les contraintes de disponibilité (liées à la capacité) attachées aux ressources en jeu.
Il se trouve que, par construction, toute instance de [C]SingleCapacitatedResource[/C] est dotée d'une contrainte prédéfinie ([C]StandardCapacityAvailabilityConstraint[/C]) qui stipule, dans sa méthode [C]CapacityOfferDemandComparison[/C], que la demande ne doit pas dépasser l'offre en capacité. Cette méthode compare les deux instances de [C]MultipleResourceCapacity[/C] évoquées ci-dessus. La contrainte est satisfaite si la méthode renvoie {\em vrai} et dans ce cas, le transfert est autorisé.\\

Poursuivant l'illustration, le transfert ne sera pas autorisé si l'information sur le transfert est la suivante, dans le fichier de déclaration externalisée :
[P]
\begin{verbatim}
+ I multiStock pSrcStock
    p1Content = 10;
    p1Content = 3;  // l'offre
;
+ I multiStock pDestStock
;
+ I transferInformation pTrInfo1,
	transferQuantityId = P1_CONTENT;		
	transferQuantityIntValue = 5;  // demande > offre !
;
+ I myTransferActivity 	pTrAct
	operatedObjectSpecAttribute =
		+ I operatedObjectSpecification laSource,
		    //...
			preExpandedList << <I><, pSrcStock>;
		; 
	;
	destinationSpecAttribute =
		+ I operatedObjectSpecification laDestination,
		    //...
			preExpandedList << <I><, pDestStock>;
		; 
	;
	transferInformationAttribute = <I><, pTrInfo1>;
;
\end{verbatim}
[/P]

Noter que la contrainte [C]StandardCapacityAvailabilityConstraint[/C] est attachée à la ressource comme élément de la liste valeur du descripteur [C]CapacityAvailabilityAttribute[/C]. Bien que naturelle, on peut l'enlever, et la remplacer ou la compléter par d'autres spécifiques au domaine d'application. De la manière suivante, dans un code internalisé :
[P]
\begin{verbatim}
bool myCapacityOfferDemandComparison_Body(EntityMethod* pM) {
  // une critere de satisfaction spécifique à l'application
  bool result = true;
  Entity* pConstraint = pM->DescribedEntity();
  Entity* pCapRsc = pConstraint->GetEntityConstValue(DESCRIBED_RESOURCE);
  Entity* pMultipleCapDemand 
    = pCapRsc->GetEntityConstValue(RESOURCE_CAPACITY_DEMAND);
  Entity* pMultipleCapSupply 
    = pCapRsc->GetEntityConstValue(RESOURCE_CAPACITY_SUPPLY);
  // ... établissement de result à faux dans une situation
  // où la contrainte doit être perçue comme non satisfaite
  return result;
}
{
  // ...
  MultipleResourceCapacity* pMultRscCap = 
    pSrcStock->GetEntityConstValue(CAPACITY_AVAILABILITY_ATTRIBUTE);
  int nbCtr = pMultRscCap->GetNumberOfElements();
  for(int k=nbCtr-1;k>=0;k--) {
    BasicEntity* pCtr_k = pMultRscCap->GetElement(k);
    if(pCtr_k->IsInstanceOf(STANDARD_CAPACITY_AVAILABILITY_CONSTRAINT)) {
      pMultRscCap->RemoveElement(k);
      break;
    }
  }
  Entity* pNewCtr = new CapacityAvailabilityConstraint();
  pNewCtr->AssignBody(CAPACITY_OFFER_DEMAND_COMPARISON, 
                      myCapacityOfferDemandComparison_Body);
  pNewCtr->SetEntityConstValue(DESCRIBED_RESOURCE, pSrcStock);
  pMultRscCap->AddElement(pNewCtr);
  // ...
}
\end{verbatim}
[/P]

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny pEntity*} & {\tiny [C]ResourceCapacitySupply[/C] RESOURCE_CAPACITY_SUPPLY, resourceCapacitySupply} & {\tiny NULL} & {\tiny une instance de [C]MultipleResourceCapacity[/C]}\hline
{\tiny C} & {\tiny pEntity*} & {\tiny [C]ResourceCapacityDemand[/C] RESOURCE_CAPACITY_DEMAND, resourceCapacityDemand} & {\tiny NULL} & {\tiny une instance de  [C]MultipleResourceCapacity[/C]}\hline
{\tiny C} & {\tiny pEntityTab*} & {\tiny [C]CapacityAvailabilityAttribute[/C] CAPACITY_AVAILABILITY_ATTRIBUTE capacityAvailabilityAttribute} & {\tiny un tableau vide} & {\tiny {\em une liste d'instances de [C]CapacityAvailabilityConstraint[/C]}
\end{tabular}
*/ 
//@{

/**@name La classe de base\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]SingleCapacitatedResource[/C]} & {\tiny SINGLE_CAPACITATED_RESOURCE} & {\tiny singleCapacitatedResource} & {\tiny } & {\tiny }
\end{tabular}
*/ 
//@{
        ///
class SingleCapacitatedResource : public SingleResource
{
public:
          // Constructeur de base.
  SingleCapacitatedResource();
          // Constructeur par copie.
  SingleCapacitatedResource(const SingleCapacitatedResource&);
          // Destructeur.
  ~SingleCapacitatedResource();

/**@name [SC]Etablissement de la capacité[/SC]\\\\

Ces services spéficient la capacité offerte par une ressource, c'est-à-dire disponible à la consommation ou l'usage.\\

Ils créent une instance de [C]ResourceCapacity[/C] et l'ajoutent comme élément d'une [C]MultipleResourceCapacity[/C], celle qui est la valeur du descripteur [C]ResourceCapacitySupply[/C] de la ressource.\\

Ils sont argumentés par :
\begin{itemize}
\item un pointeur sur un descripteur de la ressource : celui dont la valeur va fluctuer par la consommation ou l'usage de la ressource ;
\item une quantité, qui indique la mesure de la capacité.
\end{itemize}
*/
//@{

         /// Crée une instance de [C]FloatResourceCapacity[/C], value son descripteur [C]DescriptorIdAttribute[/C] avec le symbole de classe de {\small {\tt pD}}, et son descripteur [C]AnonymousFloatAttribute[/C] avec {\small {\tt value}}.
void InstallFloatCapacity(Descriptor* pD, float value); 

         /// Crée une instance de [C]IntegerResourceCapacity[/C], value son descripteur [C]DescriptorIdAttribute[/C] avec le symbole de classe de {\small {\tt pD}}, et son descripteur [C]AnonymousIntAttribute[/C] avec {\small {\tt value}}.
void InstallIntegerCapacity(Descriptor* pD, int value);

         /// Crée une instance de [C]IntegerResourceCapacity[/C], value son descripteur [C]DescriptorIdAttribute[/C] avec le symbole C_ELEMENTS, et son descripteur [C]AnonymousIntAttribute[/C] avec {\small {\tt value}}.
void InstallElementCapacity(int value);
//@} 
  
/**@name [SC]Test des contraintes de capacité[/SC] */
//@{
         /// Renvoie {\em vrai} si la ressource satisfait toutes les contraintes de disponibilité liées à sa capacité qui lui sont attachées.\index{méthodes!CheckCapacityAvailabilityConstraints}
Boolean CheckCapacityAvailabilityConstraints();  // _no swig_

//@} 
 };

//@}  // fin classe de base ressources atomiques à capacité

//X//////////////////////////////////////////////////////////////////////////
//
// resources atomiques 'CAPACITATED' 'REUSABLE'
//
//X//////////////////////////////////////////////////////////////////////////

/**@name Ressources atomiques à capacité réutilisable\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]SingleCapacitatedReusableResource[/C]} & {\tiny SINGLE_CAPACITATED_REUSABLE_RESOURCE} & {\tiny singleCapacitatedReusableResource} & {\tiny } & {\tiny }
\end{tabular}

Ces ressources ont une capacité (multidimensionnelle) exploitable par une activité entre le début et la fin de sa mise en oeuvre. En cet instant de fin, la partie de la capacité en question peut être restituée à la ressource.\\

Les services [Y]UseCapacity[/Y] rendent provisoirement indisponible une partie de la capacité quantitative offerte par une ressource. Cette partie devient indisponible à la consommation ou l'usage des tiers.\\
Les services [Y]RefillCapacity[/Y] rend à nouveau disponible une partie de la capacité quantitative offerte par une ressource.\\

Ils sont argumentés par :
\begin{itemize}
\item un symbole de classe d'un descripteur de la ressource : celui dont la valeur va fluctuer par l'usage ou la restitution de la ressource ;
\item une quantité, qui indique la mesure de la capacité utilisée ou restituée.
\end{itemize}

Ils modifient la valeur du descripteur [C]ResourceCapacitySupply[/C] de la ressource (cette valeur est une instance de [C]MultipleResourceCapacity[/C]). \\
Plus précisément, ils modifient son élément (instance d'une sous-classe typée de [C]ResourceCapacity[/C]) dont le descripteur [C]DescriptorIdAttribute[/C] a le symbole de classe passé en argument.\\
Plus précisément encore :
\begin{itemize}
\item si la ressource est un [C]SingleStock[/C], ils modifient le niveau courant (par exemple [C]IntCurrentLevel[/C], et ajuste en conséquence les capacites 'en entrée' et 'en sortie' de la ressource ;
\item sinon, ils modifient la valeur du descripteur [C]AnonymousFloatAttribute[/C] de la capacité.
\end{itemize}
*/ 
//@{
        ///
class SingleCapacitatedReusableResource : public SingleCapacitatedResource
{
public:
          // Constructeur de base.
  SingleCapacitatedReusableResource();
          // Constructeur par copie.
  SingleCapacitatedReusableResource(const SingleCapacitatedReusableResource&);
          // Destructeur.
  ~SingleCapacitatedReusableResource();

/**@name [SC]Usage et restitution de la capacité[/SC] */
//@{
          /// Utilise une partie ({\small {\tt value}}) de capacité, dans l'instance de [C]IntegerResourceCapacity[/C] dont le descripteur [C]DescriptorIdAttribute[/C] est {\small {\tt id}}. 
void UseCapacity(UserDescriptorId id, int value);
          /// Utilise une partie ({\small {\tt value}}) de capacité, dans l'instance de [C]FloatResourceCapacity[/C] dont le descripteur [C]DescriptorIdAttribute[/C] est {\small {\tt id}}.
void UseCapacity(UserDescriptorId, float);
          //
void UseCapacity(UserDescriptorId, double);
          /// Restitue une partie ({\small {\tt value}}) de capacité, dans l'instance de [C]IntegerResourceCapacity[/C] dont le descripteur [C]DescriptorIdAttribute[/C] est {\small {\tt id}}. 
void RefillCapacity(UserDescriptorId, int);
          /// Restitue une partie ({\small {\tt value}}) de capacité, dans l'instance de [C]FloatResourceCapacity[/C] dont le descripteur [C]DescriptorIdAttribute[/C] est {\small {\tt id}}. 
void RefillCapacity(UserDescriptorId, float);
          //
void RefillCapacity(UserDescriptorId, double);
//@}

 };

//@}  // fin ressources atomiques à capacité réutilisable

/**@name Une ressource atomique à capacité réutilisable : les stocks\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]SingleStock[/C]} & {\tiny SINGLE_STOCK} & {\tiny singleStock} & {\tiny } & {\tiny }
\end{tabular}

Un stock est une ressource caractérisée par un niveau, dont la valeur courante est limitée inférieurement et supérieurement. Ces trois valeurs sont installées dans une instance de la classe [C]ResourceCapacity[/C] attachée au stock.\\

Les services [Y]InstallIntegerCapacity[/Y], [Y]InstallFloatCapacity[/Y] et [Y]InstallElementCapacity[/Y] de la classe de base [C]SingleStock[/C] font deux choses :
\begin{itemize}
\item créer une instance d'une sous-classe dédiée et typée de [C]ResourceCapacity[/C] (par exemple [C]IntegerStockCapacity[/C]), et lui attacher les bornes inférieures et supérieures du niveau, ainsi que le niveau courant ;
\item établir deux valeurs de capacité : la capacité "à recevoir" et la capacité "à offrir" :
\begin{itemize}
\item la première est la différence entre la borne supérieure et la valeur courante ;
\item la seconde est la différence entre la valeur courante et la borne inférieure.
\end{itemize}
\end{itemize}

Le niveau courant, et par conséquent les capacités "à recevoir" et "à offrir", varient ensuite par :
\begin{itemize}
\item les services explicites [Y]UseCapacity[/Y] et [Y]RefillCapacity[/Y], réalisés sur la classe SingleCapacitedReusableResource[/C] ;
\item au cours d'une opération de transfert.
\end{itemize}
*/ 
//@{
        ///
//class SingleStock : public SingleCapacitatedConsumableResource
class SingleStock : public SingleCapacitatedReusableResource   // modif JPR 120206
{
public:
          // Constructeur de base.
  SingleStock();
          // Constructeur par copie.
  SingleStock(const SingleStock&);
          // Destructeur.
  ~SingleStock();

/**@name [SC]Etablissement de la capacité[/SC]\\\\
 */
//@{
          /// Crée une instance de [C]FloatResourceCapacity[/C], value son descripteur [C]DescriptorIdAttribute[/C] avec le symbole de classe de {\small {\tt pD}} ; les descripteurs [C]FloatUpperLevel[/C], [C]FloatLowerLevel[/C] et [C]FloatCurrentLevel[/C] prennent les valeurs {\small {\tt upperLevel}}, {\small {\tt lowerLevel}} et {\small {\tt currentLevel}}.
void InstallFloatCapacity(Descriptor* pD,
			  float upperLevel, 
			  float lowerLevel, 
			  float currentLevel); 
          /// Crée une instance de [C]IntegerResourceCapacity[/C], value son descripteur [C]DescriptorIdAttribute[/C] avec le symbole de classe de {\small {\tt pD}} ; les descripteurs [C]IntUpperLevel[/C], [C]IntLowerLevel[/C] et [C]IntCurrentLevel[/C] prennent les valeurs {\small {\tt upperLevel}}, {\small {\tt lowerLevel}} et {\small {\tt currentLevel}}.
void InstallIntegerCapacity(Descriptor* pD,
			  int upperLevel, 
			  int lowerLevel, 
			  int currentLevel);
          /// Crée une instance de [C]IntegerResourceCapacity[/C], value son descripteur [C]DescriptorIdAttribute[/C] avec le symbole C_ELEMENTS ; les descripteurs [C]IntUpperLevel[/C], [C]IntLowerLevel[/C] et [C]IntCurrentLevel[/C] prennent les valeurs {\small {\tt upperLevel}}, {\small {\tt lowerLevel}} et {\small {\tt currentLevel}}.
void InstallElementCapacity(int upperLevel, 
			    int lowerLevel, 
			    int currentLevel);  
//@}
 };

//@}  // fin stocks

//X//////////////////////////////////////////////////////////////////////////
//
// resources atomiques 'CAPACITATED' 'CONSUMABLE'
//
//X//////////////////////////////////////////////////////////////////////////

/**@name Ressources atomiques à capacité consommable\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]SingleCapacitatedConsumableResource[/C]} & {\tiny SINGLE_CAPACITATED_CONSUMABLE_RESOURCE} & {\tiny singleCapacitatedConsumableResource} & {\tiny } & {\tiny }
\end{tabular}

Ces ressources ont une capacité (multidimensionnelle) exploitable par une activité. En l'instant de fin de sa mise en oeuvre, la partie de la capacité en question ne peut pas être restituée à la ressource.\\

Les services [Y]ConsumeCapacity[/Y] rendent définitivement indisponible à l'usage des tiers une partie de la capacité quantitative offerte par une ressource.\\

Ils sont argumentés par :
\begin{itemize}
\item un symbole de classe d'un descripteur de la ressource : celui dont la valeur va diminuer par la consommation de la ressource ;
\item une quantité, qui indique la mesure de la capacité consomée.
\end{itemize}

Ils modifient la valeur du descripteur [C]ResourceCapacitySupply[/C] de la ressource. C'est une instance de [C]MultipleResourceCapacity[/C]. Plus précisément ils modifient son élément (d'une sous-classe typée de [C]ResourceCapacity[/C]) dont le descripteur [C]DescriptorIdAttribute[/C] a le symbole de classe passé en argument.
*/ 
//@{
        ///
class SingleCapacitatedConsumableResource : public SingleCapacitatedResource
{
public:
          // Constructeur de base.
  SingleCapacitatedConsumableResource();
          // Constructeur par copie.
  SingleCapacitatedConsumableResource(const SingleCapacitatedConsumableResource&);
          // Destructeur.
  ~SingleCapacitatedConsumableResource();

/**@name [SC]Consommation de la capacité[/SC] */
//@{
          /// Consomme une partie ({\small {\tt value}}) de capacité, dans l'instance de [C]IntegerResourceCapacity[/C] dont le descripteur [C]DescriptorIdAttribute[/C] est {\small {\tt id}}. 
void ConsumeCapacity(UserDescriptorId, int);
          /// Consomme une partie ({\small {\tt value}}) de capacité, dans l'instance de [C]FloatResourceCapacity[/C] dont le descripteur [C]DescriptorIdAttribute[/C] est {\small {\tt id}}. 
void ConsumeCapacity(UserDescriptorId, float);
          //
void ConsumeCapacity(UserDescriptorId, double);
//@}
 
 };

//@}  // fin ressources atomiques à capacité consommable

//@}  // fin ressources atomiques à capacité

//X//////////////////////////////////////////////////////////////////////////
//
// cas particulier de DS Reusable : les 'objets opérés'
//
//X//////////////////////////////////////////////////////////////////////////

/**@name Les objets opérés vus comme des ressources atomiques

Une activité primitive ([C]PrimitiveActivity[/C]) met en oeuvre une opération ([C]Operation[/C]) sur des entités, élément ou composants du système simulé. Ces entités sont spécifiées par une instance de [C]OperatedObjectSpecification[/C], valeur du descripteur [C]OperatedObjectSpecAttribute[/C] de l'activité.\\

Pour pouvoir poser et faire jouer des contraintes sur la disponibilité de ces entités à être opérées, il faut que la procédure "moteur" d'allocation puisse les percevoir comme des ressources, même quand elles ne sont déclarées comme telles (c'est-à-dire instance de [C]Resource[/C]). Par exemple une activité qui opère une pièce est censée mobiliser cette pièce, laquelle devient indisponible pour être opérée par une autre activité.\\

A cette fin, la procédure d'allocation, et plus précisément le service attaché à la spécification d'objets opérés et qui la "développe", encapsule chaque entité dans une instance (indirecte) de la présente classe [C]OperatedObject[/C] :
\begin{itemize}
\item [C]OperatedResource[/C] si l'entité est déjà une ressource,
\item [C]NonSpecialOperatedObject[/C] sinon.
\end{itemize}

Cette classe est en outre dotée de descripteurs et de services qui permettent de "tracer" et d'accéder à l'évolution des quantités et des proportions opérée et non encore opérées des entités :
\begin{itemize}
\item La valeur de ces descripteurs est établie par le moteur de simulation, et le code de l'application n'intervient normalement pas dans ce processus.
\item Par contre, le code de l'application peut trouver intérêt à connaître ces valeurs.
\end{itemize}
*/ 
//@{ 

//X//////////////////////////////////////////////////////////////////////////
//
// CLASSES DE BASE
//
//X//////////////////////////////////////////////////////////////////////////


/**@name La classe générique des objets opérés\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]OperatedObject[/C]} & {\tiny OPERATED_OBJECT} & {\tiny operatedObject} & {\tiny } & {\tiny }
\end{tabular}

Cette classe ne reçoit pas d'instances directes.\\

Le descripteur [C]ResourceEntityMemory[/C] est valué par le moteur de simulation au moment ou une opération débute sur un objet opéré. La valeur est une "photographie" de l'entité prise en cet instant dans le code, sous la forme d'une copie profonde de l'instance. 

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny int} & {\tiny [C]IsDestination[/C] IS_DESTINATION isDestination} & {\tiny {\em FALSE}} & {\tiny TRUE si la ressource est destinataire d'un transfert}\hline
{\tiny C} & {\tiny pEntity*} & {\tiny [C]ResourceEntity[/C] RESOURCE_ENTITY resourceEntity} & {\tiny NULL} & {\tiny l'entité qui, ainsi attachée, joue le rôle d'une ressource}\hline
{\tiny C} & {\tiny pEntity*} & {\tiny [C]ResourceEntityMemory[/C] RESOURCE_ENTITY_MEMORY resourceEntityMemory} & {\tiny NULL} & {\tiny une copie de l'entité en début d'opération}\hline
{\tiny V} & {\tiny pEntity*} & {\tiny [C]NonOperatedEntityList[/C] NON_OPERATED_ENTITY_LIST nonOperatedEntityList} & {\tiny un tableau vide} & {\tiny les entités non encore opérées par une [C]UnitGranulatedOperation[/C]}\hline
{\tiny V} & {\tiny pEntity*} & {\tiny [C]OperatedEntityList[/C] OPERATED_ENTITY_LIST OperatedEntityList} & {\tiny un tableau vide} & {\tiny les entités déjà opérées par une [C]UnitGranulatedOperation[/C]}\hline
{\tiny V} & {\tiny int} & {\tiny [C]NonOperatedIntQuantity[/C] NON_OPERATED_INT_QUANTITY nonOperatedIntQuantity} & {\tiny 0} & {\tiny la quantité entière non encore opérée par une [C]IntQuantityGranulatedOperation[/C]}\hline
{\tiny V} & {\tiny int} & {\tiny [C]OperatedIntQuantity[/C] OPERATED_INT_QUANTITY operatedIntQuantity} & {\tiny 0} & {\tiny la quantité entière déjà opérée par une [C]IntQuantityGranulatedOperation[/C]}\hline
{\tiny V} & {\tiny float} & {\tiny [C]NonOperatedFloatQuantity[/C] NON_OPERATED_FLOAT_QUANTITY nonOperatedFloatQuantity} & {\tiny 0.} & {\tiny la quantité 'flottante' non encore opérée par une [C]IntQuantityGranulatedOperation[/C]}\hline
{\tiny V} & {\tiny float} & {\tiny [C]OperatedFloatQuantity[/C] OPERATED_FLOAT_QUANTITY operatedFloatQuantity} & {\tiny 0.} & {\tiny la quantité 'flottante' déjà opérée par une [C]IntQuantityGranulatedOperation[/C]}\hline
{\tiny V} & {\tiny float} & {\tiny [C]NonOperatedProportion[/C] NON_OPERATED_PROPORTION nonOperatedProportion} & {\tiny 1.} & {\tiny le pourcentage ([0. 1.]) non encore opérée par une [C]TimeGranulatedOperation[/C]}\hline
{\tiny V} & {\tiny int} & {\tiny [C]OperatedProportion[/C] OPERATED_PROPORTION operatedProportion} & {\tiny 0} & {\tiny le pourcentage ([0. 1.]) déjà opéré par une [C]TimeGranulatedOperation[/C]}
\end{tabular}

  {\bf Méthodes :}\\
\begin{itemize}
\item [C]InvolvedEntities[/C] établit la liste des entités opérées par l'opération (elles seront opérées dans l'ordre de la liste). La valeur de l'argument [C]ClassIdArgument[/C] est le symbole de la classe dont les entités opérées doivent être des instances.
\end{itemize}

\begin{tabular}{ccccl}
{\tiny type} & {\tiny classe, symbole, nom} & {\tiny corps par défaut} & {\tiny classes des arguments}\hline
\hline
{\tiny pEntityTab*} & {\tiny [C]InvolvedEntities[/C] INVOLVED_ENTITIES involvedEntities} & {\tiny} & {\tiny [C]ClassIdArgument[/C]}
\end{tabular}
  
  {\bf Arguments des méthodes :}

\begin{tabular}{ccccl}
{\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny domaine}\hline
\hline
{\tiny UserClassId} & {\tiny [C]ClassIdArgument[/C] CLASSID_ARGUMENT, "classIdArgument} & {\tiny BD_BASIC_ENTITY} & {\tiny un symbole de classe d'entités}
\end{tabular}

*/
//bool operatedObject_availabilityPredicate_Body(EntityMethod*);
//@{
        ///
class OperatedObject : public SingleDiscreteStateReusableResource
{
public:
          // Constructeur de base.
  OperatedObject();
          // Constructeur par copie.
  OperatedObject(const OperatedObject&);
          // Destructeur.
  ~OperatedObject();

/**@name [SC]Désignation de l'objet opéré courant d'une activité[/SC] */

//@{
          /// Renvoie NULL si l'opération passée en argument a traité l'objet opéré auquel ce message est adreesé. Sinon, renvoie cet objet opéré.\index{méthodes!CurrentOperatedObject}
  OperatedObject*     CurrentOperatedObject(Operation*);

          /// Dans une activité de transfert, renvoie NULL si l'opération passée en argument a traité l'objet opéré en position de dstination auquel ce message est adreesé. Sinon, renvoie cet objet destinataire.\index{méthodes!CurrentTransferDestination}
  OperatedObject*     CurrentTransferDestination(UserDescriptorId);
//@}


/**@name [SC]Accés à l'état de la ressource en début d'opération[/SC] */

//@{
          // Etablit la valeur de l'entité-ressource en état initial. C'est une entité qui est une copie de l'entité-ressource dans l'état où elle se trouvait au début de la réalisation de l'opération.
void        SetInitialStateResourceEntity(BasicEntity*);

          /// Renvoie l'entité en position d'objet opéré d'une activité, dans l'état où elle se trouvait au début de la réalisation de l'opération, et renvoie NULL si l'opération est terminée.\index{méthodes!GetInitialStateResourceEntity}
BasicEntity*     GetInitialStateResourceEntity();
//@}

/**@name [SC]Accès aux parties opérées et non opérées de l'entité[/SC]

\index{méthodes!GetOperated...Quantity}\index{méthodes!GetNonOperated...Quantity}
\index{méthodes!GetOperatedEntities}\index{méthodes!GetNonOperatedEntities}
\index{méthodes!GetOperatedProportion}\index{méthodes!GetNonOperatedProportion} 
*/
//@{
          //
void SetSubOperatedObjects(pEntityTab*);  // utile si POP.MULT ou LOCATION
          //
pEntityTab* GetSubOperatedObjects(); // utile si POP.MULT ou LOCATION
          //    
//void SetFullyOperated(Boolean);          
          //    
//Boolean GetFullyOperated(); 
          //    
void AddFullyOperatedBy(Operation*); 
          //    
void RemoveFullyOperatedBy(Operation*);            
          //    
Boolean IsFullyOperatedBy(Operation*);           
          //             
void SetNonOperatedEntities(pEntityTab*);            
          //             
void SetNonOperatedEntities(pEntityTab*, int);  // + JPR 041207            
          //             
void AddNonOperatedEntities(pEntityTab*, int);  // + JPR 041207
          /// Dans le tableau d'entités à opérer en séquence par une activité du jeu renvoyé par [Y]SelectBestAlternative[/Y], renvoie la liste de celles pas encore traitées par l'opération ([C]UnitGranulatedOperation[/C]).\index{méthodes!GetNonOperatedEntities}
pEntityTab* GetNonOperatedEntities();    
          // Renvoie la taille du tableau renvoyé par valeur de [Y]GetNonOperatedEntities[/Y].    
int GetNonOperatedEntitiesNumber();      
          //      
void SetNonOperatedModelElemNumber(int);      
          // Renvoie  ... (à coder)    
int GetNonOperatedModelElemNumber(); 
          //    
void SetOperatedModelElemNumber(int); 
          // Renvoie  ... (à coder)     
int GetOperatedModelElemNumber(); 
          //       
void SetNonOperatedIntQuantity(int);  
          /// L'activité du jeu renvoyé par [Y]SelectBestAlternative[/Y] demande à son opération [C]IntQuantityGranulatedOperation[/C] de traiter une certaine quantité d'une certaine grandeur ; ce service renvoie la quantité non encore traitée.\index{méthodes!GetNonOperatedIntQuantity}       
int GetNonOperatedIntQuantity();     
          //       
void SetNonOperatedFloatQuantity(float);
          /// L'activité du jeu renvoyé par [Y]SelectBestAlternative[/Y] demande à son opération [C]FloatQuantityGranulatedOperation[/C] de traiter une certaine quantité d'une certaine grandeur ; ce service renvoie la quantité non encore traitée.\index{méthodes!GetNonOperatedFloatQuantity}      
float GetNonOperatedFloatQuantity();  
          //            
void SetOperatedIntQuantity(int);     
          /// L'activité du jeu renvoyé par [Y]SelectBestAlternative[/Y] demande à son opération [C]IntQuantityGranulatedOperation[/C] de traiter une certaine quantité d'une certaine grandeur ; ce service renvoie la quantité déjà traitée.\index{méthodes!GetOperatedIntQuantity}             
int GetOperatedIntQuantity();             
          //            
void SetOperatedFloatQuantity(float);      
          /// L'activité du jeu renvoyé par [Y]SelectBestAlternative[/Y] demande à son opération [C]FloatQuantityGranulatedOperation[/C] de traiter une certaine quantité d'une certaine grandeur ; ce service renvoie la quantité déjà traitée.\index{méthodes!GetOperatedFloatQuantity}              
float GetOperatedFloatQuantity();    
          //         
void SetNonOperatedProportion(float);           
          /// L'activité du jeu renvoyé par [Y]SelectBestAlternative[/Y] demande à son opération [C]TimeQuantityGranulatedOperation[/C] de traiter totalement l'ensemble des objets opérés ; ce service renvoie le pourcentage du travail non encore effectué.\index{méthodes!GetNonOperatedProportion}
float GetNonOperatedProportion(); 
          //        
void SetOperatedProportion(float);
          /// L'activité du jeu renvoyé par [Y]SelectBestAlternative[/Y] demande à son opération [C]TimeQuantityGranulatedOperation[/C] de traiter totalement l'ensemble des objets opérés ; ce service renvoie le pourcentage du travail déjà effectué.\index{méthodes!GetOperatedProportion}       
float GetOperatedProportion();
//@}

private:
/*
  Entity*     mRscEntityIntialState;
  
  pEntityTab* mSubOperatedObjects;         
  Boolean     mFullyOperated;                
  pEntityTab* mNonOperatedEntities;           
  pEntityTab* mOperatedEntities;               
  int         mNonOperatedEntitiesNumber;      
  int         mOperatedEntitiesNumber;         
  int         mNonOperatedModelElemNumber;     
  int         mOperatedModelElemNumber;        
  float       mNonOperatedQuantity;                
  float       mOperatedQuantity;                   
  float       mNonOperatedPercentage;          
  float       mOperatedPercentage; 
*/
          
};
//@}

/**@name Les objets opérés qui sont eux-mêmes des ressources\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]OperatedResource[/C]} & {\tiny OPERATED_RESOURCE} & {\tiny operatedResource} & {\tiny } & {\tiny }
\end{tabular}

Cette classe ne possède ni descripteur, ni méthode, ni service particulier.\\

Elle est identifiée pour reconnaître plus aisément et traiter le cas particulier des objets opérés qui sont aussi des instances de [C]Resource[/C].
*/
//@{
        ///
class OperatedResource : public OperatedObject
{
public:
          // Constructeur de base.
  OperatedResource();
          // Constructeur par copie.
  OperatedResource(const OperatedResource&);
          // Destructeur.
  ~OperatedResource();


private:
};

//@}  // fin OperatedResource

/**@name Les objets opérés non spéciaux\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]NonSpecialOperatedObject[/C]} & {\tiny NON_SPECIAL_OPERATED_OBJECT} & {\tiny nonSpecialOperatedObject} & {\tiny } & {\tiny }
\end{tabular}

Cette classe ne possède ni descripteur, ni méthode, ni service particulier.\\

Elle est identifiée pour reconnaître plus aisément et traiter le cas des objets opérés qui ne sont pas des instances de [C]Resource[/C], ni de [C]Location[/C], ni de [C]Population[/C].
*/
//@{
        ///
class NonSpecialOperatedObject : public OperatedObject
{
public:
          // Constructeur de base.
  NonSpecialOperatedObject();
          // Constructeur par copie.
  NonSpecialOperatedObject(const NonSpecialOperatedObject&);
          // Destructeur.
  ~NonSpecialOperatedObject();


private:
};
pEntityTab*     operatedResource_involvedEntities_Body(EntityMethod*);
pEntityTab*     nonSpecialOperatedObject_involvedEntities_Body(EntityMethod*);

//@}  // fin NonSpecialOperatedObject

//@}  // fin operatedObjects vus comme ressources

//@}  // fin  SingleRsc

// la suite de 'Les ressources' dans C_AggregatedResources.h

#endif  // _C_SINGLERESOURCE_HH
